@@ -14,12 +14,13 @@
 
 import webob
 
+from oslo import messaging
+
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api import xmlutil
 from cinder import exception
 from cinder.openstack.common import log as logging
-from cinder.openstack.common.rpc import common as rpc_common
 from cinder.openstack.common import strutils
 from cinder import utils
 from cinder import volume
@@ -76,7 +77,11 @@ class VolumeActionsController(wsgi.Controller):
     def _attach(self, req, id, body):
         """Add attachment metadata."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         # instance uuid is an option now
         instance_uuid = None
         if 'instance_uuid' in body['os-attach']:
@@ -116,7 +121,11 @@ class VolumeActionsController(wsgi.Controller):
     def _detach(self, req, id, body):
         """Clear attachment metadata."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         self.volume_api.detach(context, volume)
         return webob.Response(status_int=202)
 
@@ -124,7 +133,11 @@ class VolumeActionsController(wsgi.Controller):
     def _reserve(self, req, id, body):
         """Mark volume as reserved."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         self.volume_api.reserve_volume(context, volume)
         return webob.Response(status_int=202)
 
@@ -132,7 +145,11 @@ class VolumeActionsController(wsgi.Controller):
     def _unreserve(self, req, id, body):
         """Unmark volume as reserved."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         self.volume_api.unreserve_volume(context, volume)
         return webob.Response(status_int=202)
 
@@ -140,7 +157,11 @@ class VolumeActionsController(wsgi.Controller):
     def _begin_detaching(self, req, id, body):
         """Update volume status to 'detaching'."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         self.volume_api.begin_detaching(context, volume)
         return webob.Response(status_int=202)
 
@@ -148,7 +169,11 @@ class VolumeActionsController(wsgi.Controller):
     def _roll_detaching(self, req, id, body):
         """Roll back volume status to 'in-use'."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         self.volume_api.roll_detaching(context, volume)
         return webob.Response(status_int=202)
 
@@ -156,20 +181,41 @@ class VolumeActionsController(wsgi.Controller):
     def _initialize_connection(self, req, id, body):
         """Initialize volume attachment."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
-        connector = body['os-initialize_connection']['connector']
-        info = self.volume_api.initialize_connection(context,
-                                                     volume,
-                                                     connector)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+        try:
+            connector = body['os-initialize_connection']['connector']
+        except KeyError:
+            raise webob.exc.HTTPBadRequest("Must specify 'connector'")
+        try:
+            info = self.volume_api.initialize_connection(context,
+                                                         volume,
+                                                         connector)
+        except exception.VolumeBackendAPIException as error:
+            msg = _("Unable to fetch connection information from backend.")
+            raise webob.exc.HTTPInternalServerError(msg)
+
         return {'connection_info': info}
 
     @wsgi.action('os-terminate_connection')
     def _terminate_connection(self, req, id, body):
         """Terminate volume attachment."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
-        connector = body['os-terminate_connection']['connector']
-        self.volume_api.terminate_connection(context, volume, connector)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+        try:
+            connector = body['os-terminate_connection']['connector']
+        except KeyError:
+            raise webob.exc.HTTPBadRequest("Must specify 'connector'")
+        try:
+            self.volume_api.terminate_connection(context, volume, connector)
+        except exception.VolumeBackendAPIException as error:
+            msg = _("Unable to terminate volume connection from backend.")
+            raise webob.exc.HTTPInternalServerError(explanation=msg)
         return webob.Response(status_int=202)
 
     @wsgi.response(202)
@@ -179,21 +225,27 @@ class VolumeActionsController(wsgi.Controller):
     def _volume_upload_image(self, req, id, body):
         """Uploads the specified volume to image service."""
         context = req.environ['cinder.context']
-        try:
-            params = body['os-volume_upload_image']
-        except (TypeError, KeyError):
-            msg = _("Invalid request body")
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-
+        params = body['os-volume_upload_image']
         if not params.get("image_name"):
             msg = _("No image_name was specified in request.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         force = params.get('force', False)
+        if isinstance(force, basestring):
+            try:
+                force = strutils.bool_from_string(force, strict=False)
+            except ValueError:
+                msg = _("Bad value for 'force' parameter.")
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+        elif not isinstance(force, bool):
+            msg = _("'force' is not string or bool.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
         try:
             volume = self.volume_api.get(context, id)
         except exception.VolumeNotFound as error:
             raise webob.exc.HTTPNotFound(explanation=error.msg)
+
         authorize(context, "upload_image")
         image_metadata = {"container_format": params.get("container_format",
                                                          "bare"),
@@ -208,7 +260,7 @@ class VolumeActionsController(wsgi.Controller):
             raise webob.exc.HTTPBadRequest(explanation=error.msg)
         except ValueError as error:
             raise webob.exc.HTTPBadRequest(explanation=unicode(error))
-        except rpc_common.RemoteError as error:
+        except messaging.RemoteError as error:
             msg = "%(err_type)s: %(err_msg)s" % {'err_type': error.exc_type,
                                                  'err_msg': error.value}
             raise webob.exc.HTTPBadRequest(explanation=msg)
@@ -218,14 +270,18 @@ class VolumeActionsController(wsgi.Controller):
     def _extend(self, req, id, body):
         """Extend size of volume."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
         try:
-            _val = int(body['os-extend']['new_size'])
-        except (KeyError, ValueError):
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
+        try:
+            int(body['os-extend']['new_size'])
+        except (KeyError, ValueError, TypeError):
             msg = _("New volume size must be specified as an integer.")
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        size = body['os-extend']['new_size']
+        size = int(body['os-extend']['new_size'])
         self.volume_api.extend(context, volume, size)
         return webob.Response(status_int=202)
 
@@ -233,9 +289,17 @@ class VolumeActionsController(wsgi.Controller):
     def _volume_readonly_update(self, req, id, body):
         """Update volume readonly flag."""
         context = req.environ['cinder.context']
-        volume = self.volume_api.get(context, id)
+        try:
+            volume = self.volume_api.get(context, id)
+        except exception.VolumeNotFound as error:
+            raise webob.exc.HTTPNotFound(explanation=error.msg)
+
+        try:
+            readonly_flag = body['os-update_readonly_flag']['readonly']
+        except KeyError:
+            msg = _("Must specify readonly in request.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        readonly_flag = body['os-update_readonly_flag'].get('readonly')
         if isinstance(readonly_flag, basestring):
             try:
                 readonly_flag = strutils.bool_from_string(readonly_flag,
@@ -251,6 +315,21 @@ class VolumeActionsController(wsgi.Controller):
         self.volume_api.update_readonly_flag(context, volume, readonly_flag)
         return webob.Response(status_int=202)
 
+    @wsgi.action('os-retype')
+    def _retype(self, req, id, body):
+        """Change type of existing volume."""
+        context = req.environ['cinder.context']
+        volume = self.volume_api.get(context, id)
+        try:
+            new_type = body['os-retype']['new_type']
+        except KeyError:
+            msg = _("New volume type must be specified.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        policy = body['os-retype'].get('migration_policy')
+
+        self.volume_api.retype(context, volume, new_type, policy)
+        return webob.Response(status_int=202)
+
 
 class Volume_actions(extensions.ExtensionDescriptor):
     """Enable volume actions
