@@ -1,4 +1,4 @@
-#   Copyright 2012 OpenStack, LLC.
+#   Copyright 2012 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -29,21 +29,18 @@ class VolumeHostAttributeController(wsgi.Controller):
         super(VolumeHostAttributeController, self).__init__(*args, **kwargs)
         self.volume_api = volume.API()
 
-    def _add_volume_host_attribute(self, context, resp_volume):
-        try:
-            db_volume = self.volume_api.get(context, resp_volume['id'])
-        except Exception:
-            return
-        else:
-            key = "%s:host" % Volume_host_attribute.alias
-            resp_volume[key] = db_volume['host']
+    def _add_volume_host_attribute(self, context, req, resp_volume):
+        db_volume = req.cached_resource_by_id(resp_volume['id'])
+        key = "%s:host" % Volume_host_attribute.alias
+        resp_volume[key] = db_volume['host']
 
     @wsgi.extends
     def show(self, req, resp_obj, id):
         context = req.environ['cinder.context']
         if authorize(context):
             resp_obj.attach(xml=VolumeHostAttributeTemplate())
-            self._add_volume_host_attribute(context, resp_obj.obj['volume'])
+            volume = resp_obj.obj['volume']
+            self._add_volume_host_attribute(context, req, volume)
 
     @wsgi.extends
     def detail(self, req, resp_obj):
@@ -51,7 +48,7 @@ class VolumeHostAttributeController(wsgi.Controller):
         if authorize(context):
             resp_obj.attach(xml=VolumeListHostAttributeTemplate())
             for volume in list(resp_obj.obj['volumes']):
-                self._add_volume_host_attribute(context, volume)
+                self._add_volume_host_attribute(context, req, volume)
 
 
 class Volume_host_attribute(extensions.ExtensionDescriptor):
