@@ -1,4 +1,4 @@
-# Copyright (c) 2013 OpenStack, LLC.
+# Copyright (c) 2013 OpenStack Foundation
 #
 # All Rights Reserved.
 #
@@ -20,10 +20,8 @@ WSGI middleware for OpenStack API controllers.
 
 import routes
 
-from cinder.api.middleware import fault
 from cinder.api.openstack import wsgi
 from cinder.openstack.common import log as logging
-from cinder import utils
 from cinder import wsgi as base_wsgi
 
 
@@ -55,15 +53,12 @@ class ProjectMapper(APIMapper):
 
 
 class APIRouter(base_wsgi.Router):
-    """
-    Routes requests on the OpenStack API to the appropriate controller
-    and method.
-    """
+    """Routes requests on the API to the appropriate controller and method."""
     ExtensionManager = None  # override in subclasses
 
     @classmethod
     def factory(cls, global_config, **local_config):
-        """Simple paste factory, :class:`cinder.wsgi.Router` doesn't have"""
+        """Simple paste factory, :class:`cinder.wsgi.Router` doesn't have."""
         return cls()
 
     def __init__(self, ext_mgr=None):
@@ -102,18 +97,20 @@ class APIRouter(base_wsgi.Router):
 
     def _setup_extensions(self, ext_mgr):
         for extension in ext_mgr.get_controller_extensions():
-            ext_name = extension.extension.name
             collection = extension.collection
             controller = extension.controller
 
             if collection not in self.resources:
                 LOG.warning(_('Extension %(ext_name)s: Cannot extend '
-                              'resource %(collection)s: No such resource') %
-                            locals())
+                              'resource %(collection)s: No such resource'),
+                            {'ext_name': extension.extension.name,
+                             'collection': collection})
                 continue
 
             LOG.debug(_('Extension %(ext_name)s extending resource: '
-                        '%(collection)s') % locals())
+                        '%(collection)s'),
+                      {'ext_name': extension.extension.name,
+                       'collection': collection})
 
             resource = self.resources[collection]
             resource.register_actions(controller)
@@ -123,8 +120,11 @@ class APIRouter(base_wsgi.Router):
         raise NotImplementedError
 
 
-class FaultWrapper(fault.FaultWrapper):
+class FaultWrapper(base_wsgi.Middleware):
+
     def __init__(self, application):
         LOG.warn(_('cinder.api.openstack:FaultWrapper is deprecated. Please '
                    'use cinder.api.middleware.fault:FaultWrapper instead.'))
-        super(FaultWrapper, self).__init__(application)
+        # Avoid circular imports from here. Can I just remove this class?
+        from cinder.api.middleware import fault
+        super(FaultWrapper, self).__init__(fault.FaultWrapper(application))
