@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -24,8 +22,8 @@ from cinder import volume
 from webob import exc
 
 
-class Controller(object):
-    """ The volume metadata API controller for the OpenStack API """
+class Controller(wsgi.Controller):
+    """The volume metadata API controller for the OpenStack API."""
 
     def __init__(self):
         self.volume_api = volume.API()
@@ -42,7 +40,7 @@ class Controller(object):
 
     @wsgi.serializers(xml=common.MetadataTemplate)
     def index(self, req, volume_id):
-        """ Returns the list of metadata for a given volume"""
+        """Returns the list of metadata for a given volume."""
         context = req.environ['cinder.context']
         return {'metadata': self._get_metadata(context, volume_id)}
 
@@ -124,14 +122,14 @@ class Controller(object):
             raise exc.HTTPBadRequest(explanation=msg)
 
         except exception.InvalidVolumeMetadata as error:
-            raise exc.HTTPBadRequest(explanation=unicode(error))
+            raise exc.HTTPBadRequest(explanation=error.msg)
 
         except exception.InvalidVolumeMetadataSize as error:
-            raise exc.HTTPRequestEntityTooLarge(explanation=unicode(error))
+            raise exc.HTTPRequestEntityTooLarge(explanation=error.msg)
 
     @wsgi.serializers(xml=common.MetaItemTemplate)
     def show(self, req, volume_id, id):
-        """ Return a single metadata item """
+        """Return a single metadata item."""
         context = req.environ['cinder.context']
         data = self._get_metadata(context, volume_id)
 
@@ -142,7 +140,7 @@ class Controller(object):
             raise exc.HTTPNotFound(explanation=msg)
 
     def delete(self, req, volume_id, id):
-        """ Deletes an existing metadata """
+        """Deletes an existing metadata."""
         context = req.environ['cinder.context']
 
         metadata = self._get_metadata(context, volume_id)
