@@ -27,6 +27,7 @@ from cinder.openstack.common import log as logging
 from cinder.openstack.common import uuidutils
 from cinder import utils
 from cinder import volume as cinder_volume
+from cinder.volume import utils as volume_utils
 from cinder.volume import volume_types
 
 
@@ -105,6 +106,8 @@ def _translate_volume_summary_view(context, vol, image_id=None):
     d['snapshot_id'] = vol['snapshot_id']
     d['source_volid'] = vol['source_volid']
 
+    d['encrypted'] = vol['encryption_key_id'] is not None
+
     if image_id:
         d['image_id'] = image_id
 
@@ -381,15 +384,25 @@ class VolumeController(wsgi.Controller):
 
         snapshot_id = volume.get('snapshot_id')
         if snapshot_id is not None:
-            kwargs['snapshot'] = self.volume_api.get_snapshot(context,
-                                                              snapshot_id)
+            try:
+                kwargs['snapshot'] = self.volume_api.get_snapshot(context,
+                                                                  snapshot_id)
+            except exception.NotFound:
+                explanation = _('snapshot id:%s not found') % snapshot_id
+                raise exc.HTTPNotFound(explanation=explanation)
+
         else:
             kwargs['snapshot'] = None
 
         source_volid = volume.get('source_volid')
         if source_volid is not None:
-            kwargs['source_volume'] = self.volume_api.get_volume(context,
-                                                                 source_volid)
+            try:
+                kwargs['source_volume'] = \
+                    self.volume_api.get_volume(context,
+                                               source_volid)
+            except exception.NotFound:
+                explanation = _('source vol id:%s not found') % source_volid
+                raise exc.HTTPNotFound(explanation=explanation)
         else:
             kwargs['source_volume'] = None
 
@@ -459,6 +472,8 @@ class VolumeController(wsgi.Controller):
 
         try:
             volume = self.volume_api.get(context, id)
+            volume_utils.notify_about_volume_usage(context, volume,
+                                                   'update.start')
             self.volume_api.update(context, volume, update_dict)
         except exception.NotFound:
             raise exc.HTTPNotFound()
@@ -467,6 +482,9 @@ class VolumeController(wsgi.Controller):
 
         self._add_visible_admin_metadata(context, volume)
 
+        volume_utils.notify_about_volume_usage(context, volume,
+                                               'update.end')
+
         return {'volume': _translate_volume_detail_view(context, volume)}
 
 
