@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2012 OpenStack LLC.
+# Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -66,12 +64,19 @@ class ViewBuilder(common.ViewBuilder):
                 'snapshot_id': volume.get('snapshot_id'),
                 'source_volid': volume.get('source_volid'),
                 'metadata': self._get_volume_metadata(volume),
-                'links': self._get_links(request, volume['id'])
+                'links': self._get_links(request, volume['id']),
+                'user_id': volume.get('user_id'),
+                'bootable': str(volume.get('bootable')).lower(),
+                'encrypted': self._is_volume_encrypted(volume)
             }
         }
 
+    def _is_volume_encrypted(self, volume):
+        """Determine if volume is encrypted."""
+        return volume.get('encryption_key_id') is not None
+
     def _get_attachments(self, volume):
-        """Retrieves the attachments of the volume object"""
+        """Retrieve the attachments of the volume object."""
         attachments = []
 
         if volume['attach_status'] == 'attached':
@@ -84,6 +89,7 @@ class ViewBuilder(common.ViewBuilder):
 
             d['volume_id'] = volume_id
             d['server_id'] = volume['instance_uuid']
+            d['host_name'] = volume['attached_host']
             if volume.get('mountpoint'):
                 d['device'] = volume['mountpoint']
             attachments.append(d)
@@ -91,7 +97,7 @@ class ViewBuilder(common.ViewBuilder):
         return attachments
 
     def _get_volume_metadata(self, volume):
-        """Retrieves the metadata of the volume object"""
+        """Retrieve the metadata of the volume object."""
         if volume.get('volume_metadata'):
             metadata = volume.get('volume_metadata')
             return dict((item['key'], item['value']) for item in metadata)
@@ -102,7 +108,7 @@ class ViewBuilder(common.ViewBuilder):
         return {}
 
     def _get_volume_type(self, volume):
-        """Retrieves the type the volume object is"""
+        """Retrieve the type the volume object."""
         if volume['volume_type_id'] and volume.get('volume_type'):
             return volume['volume_type']['name']
         else:
