@@ -14,7 +14,10 @@
 #    under the License.
 
 from cinder.api import common
-from cinder.api import microversions as mv
+from cinder.openstack.common import log as logging
+
+
+LOG = logging.getLogger(__name__)
 
 
 class ViewBuilder(common.ViewBuilder):
@@ -26,15 +29,13 @@ class ViewBuilder(common.ViewBuilder):
         """Initialize view builder."""
         super(ViewBuilder, self).__init__()
 
-    def summary_list(self, request, transfers, origin_transfer_count):
+    def summary_list(self, request, transfers):
         """Show a list of transfers without many details."""
-        return self._list_view(self.summary, request, transfers,
-                               origin_transfer_count)
+        return self._list_view(self.summary, request, transfers)
 
-    def detail_list(self, request, transfers, origin_transfer_count):
+    def detail_list(self, request, transfers):
         """Detailed view of a list of transfers ."""
-        return self._list_view(self.detail, request, transfers,
-                               origin_transfer_count)
+        return self._list_view(self.detail, request, transfers)
 
     def summary(self, request, transfer):
         """Generic, non-detailed view of a transfer."""
@@ -50,7 +51,7 @@ class ViewBuilder(common.ViewBuilder):
 
     def detail(self, request, transfer):
         """Detailed view of a single transfer."""
-        detail_body = {
+        return {
             'transfer': {
                 'id': transfer.get('id'),
                 'created_at': transfer.get('created_at'),
@@ -59,22 +60,10 @@ class ViewBuilder(common.ViewBuilder):
                 'links': self._get_links(request, transfer['id'])
             }
         }
-        req_version = request.api_version_request
-        if req_version.matches(mv.TRANSFER_WITH_SNAPSHOTS):
-            detail_body['transfer'].update({'no_snapshots':
-                                            transfer.get('no_snapshots')})
-        if req_version.matches(mv.TRANSFER_WITH_HISTORY):
-            transfer_history = {
-                'destination_project_id': transfer['destination_project_id'],
-                'source_project_id': transfer['source_project_id'],
-                'accepted': transfer['accepted']
-            }
-            detail_body['transfer'].update(transfer_history)
-        return detail_body
 
     def create(self, request, transfer):
         """Detailed view of a single transfer when created."""
-        create_body = {
+        return {
             'transfer': {
                 'id': transfer.get('id'),
                 'created_at': transfer.get('created_at'),
@@ -84,27 +73,14 @@ class ViewBuilder(common.ViewBuilder):
                 'links': self._get_links(request, transfer['id'])
             }
         }
-        req_version = request.api_version_request
-        if req_version.matches(mv.TRANSFER_WITH_SNAPSHOTS):
-            create_body['transfer'].update({'no_snapshots':
-                                            transfer.get('no_snapshots')})
-        if req_version.matches(mv.TRANSFER_WITH_HISTORY):
-            transfer_history = {
-                'destination_project_id': transfer['destination_project_id'],
-                'source_project_id': transfer['source_project_id'],
-                'accepted': transfer['accepted']
-            }
-            create_body['transfer'].update(transfer_history)
-        return create_body
 
-    def _list_view(self, func, request, transfers, origin_transfer_count):
+    def _list_view(self, func, request, transfers):
         """Provide a view for a list of transfers."""
         transfers_list = [func(request, transfer)['transfer'] for transfer in
                           transfers]
         transfers_links = self._get_collection_links(request,
                                                      transfers,
-                                                     self._collection_name,
-                                                     origin_transfer_count)
+                                                     self._collection_name)
         transfers_dict = dict(transfers=transfers_list)
 
         if transfers_links:
