@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -91,6 +89,13 @@ class InitiatorConnector(executor.Executor):
                                   use_multipath=use_multipath,
                                   device_scan_attempts=device_scan_attempts,
                                   *args, **kwargs)
+        elif protocol == "ISER":
+            return ISERConnector(root_helper=root_helper,
+                                 driver=driver,
+                                 execute=execute,
+                                 use_multipath=use_multipath,
+                                 device_scan_attempts=device_scan_attempts,
+                                 *args, **kwargs)
         elif protocol == "FIBRE_CHANNEL":
             return FibreChannelConnector(root_helper=root_helper,
                                          driver=driver,
@@ -444,7 +449,7 @@ class ISCSIConnector(InitiatorConnector):
                                    check_exit_code=[0, 255])
             except putils.ProcessExecutionError as err:
                 #as this might be one of many paths,
-                #only set successfull logins to startup automatically
+                #only set successful logins to startup automatically
                 if err.exit_code in [15]:
                     self._iscsiadm_update(connection_properties,
                                           "node.startup",
@@ -532,6 +537,15 @@ class ISCSIConnector(InitiatorConnector):
         self._run_multipath('-r', check_exit_code=[0, 1, 21])
 
 
+class ISERConnector(ISCSIConnector):
+
+    def _get_device_path(self, iser_properties):
+        return ("/dev/disk/by-path/ip-%s-iser-%s-lun-%s" %
+                (iser_properties['target_portal'],
+                 iser_properties['target_iqn'],
+                 iser_properties.get('target_lun', 0)))
+
+
 class FibreChannelConnector(InitiatorConnector):
     """Connector class to attach/detach Fibre Channel volumes."""
 
