@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
@@ -45,7 +43,6 @@ these objects be simple dictionaries.
 
 from oslo.config import cfg
 
-from cinder import exception
 from cinder.openstack.common.db import api as db_api
 
 
@@ -79,11 +76,6 @@ _BACKEND_MAPPING = {'sqlalchemy': 'cinder.db.sqlalchemy.api'}
 IMPL = db_api.DBAPI(backend_mapping=_BACKEND_MAPPING)
 
 
-class NoMoreTargets(exception.CinderException):
-    """No more available targets"""
-    pass
-
-
 ###################
 
 
@@ -213,9 +205,11 @@ def volume_get(context, volume_id):
     return IMPL.volume_get(context, volume_id)
 
 
-def volume_get_all(context, marker, limit, sort_key, sort_dir):
+def volume_get_all(context, marker, limit, sort_key, sort_dir,
+                   filters=None):
     """Get all volumes."""
-    return IMPL.volume_get_all(context, marker, limit, sort_key, sort_dir)
+    return IMPL.volume_get_all(context, marker, limit, sort_key, sort_dir,
+                               filters=filters)
 
 
 def volume_get_all_by_host(context, host):
@@ -229,10 +223,10 @@ def volume_get_all_by_instance_uuid(context, instance_uuid):
 
 
 def volume_get_all_by_project(context, project_id, marker, limit, sort_key,
-                              sort_dir):
+                              sort_dir, filters=None):
     """Get all volumes belonging to a project."""
     return IMPL.volume_get_all_by_project(context, project_id, marker, limit,
-                                          sort_key, sort_dir)
+                                          sort_key, sort_dir, filters=filters)
 
 
 def volume_get_iscsi_target_num(context, volume_id):
@@ -316,12 +310,13 @@ def snapshot_metadata_get(context, snapshot_id):
 
 def snapshot_metadata_delete(context, snapshot_id, key):
     """Delete the given metadata item."""
-    IMPL.snapshot_metadata_delete(context, snapshot_id, key)
+    return IMPL.snapshot_metadata_delete(context, snapshot_id, key)
 
 
 def snapshot_metadata_update(context, snapshot_id, metadata, delete):
     """Update metadata if it exists, otherwise create it."""
-    IMPL.snapshot_metadata_update(context, snapshot_id, metadata, delete)
+    return IMPL.snapshot_metadata_update(context, snapshot_id,
+                                         metadata, delete)
 
 
 ####################
@@ -334,12 +329,12 @@ def volume_metadata_get(context, volume_id):
 
 def volume_metadata_delete(context, volume_id, key):
     """Delete the given metadata item."""
-    IMPL.volume_metadata_delete(context, volume_id, key)
+    return IMPL.volume_metadata_delete(context, volume_id, key)
 
 
 def volume_metadata_update(context, volume_id, metadata, delete):
     """Update metadata if it exists, otherwise create it."""
-    IMPL.volume_metadata_update(context, volume_id, metadata, delete)
+    return IMPL.volume_metadata_update(context, volume_id, metadata, delete)
 
 
 ##################
@@ -352,12 +347,13 @@ def volume_admin_metadata_get(context, volume_id):
 
 def volume_admin_metadata_delete(context, volume_id, key):
     """Delete the given metadata item."""
-    IMPL.volume_admin_metadata_delete(context, volume_id, key)
+    return IMPL.volume_admin_metadata_delete(context, volume_id, key)
 
 
 def volume_admin_metadata_update(context, volume_id, metadata, delete):
     """Update metadata if it exists, otherwise create it."""
-    IMPL.volume_admin_metadata_update(context, volume_id, metadata, delete)
+    return IMPL.volume_admin_metadata_update(context, volume_id, metadata,
+                                             delete)
 
 
 ##################
@@ -434,7 +430,7 @@ def volume_type_extra_specs_get(context, volume_type_id):
 
 def volume_type_extra_specs_delete(context, volume_type_id, key):
     """Delete the given extra specs item."""
-    IMPL.volume_type_extra_specs_delete(context, volume_type_id, key)
+    return IMPL.volume_type_extra_specs_delete(context, volume_type_id, key)
 
 
 def volume_type_extra_specs_update_or_create(context,
@@ -443,9 +439,9 @@ def volume_type_extra_specs_update_or_create(context,
     """Create or update volume type extra specs. This adds or modifies the
     key/value pairs specified in the extra specs dict argument
     """
-    IMPL.volume_type_extra_specs_update_or_create(context,
-                                                  volume_type_id,
-                                                  extra_specs)
+    return IMPL.volume_type_extra_specs_update_or_create(context,
+                                                         volume_type_id,
+                                                         extra_specs)
 
 
 ###################
@@ -459,12 +455,14 @@ def volume_type_encryption_delete(context, volume_type_id):
     return IMPL.volume_type_encryption_delete(context, volume_type_id)
 
 
-# TODO(joel-coffman): split into two functions -- update and create
-def volume_type_encryption_update_or_create(context, volume_type_id,
-                                            encryption_specs):
-    return IMPL.volume_type_encryption_update_or_create(context,
-                                                        volume_type_id,
-                                                        encryption_specs)
+def volume_type_encryption_create(context, volume_type_id, encryption_specs):
+    return IMPL.volume_type_encryption_create(context, volume_type_id,
+                                              encryption_specs)
+
+
+def volume_type_encryption_update(context, volume_type_id, encryption_specs):
+    return IMPL.volume_type_encryption_update(context, volume_type_id,
+                                              encryption_specs)
 
 
 def volume_type_encryption_volume_get(context, volume_type_id, session=None):
@@ -521,12 +519,12 @@ def qos_specs_disassociate_all(context, qos_specs_id):
 
 def qos_specs_delete(context, qos_specs_id):
     """Delete the qos_specs."""
-    IMPL.qos_specs_delete(context, qos_specs_id)
+    return IMPL.qos_specs_delete(context, qos_specs_id)
 
 
 def qos_specs_item_delete(context, qos_specs_id, key):
     """Delete specified key in the qos_specs."""
-    IMPL.qos_specs_item_delete(context, qos_specs_id, key)
+    return IMPL.qos_specs_item_delete(context, qos_specs_id, key)
 
 
 def qos_specs_update(context, qos_specs_id, specs):
@@ -535,7 +533,7 @@ def qos_specs_update(context, qos_specs_id, specs):
     This adds or modifies the key/value pairs specified in the
     specs dict argument for a given qos_specs.
     """
-    IMPL.qos_specs_update(context, qos_specs_id, specs)
+    return IMPL.qos_specs_update(context, qos_specs_id, specs)
 
 
 ###################
@@ -565,20 +563,21 @@ def volume_snapshot_glance_metadata_get(context, snapshot_id):
 
 
 def volume_glance_metadata_copy_to_snapshot(context, snapshot_id, volume_id):
-    """
-    Update the Glance metadata for a snapshot by copying all of the key:value
-    pairs from the originating volume. This is so that a volume created from
-    the snapshot will retain the original metadata.
+    """Update the Glance metadata for a snapshot.
+
+    This will copy all of the key:value pairs from the originating volume,
+    to ensure that a volume created from the snapshot will retain the
+    original metadata.
     """
     return IMPL.volume_glance_metadata_copy_to_snapshot(context, snapshot_id,
                                                         volume_id)
 
 
 def volume_glance_metadata_copy_to_volume(context, volume_id, snapshot_id):
-    """
-    Update the Glance metadata from a volume (created from a snapshot) by
-    copying all of the key:value pairs from the originating snapshot. This is
-    so that the Glance metadata from the original volume is retained.
+    """Update the Glance metadata from a volume (created from a snapshot).
+
+    This will copy all of the key:value pairs from the originating snapshot,
+    to ensure that the Glance metadata from the original volume is retained.
     """
     return IMPL.volume_glance_metadata_copy_to_volume(context, volume_id,
                                                       snapshot_id)
@@ -597,10 +596,11 @@ def volume_glance_metadata_delete_by_snapshot(context, snapshot_id):
 def volume_glance_metadata_copy_from_volume_to_volume(context,
                                                       src_volume_id,
                                                       volume_id):
-    """
-    Update the Glance metadata for a volume by copying all of the key:value
-    pairs from the originating volume. This is so that a volume created from
-    the volume (clone) will retain the original metadata.
+    """Update the Glance metadata for a volume by copying all of the key:value
+    pairs from the originating volume.
+
+    This is so that a volume created from the volume (clone) will retain the
+    original metadata.
     """
     return IMPL.volume_glance_metadata_copy_from_volume_to_volume(
         context,
@@ -677,13 +677,6 @@ def quota_class_destroy_all_by_name(context, class_name):
 ###################
 
 
-def quota_usage_create(context, project_id, resource, in_use, reserved,
-                       until_refresh):
-    """Create a quota usage for the given project and resource."""
-    return IMPL.quota_usage_create(context, project_id, resource,
-                                   in_use, reserved, until_refresh)
-
-
 def quota_usage_get(context, project_id, resource):
     """Retrieve a quota usage or raise if it does not exist."""
     return IMPL.quota_usage_get(context, project_id, resource)
@@ -780,8 +773,7 @@ def backup_get_all_by_project(context, project_id):
 
 
 def backup_update(context, backup_id, values):
-    """
-    Set the given properties on a backup and update it.
+    """Set the given properties on a backup and update it.
 
     Raises NotFound if backup does not exist.
     """
