@@ -1,8 +1,7 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
+# Copyright 2014 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -28,11 +27,11 @@ from oslo.config import cfg
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy import or_
 from sqlalchemy.orm import joinedload, joinedload_all
+from sqlalchemy.orm import RelationshipProperty
 from sqlalchemy.sql.expression import literal_column
 from sqlalchemy.sql import func
 
 from cinder.common import sqlalchemyutils
-from cinder import db
 from cinder.db.sqlalchemy import models
 from cinder import exception
 from cinder.openstack.common.db import exception as db_exc
@@ -147,7 +146,7 @@ def require_volume_exists(f):
     """
 
     def wrapper(context, volume_id, *args, **kwargs):
-        db.volume_get(context, volume_id)
+        volume_get(context, volume_id)
         return f(context, volume_id, *args, **kwargs)
     wrapper.__name__ = f.__name__
     return wrapper
@@ -161,7 +160,7 @@ def require_snapshot_exists(f):
     """
 
     def wrapper(context, snapshot_id, *args, **kwargs):
-        db.api.snapshot_get(context, snapshot_id)
+        snapshot_get(context, snapshot_id)
         return f(context, snapshot_id, *args, **kwargs)
     wrapper.__name__ = f.__name__
     return wrapper
@@ -198,48 +197,6 @@ def model_query(context, *args, **kwargs):
     return query
 
 
-def exact_filter(query, model, filters, legal_keys):
-    """Applies exact match filtering to a query.
-
-    Returns the updated query.  Modifies filters argument to remove
-    filters consumed.
-
-    :param query: query to apply filters to
-    :param model: model object the query applies to, for IN-style
-                  filtering
-    :param filters: dictionary of filters; values that are lists,
-                    tuples, sets, or frozensets cause an 'IN' test to
-                    be performed, while exact matching ('==' operator)
-                    is used for other values
-    :param legal_keys: list of keys to apply exact filtering to
-    """
-
-    filter_dict = {}
-
-    # Walk through all the keys
-    for key in legal_keys:
-        # Skip ones we're not filtering on
-        if key not in filters:
-            continue
-
-        # OK, filtering on this key; what value do we search for?
-        value = filters.pop(key)
-
-        if isinstance(value, (list, tuple, set, frozenset)):
-            # Looking for values in a list; apply to query directly
-            column_attr = getattr(model, key)
-            query = query.filter(column_attr.in_(value))
-        else:
-            # OK, simple exact match; save for later
-            filter_dict[key] = value
-
-    # Apply simple exact matches
-    if filter_dict:
-        query = query.filter_by(**filter_dict)
-
-    return query
-
-
 def _sync_volumes(context, project_id, session, volume_type_id=None,
                   volume_type_name=None):
     (volumes, gigs) = _volume_data_get_for_project(
@@ -670,12 +627,6 @@ def _quota_usage_create(context, project_id, resource, in_use, reserved,
     return quota_usage_ref
 
 
-@require_admin_context
-def quota_usage_create(context, project_id, resource, in_use, reserved,
-                       until_refresh):
-    return _quota_usage_create(context, project_id, resource, in_use, reserved,
-                               until_refresh)
-
 ###################
 
 
@@ -1014,7 +965,7 @@ def volume_allocate_iscsi_target(context, volume_id, host):
         # NOTE(vish): if with_lockmode isn't supported, as in sqlite,
         #             then this has concurrency issues
         if not iscsi_target_ref:
-            raise db.NoMoreTargets()
+            raise exception.NoMoreTargets()
 
         iscsi_target_ref.volume_id = volume_id
         session.add(iscsi_target_ref)
@@ -1036,6 +987,7 @@ def volume_attached(context, volume_id, instance_uuid, host_name, mountpoint):
         volume_ref['instance_uuid'] = instance_uuid
         volume_ref['attached_host'] = host_name
         volume_ref.save(session=session)
+        return volume_ref
 
 
 @require_context
@@ -1128,21 +1080,21 @@ def volume_destroy(context, volume_id):
     session = get_session()
     now = timeutils.utcnow()
     with session.begin():
-        session.query(models.Volume).\
+        model_query(context, models.Volume, session=session).\
             filter_by(id=volume_id).\
             update({'status': 'deleted',
                     'deleted': True,
                     'deleted_at': now,
                     'updated_at': literal_column('updated_at')})
-        session.query(models.IscsiTarget).\
+        model_query(context, models.IscsiTarget, session=session).\
             filter_by(volume_id=volume_id).\
             update({'volume_id': None})
-        session.query(models.VolumeMetadata).\
+        model_query(context, models.VolumeMetadata, session=session).\
             filter_by(volume_id=volume_id).\
             update({'deleted': True,
                     'deleted_at': now,
                     'updated_at': literal_column('updated_at')})
-        session.query(models.VolumeAdminMetadata).\
+        model_query(context, models.VolumeAdminMetadata, session=session).\
             filter_by(volume_id=volume_id).\
             update({'deleted': True,
                     'deleted_at': now,
@@ -1203,20 +1155,30 @@ def volume_get(context, volume_id):
 
 
 @require_admin_context
-def volume_get_all(context, marker, limit, sort_key, sort_dir):
+def volume_get_all(context, marker, limit, sort_key, sort_dir,
+                   filters=None):
+    """Retrieves all volumes.
+
+    :param context: context to query under
+    :param marker: the last item of the previous page, used to determine the
+                   next page of results to return
+    :param limit: maximum number of items to return
+    :param sort_key: single attributes by which results should be sorted
+    :param sort_dir: direction in which results should be sorted (asc, desc)
+    :param filters: Filters for the query. A filter key/value of
+                    'no_migration_targets'=True causes volumes with either
+                    a NULL 'migration_status' or a 'migration_status' that
+                    does not start with 'target:' to be retrieved.
+    :returns: list of matching volumes
+    """
     session = get_session()
     with session.begin():
-        query = _volume_get_query(context, session=session)
-
-        marker_volume = None
-        if marker is not None:
-            marker_volume = _volume_get(context, marker, session=session)
-
-        query = sqlalchemyutils.paginate_query(query, models.Volume, limit,
-                                               [sort_key, 'created_at', 'id'],
-                                               marker=marker_volume,
-                                               sort_dir=sort_dir)
-
+        # Generate the query
+        query = _generate_paginate_query(context, session, marker, limit,
+                                         sort_key, sort_dir, filters)
+        # No volumes would match, return empty list
+        if query == None:
+            return []
         return query.all()
 
 
@@ -1242,25 +1204,136 @@ def volume_get_all_by_instance_uuid(context, instance_uuid):
 
 @require_context
 def volume_get_all_by_project(context, project_id, marker, limit, sort_key,
-                              sort_dir):
+                              sort_dir, filters=None):
+    """"Retrieves all volumes in a project.
+
+    :param context: context to query under
+    :param project_id: project for all volumes being retrieved
+    :param marker: the last item of the previous page, used to determine the
+                   next page of results to return
+    :param limit: maximum number of items to return
+    :param sort_key: single attributes by which results should be sorted
+    :param sort_dir: direction in which results should be sorted (asc, desc)
+    :param filters: Filters for the query. A filter key/value of
+                    'no_migration_targets'=True causes volumes with either
+                    a NULL 'migration_status' or a 'migration_status' that
+                    does not start with 'target:' to be retrieved.
+    :returns: list of matching volumes
+    """
     session = get_session()
     with session.begin():
         authorize_project_context(context, project_id)
-        query = _volume_get_query(context, session).\
-            filter_by(project_id=project_id)
-
-        marker_volume = None
-        if marker is not None:
-            marker_volume = _volume_get(context, marker, session)
-
-        query = sqlalchemyutils.paginate_query(query, models.Volume, limit,
-                                               [sort_key, 'created_at', 'id'],
-                                               marker=marker_volume,
-                                               sort_dir=sort_dir)
-
+        # Add in the project filter without modifying the given filters
+        filters = filters.copy() if filters else {}
+        filters['project_id'] = project_id
+        # Generate the query
+        query = _generate_paginate_query(context, session, marker, limit,
+                                         sort_key, sort_dir, filters)
+        # No volumes would match, return empty list
+        if query == None:
+            return []
         return query.all()
 
 
+def _generate_paginate_query(context, session, marker, limit, sort_key,
+                             sort_dir, filters):
+    """Generate the query to include the filters and the paginate options.
+
+    Returns a query with sorting / pagination criteria added or None
+    if the given filters will not yield any results.
+
+    :param context: context to query under
+    :param session: the session to use
+    :param marker: the last item of the previous page; we returns the next
+                    results after this value.
+    :param limit: maximum number of items to return
+    :param sort_key: single attributes by which results should be sorted
+    :param sort_dir: direction in which results should be sorted (asc, desc)
+    :param filters: dictionary of filters; values that are lists,
+                    tuples, sets, or frozensets cause an 'IN' test to
+                    be performed, while exact matching ('==' operator)
+                    is used for other values
+    :returns: updated query or None
+    """
+    query = _volume_get_query(context, session=session)
+
+    if filters:
+        filters = filters.copy()
+
+        # 'no_migration_targets' is unique, must be either NULL or
+        # not start with 'target:'
+        if ('no_migration_targets' in filters and
+                filters['no_migration_targets'] == True):
+            filters.pop('no_migration_targets')
+            try:
+                column_attr = getattr(models.Volume, 'migration_status')
+                conditions = [column_attr == None,
+                              column_attr.op('NOT LIKE')('target:%')]
+                query = query.filter(or_(*conditions))
+            except AttributeError:
+                log_msg = _("'migration_status' column could not be found.")
+                LOG.debug(log_msg)
+                return None
+
+        # Apply exact match filters for everything else, ensure that the
+        # filter value exists on the model
+        for key in filters.keys():
+            # metadata is unique, must be a dict
+            if key == 'metadata':
+                if not isinstance(filters[key], dict):
+                    log_msg = _("'metadata' filter value is not valid.")
+                    LOG.debug(log_msg)
+                    return None
+                continue
+            try:
+                column_attr = getattr(models.Volume, key)
+                # Do not allow relationship properties since those require
+                # schema specific knowledge
+                prop = getattr(column_attr, 'property')
+                if isinstance(prop, RelationshipProperty):
+                    log_msg = (_("'%s' filter key is not valid, "
+                                 "it maps to a relationship.")) % key
+                    LOG.debug(log_msg)
+                    return None
+            except AttributeError:
+                log_msg = _("'%s' filter key is not valid.") % key
+                LOG.debug(log_msg)
+                return None
+
+        # Holds the simple exact matches
+        filter_dict = {}
+
+        # Iterate over all filters, special case the filter is necessary
+        for key, value in filters.iteritems():
+            if key == 'metadata':
+                # model.VolumeMetadata defines the backref to Volumes as
+                # 'volume_metadata', use that column attribute key
+                key = 'volume_metadata'
+                column_attr = getattr(models.Volume, key)
+                for k, v in value.iteritems():
+                    query = query.filter(column_attr.any(key=k, value=v))
+            elif isinstance(value, (list, tuple, set, frozenset)):
+                # Looking for values in a list; apply to query directly
+                column_attr = getattr(models.Volume, key)
+                query = query.filter(column_attr.in_(value))
+            else:
+                # OK, simple exact match; save for later
+                filter_dict[key] = value
+
+        # Apply simple exact matches
+        if filter_dict:
+            query = query.filter_by(**filter_dict)
+
+    marker_volume = None
+    if marker is not None:
+        marker_volume = _volume_get(context, marker, session)
+
+    return sqlalchemyutils.paginate_query(query, models.Volume, limit,
+                                          [sort_key, 'created_at', 'id'],
+                                          marker=marker_volume,
+                                          sort_dir=sort_dir)
+
+
 @require_admin_context
 def volume_get_iscsi_target_num(context, volume_id):
     result = model_query(context, models.IscsiTarget, read_deleted="yes").\
@@ -1368,7 +1441,7 @@ def _volume_x_metadata_update(context, volume_id, metadata, delete,
             meta_ref.update(item)
             meta_ref.save(session=session)
 
-        return metadata
+    return _volume_x_metadata_get(context, volume_id, model)
 
 
 def _volume_user_metadata_get_query(context, volume_id, session=None):
@@ -1500,13 +1573,13 @@ def snapshot_create(context, values):
 def snapshot_destroy(context, snapshot_id):
     session = get_session()
     with session.begin():
-        session.query(models.Snapshot).\
+        model_query(context, models.Snapshot, session=session).\
             filter_by(id=snapshot_id).\
             update({'status': 'deleted',
                     'deleted': True,
                     'deleted_at': timeutils.utcnow(),
                     'updated_at': literal_column('updated_at')})
-        session.query(models.SnapshotMetadata).\
+        model_query(context, models.SnapshotMetadata, session=session).\
             filter_by(snapshot_id=snapshot_id).\
             update({'deleted': True,
                     'deleted_at': timeutils.utcnow(),
@@ -1657,12 +1730,6 @@ def _snapshot_metadata_get_item(context, snapshot_id, key, session=None):
     return result
 
 
-@require_context
-@require_snapshot_exists
-def snapshot_metadata_get_item(context, snapshot_id, key):
-    return _snapshot_metadata_get_item(context, snapshot_id, key)
-
-
 @require_context
 @require_snapshot_exists
 def snapshot_metadata_update(context, snapshot_id, metadata, delete):
@@ -1699,7 +1766,7 @@ def snapshot_metadata_update(context, snapshot_id, metadata, delete):
             meta_ref.update(item)
             meta_ref.save(session=session)
 
-        return metadata
+    return snapshot_metadata_get(context, snapshot_id)
 
 ###################
 
@@ -1740,9 +1807,7 @@ def volume_type_create(context, values):
 
 @require_context
 def volume_type_get_all(context, inactive=False, filters=None):
-    """
-    Returns a dict describing all volume_types with name as key.
-    """
+    """Returns a dict describing all volume_types with name as key."""
     filters = filters or {}
 
     read_deleted = "yes" if inactive else "no"
@@ -1778,7 +1843,7 @@ def _volume_type_get(context, id, session=None, inactive=False):
 
 @require_context
 def volume_type_get(context, id, inactive=False):
-    """Returns a dict describing specific volume_type"""
+    """Return a dict describing specific volume_type."""
 
     return _volume_type_get(context, id, None, inactive)
 
@@ -1798,7 +1863,7 @@ def _volume_type_get_by_name(context, name, session=None):
 
 @require_context
 def volume_type_get_by_name(context, name):
-    """Returns a dict describing specific volume_type"""
+    """Return a dict describing specific volume_type."""
 
     return _volume_type_get_by_name(context, name)
 
@@ -1900,12 +1965,12 @@ def volume_type_destroy(context, id):
             msg = _('VolumeType %s deletion failed, VolumeType in use.') % id
             LOG.error(msg)
             raise exception.VolumeTypeInUse(volume_type_id=id)
-        session.query(models.VolumeTypes).\
+        model_query(context, models.VolumeTypes, session=session).\
             filter_by(id=id).\
             update({'deleted': True,
                     'deleted_at': timeutils.utcnow(),
                     'updated_at': literal_column('updated_at')})
-        session.query(models.VolumeTypeExtraSpecs).\
+        model_query(context, models.VolumeTypeExtraSpecs, session=session).\
             filter_by(volume_type_id=id).\
             update({'deleted': True,
                     'deleted_at': timeutils.utcnow(),
@@ -1979,11 +2044,6 @@ def _volume_type_extra_specs_get_item(context, volume_type_id, key,
     return result
 
 
-@require_context
-def volume_type_extra_specs_get_item(context, volume_type_id, key):
-    return _volume_type_extra_specs_get_item(context, volume_type_id, key)
-
-
 @require_context
 def volume_type_extra_specs_update_or_create(context, volume_type_id,
                                              specs):
@@ -2179,8 +2239,8 @@ def qos_specs_associations_get(context, qos_specs_id):
     extend qos specs association to other entities, such as volumes,
     sometime in future.
     """
+    # Raise QoSSpecsNotFound if no specs found
     _qos_specs_get_ref(context, qos_specs_id, None)
-
     return volume_type_qos_associations_get(context, qos_specs_id)
 
 
@@ -2307,22 +2367,35 @@ def volume_type_encryption_delete(context, volume_type_id):
 
 
 @require_admin_context
-def volume_type_encryption_update_or_create(context, volume_type_id,
-                                            values):
+def volume_type_encryption_create(context, volume_type_id, values):
     session = get_session()
-    encryption = volume_type_encryption_get(context, volume_type_id,
-                                            session)
-
-    if not encryption:
+    with session.begin():
         encryption = models.Encryption()
 
         if 'volume_type_id' not in values:
             values['volume_type_id'] = volume_type_id
 
-    encryption.update(values)
-    encryption.save(session=session)
+        encryption.update(values)
+        encryption.save(session=session)
 
-    return encryption
+        return encryption
+
+
+@require_admin_context
+def volume_type_encryption_update(context, volume_type_id, values):
+    session = get_session()
+    with session.begin():
+        encryption = volume_type_encryption_get(context, volume_type_id,
+                                                session)
+
+        if not encryption:
+            raise exception.VolumeTypeEncryptionNotFound(type_id=
+                                                         volume_type_id)
+
+        encryption.update(values)
+        encryption.save(session=session)
+
+        return encryption
 
 
 def volume_type_encryption_volume_get(context, volume_type_id, session=None):
@@ -2421,8 +2494,8 @@ def volume_snapshot_glance_metadata_get(context, snapshot_id):
 @require_context
 @require_volume_exists
 def volume_glance_metadata_create(context, volume_id, key, value):
-    """
-    Update the Glance metadata for a volume by adding a new key:value pair.
+    """Update the Glance metadata for a volume by adding a new key:value pair.
+
     This API does not support changing the value of a key once it has been
     created.
     """
@@ -2441,7 +2514,7 @@ def volume_glance_metadata_create(context, volume_id, key, value):
         vol_glance_metadata = models.VolumeGlanceMetadata()
         vol_glance_metadata.volume_id = volume_id
         vol_glance_metadata.key = key
-        vol_glance_metadata.value = value
+        vol_glance_metadata.value = str(value)
 
         vol_glance_metadata.save(session=session)
 
@@ -2451,10 +2524,11 @@ def volume_glance_metadata_create(context, volume_id, key, value):
 @require_context
 @require_snapshot_exists
 def volume_glance_metadata_copy_to_snapshot(context, snapshot_id, volume_id):
-    """
-    Update the Glance metadata for a snapshot by copying all of the key:value
-    pairs from the originating volume. This is so that a volume created from
-    the snapshot will retain the original metadata.
+    """Update the Glance metadata for a snapshot.
+
+    This copies all of the key:value pairs from the originating volume, to
+    ensure that a volume created from the snapshot will retain the
+    original metadata.
     """
 
     session = get_session()
@@ -2475,10 +2549,11 @@ def volume_glance_metadata_copy_to_snapshot(context, snapshot_id, volume_id):
 def volume_glance_metadata_copy_from_volume_to_volume(context,
                                                       src_volume_id,
                                                       volume_id):
-    """
-    Update the Glance metadata for a volume by copying all of the key:value
-    pairs from the originating volume. This is so that a volume created from
-    the volume (clone) will retain the original metadata.
+    """Update the Glance metadata for a volume.
+
+    This copies all all of the key:value pairs from the originating volume,
+    to ensure that a volume created from the volume (clone) will
+    retain the original metadata.
     """
 
     session = get_session()
@@ -2498,10 +2573,10 @@ def volume_glance_metadata_copy_from_volume_to_volume(context,
 @require_context
 @require_volume_exists
 def volume_glance_metadata_copy_to_volume(context, volume_id, snapshot_id):
-    """
-    Update the Glance metadata from a volume (created from a snapshot) by
-    copying all of the key:value pairs from the originating snapshot. This is
-    so that the Glance metadata from the original volume is retained.
+    """Update the Glance metadata from a volume (created from a snapshot) by
+    copying all of the key:value pairs from the originating snapshot.
+
+    This is so that the Glance metadata from the original volume is retained.
     """
 
     session = get_session()
@@ -2616,8 +2691,8 @@ def _transfer_get(context, transfer_id, session=None):
 
     if not is_admin_context(context):
         volume = models.Volume
-        query = query.options(joinedload('volume')).\
-            filter(volume.project_id == context.project_id)
+        query = query.filter(models.Transfer.volume_id == volume.id,
+                             volume.project_id == context.project_id)
 
     result = query.first()
 
@@ -2655,10 +2730,9 @@ def transfer_get_all(context):
 def transfer_get_all_by_project(context, project_id):
     authorize_project_context(context, project_id)
 
-    volume = models.Volume
     query = model_query(context, models.Transfer).\
-        options(joinedload('volume')).\
-        filter(volume.project_id == project_id)
+        filter(models.Volume.id == models.Transfer.volume_id,
+               models.Volume.project_id == project_id)
     results = query.all()
     return _translate_transfers(results)
 
@@ -2705,7 +2779,7 @@ def transfer_destroy(context, transfer_id):
             volume_ref['status'] = 'available'
         volume_ref.update(volume_ref)
         volume_ref.save(session=session)
-        session.query(models.Transfer).\
+        model_query(context, models.Transfer, session=session).\
             filter_by(id=transfer_id).\
             update({'deleted': True,
                     'deleted_at': timeutils.utcnow(),
