@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 OpenStack Foundation
 # Copyright 2013 NTT corp.
 # All Rights Reserved.
@@ -27,10 +25,10 @@ import random
 import shutil
 import sys
 import time
-import urlparse
 
 import glanceclient.exc
 from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
 
 from cinder import exception
 from cinder.openstack.common import jsonutils
@@ -46,6 +44,7 @@ glance_opts = [
 ]
 CONF = cfg.CONF
 CONF.register_opts(glance_opts)
+CONF.import_opt('glance_api_version', 'cinder.common.config')
 
 LOG = logging.getLogger(__name__)
 
@@ -175,7 +174,7 @@ class GlanceClientWrapper(object):
                                                    'extra': extra,
                                                    }
                     LOG.exception(error_msg)
-                    raise exception.GlanceConnectionFailed(reason=str(e))
+                    raise exception.GlanceConnectionFailed(reason=e)
                 LOG.exception(error_msg)
                 time.sleep(1)
 
@@ -414,14 +413,25 @@ def _convert_to_string(metadata):
 
 
 def _extract_attributes(image):
+    #NOTE(hdd): If a key is not found, base.Resource.__getattr__() may perform
+    # a get(), resulting in a useless request back to glance. This list is
+    # therefore sorted, with dependent attributes as the end
+    # 'deleted_at' depends on 'deleted'
+    # 'checksum' depends on 'status' == 'active'
     IMAGE_ATTRIBUTES = ['size', 'disk_format', 'owner',
-                        'container_format', 'checksum', 'id',
+                        'container_format', 'status', 'id',
                         'name', 'created_at', 'updated_at',
-                        'deleted_at', 'deleted', 'status',
+                        'deleted', 'deleted_at', 'checksum',
                         'min_disk', 'min_ram', 'is_public']
     output = {}
+
     for attr in IMAGE_ATTRIBUTES:
-        output[attr] = getattr(image, attr, None)
+        if attr == 'deleted_at' and not output['deleted']:
+            output[attr] = None
+        elif attr == 'checksum' and output['status'] != 'active':
+            output[attr] = None
+        else:
+            output[attr] = getattr(image, attr)
 
     output['properties'] = getattr(image, 'properties', {})
 
