@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -28,16 +26,15 @@ we should look at maybe pushing this up to Oslo
 
 import contextlib
 import os
-import re
 import tempfile
 
 from oslo.config import cfg
 
 from cinder import exception
 from cinder.openstack.common import fileutils
+from cinder.openstack.common import imageutils
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import processutils
-from cinder.openstack.common import strutils
 from cinder import units
 from cinder import utils
 from cinder.volume import utils as volume_utils
@@ -53,156 +50,25 @@ CONF = cfg.CONF
 CONF.register_opts(image_helper_opt)
 
 
-class QemuImgInfo(object):
-    BACKING_FILE_RE = re.compile((r"^(.*?)\s*\(actual\s+path\s*:"
-                                  r"\s+(.*?)\)\s*$"), re.I)
-    TOP_LEVEL_RE = re.compile(r"^([\w\d\s\_\-]+):(.*)$")
-    SIZE_RE = re.compile(r"\(\s*(\d+)\s+bytes\s*\)", re.I)
-
-    def __init__(self, cmd_output):
-        details = self._parse(cmd_output)
-        self.image = details.get('image')
-        self.backing_file = details.get('backing_file')
-        self.file_format = details.get('file_format')
-        self.virtual_size = details.get('virtual_size')
-        self.cluster_size = details.get('cluster_size')
-        self.disk_size = details.get('disk_size')
-        self.snapshots = details.get('snapshot_list', [])
-        self.encryption = details.get('encryption')
-
-    def __str__(self):
-        lines = [
-            'image: %s' % self.image,
-            'file_format: %s' % self.file_format,
-            'virtual_size: %s' % self.virtual_size,
-            'disk_size: %s' % self.disk_size,
-            'cluster_size: %s' % self.cluster_size,
-            'backing_file: %s' % self.backing_file,
-        ]
-        if self.snapshots:
-            lines.append("snapshots: %s" % self.snapshots)
-        return "\n".join(lines)
-
-    def _canonicalize(self, field):
-        # Standardize on underscores/lc/no dash and no spaces
-        # since qemu seems to have mixed outputs here... and
-        # this format allows for better integration with python
-        # - ie for usage in kwargs and such...
-        field = field.lower().strip()
-        for c in (" ", "-"):
-            field = field.replace(c, '_')
-        return field
-
-    def _extract_bytes(self, details):
-        # Replace it with the byte amount
-        real_size = self.SIZE_RE.search(details)
-        if real_size:
-            details = real_size.group(1)
-        try:
-            details = strutils.to_bytes(details)
-        except TypeError:
-            pass
-        return details
-
-    def _extract_details(self, root_cmd, root_details, lines_after):
-        consumed_lines = 0
-        real_details = root_details
-        if root_cmd == 'backing_file':
-            # Replace it with the real backing file
-            backing_match = self.BACKING_FILE_RE.match(root_details)
-            if backing_match:
-                real_details = backing_match.group(2).strip()
-        elif root_cmd in ['virtual_size', 'cluster_size', 'disk_size']:
-            # Replace it with the byte amount (if we can convert it)
-            real_details = self._extract_bytes(root_details)
-        elif root_cmd == 'file_format':
-            real_details = real_details.strip().lower()
-        elif root_cmd == 'snapshot_list':
-            # Next line should be a header, starting with 'ID'
-            if not lines_after or not lines_after[0].startswith("ID"):
-                msg = _("Snapshot list encountered but no header found!")
-                raise ValueError(msg)
-            consumed_lines += 1
-            possible_contents = lines_after[1:]
-            real_details = []
-            # This is the sprintf pattern we will try to match
-            # "%-10s%-20s%7s%20s%15s"
-            # ID TAG VM SIZE DATE VM CLOCK (current header)
-            for line in possible_contents:
-                line_pieces = line.split(None)
-                if len(line_pieces) != 6:
-                    break
-                else:
-                    # Check against this pattern occuring in the final position
-                    # "%02d:%02d:%02d.%03d"
-                    date_pieces = line_pieces[5].split(":")
-                    if len(date_pieces) != 3:
-                        break
-                    real_details.append({
-                        'id': line_pieces[0],
-                        'tag': line_pieces[1],
-                        'vm_size': line_pieces[2],
-                        'date': line_pieces[3],
-                        'vm_clock': line_pieces[4] + " " + line_pieces[5],
-                    })
-                    consumed_lines += 1
-        return (real_details, consumed_lines)
-
-    def _parse(self, cmd_output):
-        # Analysis done of qemu-img.c to figure out what is going on here
-        # Find all points start with some chars and then a ':' then a newline
-        # and then handle the results of those 'top level' items in a separate
-        # function.
-        #
-        # TODO(harlowja): newer versions might have a json output format
-        #                 we should switch to that whenever possible.
-        #                 see: http://bit.ly/XLJXDX
-        if not cmd_output:
-            cmd_output = ''
-        contents = {}
-        lines = cmd_output.splitlines()
-        i = 0
-        line_am = len(lines)
-        while i < line_am:
-            line = lines[i]
-            if not line.strip():
-                i += 1
-                continue
-            consumed_lines = 0
-            top_level = self.TOP_LEVEL_RE.match(line)
-            if top_level:
-                root = self._canonicalize(top_level.group(1))
-                if not root:
-                    i += 1
-                    continue
-                root_details = top_level.group(2).strip()
-                details, consumed_lines = self._extract_details(root,
-                                                                root_details,
-                                                                lines[i + 1:])
-                contents[root] = details
-            i += consumed_lines + 1
-        return contents
-
-
 def qemu_img_info(path):
     """Return a object containing the parsed output from qemu-img info."""
-    cmd = ('env', 'LC_ALL=C', 'LANG=C', 'qemu-img', 'info', path)
+    cmd = ('env', 'LC_ALL=C', 'qemu-img', 'info', path)
     if os.name == 'nt':
-        cmd = cmd[3:]
+        cmd = cmd[2:]
     out, err = utils.execute(*cmd, run_as_root=True)
-    return QemuImgInfo(out)
+    return imageutils.QemuImgInfo(out)
 
 
 def convert_image(source, dest, out_format):
-    """Convert image to other format"""
+    """Convert image to other format."""
     cmd = ('qemu-img', 'convert', '-O', out_format, source, dest)
     utils.execute(*cmd, run_as_root=True)
 
 
-def resize_image(source, size):
+def resize_image(source, size, run_as_root=False):
     """Changes the virtual size of the image."""
     cmd = ('qemu-img', 'resize', source, '%sG' % size)
-    utils.execute(*cmd, run_as_root=False)
+    utils.execute(*cmd, run_as_root=run_as_root)
 
 
 def fetch(context, image_service, image_id, path, _user_id, _project_id):
@@ -246,21 +112,21 @@ def fetch_verify_image(context, image_service, image_id, dest,
 
 
 def fetch_to_vhd(context, image_service,
-                 image_id, dest,
+                 image_id, dest, blocksize,
                  user_id=None, project_id=None):
     fetch_to_volume_format(context, image_service, image_id, dest, 'vpc',
-                           user_id, project_id)
+                           blocksize, user_id, project_id)
 
 
 def fetch_to_raw(context, image_service,
-                 image_id, dest,
+                 image_id, dest, blocksize,
                  user_id=None, project_id=None, size=None):
     fetch_to_volume_format(context, image_service, image_id, dest, 'raw',
-                           user_id, project_id, size)
+                           blocksize, user_id, project_id, size)
 
 
 def fetch_to_volume_format(context, image_service,
-                           image_id, dest, volume_format,
+                           image_id, dest, volume_format, blocksize,
                            user_id=None, project_id=None, size=None):
     if (CONF.image_conversion_dir and not
             os.path.exists(CONF.image_conversion_dir)):
@@ -312,7 +178,7 @@ def fetch_to_volume_format(context, image_service,
             LOG.debug(_('Copying image from %(tmp)s to volume %(dest)s - '
                         'size: %(size)s') % {'tmp': tmp, 'dest': dest,
                                              'size': image_meta['size']})
-            volume_utils.copy_volume(tmp, dest, image_meta['size'])
+            volume_utils.copy_volume(tmp, dest, image_meta['size'], blocksize)
             return
 
         data = qemu_img_info(tmp)
