@@ -19,7 +19,6 @@
 System-level utilities and helper functions.
 """
 
-import logging
 import os
 import random
 import shlex
@@ -29,7 +28,7 @@ from eventlet.green import subprocess
 from eventlet import greenthread
 
 from cinder.openstack.common.gettextutils import _
-from cinder.openstack.common import strutils
+from cinder.openstack.common import log as logging
 
 
 LOG = logging.getLogger(__name__)
@@ -103,8 +102,6 @@ def execute(*cmd, **kwargs):
     :param shell:           whether or not there should be a shell used to
                             execute this command. Defaults to false.
     :type shell:            boolean
-    :param loglevel:        log level for execute commands.
-    :type loglevel:         int.  (Should be logging.DEBUG or logging.INFO)
     :returns:               (stdout, stderr) from process execution
     :raises:                :class:`UnknownArgumentError` on
                             receiving unknown arguments
@@ -119,7 +116,6 @@ def execute(*cmd, **kwargs):
     run_as_root = kwargs.pop('run_as_root', False)
     root_helper = kwargs.pop('root_helper', '')
     shell = kwargs.pop('shell', False)
-    loglevel = kwargs.pop('loglevel', logging.DEBUG)
 
     if isinstance(check_exit_code, bool):
         ignore_exit_code = not check_exit_code
@@ -139,12 +135,11 @@ def execute(*cmd, **kwargs):
         cmd = shlex.split(root_helper) + list(cmd)
 
     cmd = map(str, cmd)
-    sanitized_cmd = strutils.mask_password(' '.join(cmd))
 
     while attempts > 0:
         attempts -= 1
         try:
-            LOG.log(loglevel, _('Running cmd (subprocess): %s'), sanitized_cmd)
+            LOG.debug(_('Running cmd (subprocess): %s'), ' '.join(cmd))
             _PIPE = subprocess.PIPE  # pylint: disable=E1101
 
             if os.name == 'nt':
@@ -169,21 +164,19 @@ def execute(*cmd, **kwargs):
             obj.stdin.close()  # pylint: disable=E1101
             _returncode = obj.returncode  # pylint: disable=E1101
             if _returncode:
-                LOG.log(loglevel, _('Result was %s') % _returncode)
+                LOG.debug(_('Result was %s') % _returncode)
                 if not ignore_exit_code and _returncode not in check_exit_code:
                     (stdout, stderr) = result
-                    sanitized_stdout = strutils.mask_password(stdout)
-                    sanitized_stderr = strutils.mask_password(stderr)
                     raise ProcessExecutionError(exit_code=_returncode,
-                                                stdout=sanitized_stdout,
-                                                stderr=sanitized_stderr,
-                                                cmd=sanitized_cmd)
+                                                stdout=stdout,
+                                                stderr=stderr,
+                                                cmd=' '.join(cmd))
             return result
         except ProcessExecutionError:
             if not attempts:
                 raise
             else:
-                LOG.log(loglevel, _('%r failed. Retrying.'), sanitized_cmd)
+                LOG.debug(_('%r failed. Retrying.'), cmd)
                 if delay_on_retry:
                     greenthread.sleep(random.randint(20, 200) / 100.0)
         finally:
