@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright (c) 2011 OpenStack, LLC.
+# Copyright (c) 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,13 +15,15 @@
 
 """Policy Engine For Cinder"""
 
+
 from oslo.config import cfg
 
 from cinder import exception
-from cinder import flags
+from cinder.openstack.common.gettextutils import _
 from cinder.openstack.common import policy
 from cinder import utils
 
+
 policy_opts = [
     cfg.StrOpt('policy_file',
                default='policy.json',
@@ -32,8 +32,8 @@ policy_opts = [
                default='default',
                help=_('Rule checked when requested rule is not found')), ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(policy_opts)
+CONF = cfg.CONF
+CONF.register_opts(policy_opts)
 
 _POLICY_PATH = None
 _POLICY_CACHE = {}
@@ -51,14 +51,28 @@ def init():
     global _POLICY_PATH
     global _POLICY_CACHE
     if not _POLICY_PATH:
-        _POLICY_PATH = utils.find_config(FLAGS.policy_file)
+        _POLICY_PATH = utils.find_config(CONF.policy_file)
     utils.read_cached_file(_POLICY_PATH, _POLICY_CACHE,
                            reload_func=_set_brain)
 
 
 def _set_brain(data):
-    default_rule = FLAGS.policy_default_rule
-    policy.set_brain(policy.HttpBrain.load_json(data, default_rule))
+    default_rule = CONF.policy_default_rule
+    policy.set_brain(policy.Brain.load_json(data, default_rule))
+
+
+def enforce_action(context, action):
+    """Checks that the action can be done by the given context.
+
+    Applies a check to ensure the context's project_id and user_id can be
+    applied to the given action using the policy enforcement api.
+    """
+
+    target = {
+        'project_id': context.project_id,
+        'user_id': context.user_id,
+    }
+    enforce(context, action, target)
 
 
 def enforce(context, action, target):
