@@ -1,4 +1,4 @@
-# Copyright (c) 2011 OpenStack, LLC.
+# Copyright (c) 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,13 +23,13 @@ from oslo.config import cfg
 
 from cinder import db
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder.openstack.common.scheduler import filters
 from cinder.openstack.common.scheduler import weights
 from cinder.openstack.common import timeutils
 from cinder import utils
 
+
 host_manager_opts = [
     cfg.ListOpt('scheduler_default_filters',
                 default=[
@@ -46,8 +46,9 @@ host_manager_opts = [
                 help='Which weigher class names to use for weighing hosts.')
 ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(host_manager_opts)
+CONF = cfg.CONF
+CONF.register_opts(host_manager_opts)
+CONF.import_opt('scheduler_driver', 'cinder.scheduler.manager')
 
 LOG = logging.getLogger(__name__)
 
@@ -99,6 +100,9 @@ class HostState(object):
         # Mutable available resources.
         # These will change as resources are virtually "consumed".
         self.total_capacity_gb = 0
+        # capacity has been allocated in cinder POV, which should be
+        # sum(vol['size'] for vol in vols_on_hosts)
+        self.allocated_capacity_gb = 0
         self.free_capacity_gb = None
         self.reserved_percentage = 0
 
@@ -128,13 +132,16 @@ class HostState(object):
 
             self.total_capacity_gb = capability['total_capacity_gb']
             self.free_capacity_gb = capability['free_capacity_gb']
+            self.allocated_capacity_gb = capability.get(
+                'allocated_capacity_gb', 0)
             self.reserved_percentage = capability['reserved_percentage']
 
             self.updated = capability['timestamp']
 
     def consume_from_volume(self, volume):
-        """Incrementally update host state from an volume"""
+        """Incrementally update host state from an volume."""
         volume_gb = volume['size']
+        self.allocated_capacity_gb += volume_gb
         if self.free_capacity_gb == 'infinite':
             # There's virtually infinite space on back-end
             pass
@@ -165,14 +172,32 @@ class HostManager(object):
                                                         'weights')
         self.weight_classes = self.weight_handler.get_all_classes()
 
+        default_filters = ['AvailabilityZoneFilter',
+                           'CapacityFilter',
+                           'CapabilitiesFilter']
+        chance = 'cinder.scheduler.chance.ChanceScheduler'
+        simple = 'cinder.scheduler.simple.SimpleScheduler'
+        if CONF.scheduler_driver == simple:
+            CONF.set_override('scheduler_default_filters', default_filters)
+            CONF.set_override('scheduler_default_weighers',
+                              ['AllocatedCapacityWeigher'])
+        elif CONF.scheduler_driver == chance:
+            CONF.set_override('scheduler_default_filters', default_filters)
+            CONF.set_override('scheduler_default_weighers',
+                              ['ChanceWeigher'])
+        else:
+            # Do nothing when some other scheduler is configured
+            pass
+
     def _choose_host_filters(self, filter_cls_names):
-        """Since the caller may specify which filters to use we need
-        to have an authoritative list of what is permissible. This
-        function checks the filter names against a predefined set
-        of acceptable filters.
+        """Return a list of available filter names.
+
+        This function checks input filter names against a predefined set
+        of acceptable filterss (all loaded filters).  If input is None,
+        it uses CONF.scheduler_default_filters instead.
         """
         if filter_cls_names is None:
-            filter_cls_names = FLAGS.scheduler_default_filters
+            filter_cls_names = CONF.scheduler_default_filters
         if not isinstance(filter_cls_names, (list, tuple)):
             filter_cls_names = [filter_cls_names]
         good_filters = []
@@ -192,13 +217,14 @@ class HostManager(object):
         return good_filters
 
     def _choose_host_weighers(self, weight_cls_names):
-        """Since the caller may specify which weighers to use, we need
-        to have an authoritative list of what is permissible. This
-        function checks the weigher names against a predefined set
-        of acceptable weighers.
+        """Return a list of available weigher names.
+
+        This function checks input weigher names against a predefined set
+        of acceptable weighers (all loaded weighers).  If input is None,
+        it uses CONF.scheduler_default_weighers instead.
         """
         if weight_cls_names is None:
-            weight_cls_names = FLAGS.scheduler_default_weighers
+            weight_cls_names = CONF.scheduler_default_weighers
         if not isinstance(weight_cls_names, (list, tuple)):
             weight_cls_names = [weight_cls_names]
 
@@ -220,7 +246,7 @@ class HostManager(object):
 
     def get_filtered_hosts(self, hosts, filter_properties,
                            filter_class_names=None):
-        """Filter hosts and return only ones passing all filters"""
+        """Filter hosts and return only ones passing all filters."""
         filter_classes = self._choose_host_filters(filter_class_names)
         return self.filter_handler.get_filtered_objects(filter_classes,
                                                         hosts,
@@ -228,7 +254,7 @@ class HostManager(object):
 
     def get_weighed_hosts(self, hosts, weight_properties,
                           weigher_class_names=None):
-        """Weigh the hosts"""
+        """Weigh the hosts."""
         weigher_classes = self._choose_host_weighers(weigher_class_names)
         return self.weight_handler.get_weighed_objects(weigher_classes,
                                                        hosts,
@@ -238,11 +264,13 @@ class HostManager(object):
         """Update the per-service capabilities based on this notification."""
         if service_name != 'volume':
             LOG.debug(_('Ignoring %(service_name)s service update '
-                        'from %(host)s'), locals())
+                        'from %(host)s'),
+                      {'service_name': service_name, 'host': host})
             return
 
         LOG.debug(_("Received %(service_name)s service update from "
-                    "%(host)s.") % locals())
+                    "%(host)s.") %
+                  {'service_name': service_name, 'host': host})
 
         # Copy the capabilities, so we don't modify the original dict
         capab_copy = dict(capabilities)
@@ -250,22 +278,25 @@ class HostManager(object):
         self.service_states[host] = capab_copy
 
     def get_all_host_states(self, context):
-        """Returns a dict of all the hosts the HostManager
-          knows about. Also, each of the consumable resources in HostState
-          are pre-populated and adjusted based on data in the db.
+        """Returns a dict of all the hosts the HostManager knows about.
 
-          For example:
+        Each of the consumable resources in HostState are
+        populated with capabilities scheduler received from RPC.
+
+        For example:
           {'192.168.1.100': HostState(), ...}
         """
 
         # Get resource usage across the available volume nodes:
-        topic = FLAGS.volume_topic
+        topic = CONF.volume_topic
         volume_services = db.service_get_all_by_topic(context, topic)
+        active_hosts = set()
         for service in volume_services:
-            if not utils.service_is_up(service) or service['disabled']:
-                LOG.warn(_("service is down or disabled."))
-                continue
             host = service['host']
+            if not utils.service_is_up(service) or service['disabled']:
+                LOG.warn(_("volume service is down or disabled. "
+                           "(host: %s)") % host)
+                continue
             capabilities = self.service_states.get(host, None)
             host_state = self.host_state_map.get(host)
             if host_state:
@@ -278,7 +309,15 @@ class HostManager(object):
                                                  service=
                                                  dict(service.iteritems()))
                 self.host_state_map[host] = host_state
-            # update host_state
+            # update attributes in host_state that scheduler is interested in
             host_state.update_from_volume_capability(capabilities)
+            active_hosts.add(host)
+
+        # remove non-active hosts from host_state_map
+        nonactive_hosts = set(self.host_state_map.keys()) - active_hosts
+        for host in nonactive_hosts:
+            LOG.info(_("Removing non-active host: %(host)s from "
+                       "scheduler cache.") % {'host': host})
+            del self.host_state_map[host]
 
         return self.host_state_map.itervalues()
