@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -23,40 +21,39 @@ inline callbacks.
 
 """
 
-
-import functools
 import os
 import shutil
 import tempfile
 import uuid
 
 import fixtures
+import logging
 import mox
 from oslo.config import cfg
+from oslo.messaging import conffixture as messaging_conffixture
 import stubout
 import testtools
+from testtools import matchers
 
-from cinder.common import config  # Need to register global_opts
+from cinder.common import config  # noqa Need to register global_opts
 from cinder.db import migration
 from cinder.openstack.common.db.sqlalchemy import session
-from cinder.openstack.common import log as logging
+from cinder.openstack.common import log as oslo_logging
 from cinder.openstack.common import timeutils
+from cinder import rpc
 from cinder import service
 from cinder.tests import conf_fixture
-
+from cinder.tests import fake_notifier
 
 test_opts = [
     cfg.StrOpt('sqlite_clean_db',
                default='clean.sqlite',
-               help='File name of clean sqlite db'),
-    cfg.BoolOpt('fake_tests',
-                default=True,
-                help='should we use everything for testing'), ]
+               help='File name of clean sqlite db'), ]
 
 CONF = cfg.CONF
 CONF.register_opts(test_opts)
 
-LOG = logging.getLogger(__name__)
+LOG = oslo_logging.getLogger(__name__)
 
 _DB_CACHE = None
 
@@ -77,7 +74,7 @@ class Database(fixtures.Fixture):
         self.engine.dispose()
         conn = self.engine.connect()
         if sql_connection == "sqlite://":
-            if db_migrate.db_version() > db_migrate.INIT_VERSION:
+            if db_migrate.db_version() > db_migrate.db_initial_version():
                 return
         else:
             testdb = os.path.join(CONF.state_path, sqlite_db)
@@ -133,7 +130,19 @@ class TestCase(testtools.TestCase):
             stderr = self.useFixture(fixtures.StringStream('stderr')).stream
             self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))
 
-        self.log_fixture = self.useFixture(fixtures.FakeLogger())
+        self.log_fixture = self.useFixture(fixtures.FakeLogger(
+            level=logging.DEBUG))
+
+        rpc.add_extra_exmods("cinder.tests")
+        self.addCleanup(rpc.clear_extra_exmods)
+        self.addCleanup(rpc.cleanup)
+
+        fs = '%(levelname)s [%(name)s] %(message)s'
+        self.messaging_conf = messaging_conffixture.ConfFixture(CONF)
+        self.messaging_conf.transport_driver = 'fake'
+        self.messaging_conf.response_timeout = 15
+        self.useFixture(self.messaging_conf)
+        rpc.init(CONF)
 
         conf_fixture.set_defaults(CONF)
         CONF([], default_config_files=[])
@@ -168,6 +177,8 @@ class TestCase(testtools.TestCase):
         self.injected = []
         self._services = []
 
+        fake_notifier.stub_notifier(self.stubs)
+
         CONF.set_override('fatal_exception_format_errors', True)
         # This will be cleaned up by the NestedTempfile fixture
         CONF.set_override('lock_path', tempfile.mkdtemp())
@@ -201,6 +212,11 @@ class TestCase(testtools.TestCase):
         for k, v in kw.iteritems():
             CONF.set_override(k, v)
 
+    def log_level(self, level):
+        """Set logging level to the specified value."""
+        log_root = logging.getLogger(None).logger
+        log_root.setLevel(level)
+
     def start_service(self, name, host=None, **kwargs):
         host = host and host or uuid.uuid4().hex
         kwargs.setdefault('host', host)
@@ -225,8 +241,8 @@ class TestCase(testtools.TestCase):
 
         """
         def raise_assertion(msg):
-            d1str = str(d1)
-            d2str = str(d2)
+            d1str = d1
+            d2str = d2
             base_msg = ('Dictionaries do not match. %(msg)s d1: %(d1str)s '
                         'd2: %(d2str)s' %
                         {'msg': msg, 'd1str': d1str, 'd2str': d2str})
@@ -267,3 +283,25 @@ class TestCase(testtools.TestCase):
                                     'd1value': d1value,
                                     'd2value': d2value,
                                 })
+
+    def assertGreater(self, first, second, msg=None):
+        """Python < v2.7 compatibility.  Assert 'first' > 'second'."""
+        try:
+            f = super(TestCase, self).assertGreater
+        except AttributeError:
+            self.assertThat(first,
+                            matchers.GreaterThan(second),
+                            message=msg or '')
+        else:
+            f(first, second, msg=msg)
+
+    def assertGreaterEqual(self, first, second, msg=None):
+        """Python < v2.7 compatibility.  Assert 'first' >= 'second'."""
+        try:
+            f = super(TestCase, self).assertGreaterEqual
+        except AttributeError:
+            self.assertThat(first,
+                            matchers.Not(matchers.LessThan(second)),
+                            message=msg or '')
+        else:
+            f(first, second, msg=msg)
