@@ -1,4 +1,4 @@
-# Copyright 2012 OpenStack LLC.
+# Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,21 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
 from lxml import etree
 import webob
 
 from cinder.api.contrib import extended_snapshot_attributes
-from cinder import exception
-from cinder import flags
 from cinder.openstack.common import jsonutils
 from cinder import test
 from cinder.tests.api import fakes
 from cinder import volume
 
 
-FLAGS = flags.FLAGS
-
-
 UUID1 = '00000000-0000-0000-0000-000000000001'
 UUID2 = '00000000-0000-0000-0000-000000000002'
 
@@ -100,17 +96,6 @@ class ExtendedSnapshotAttributesTest(test.TestCase):
                                           project_id='fake',
                                           progress='0%')
 
-    def test_no_instance_passthrough_404(self):
-
-        def fake_snapshot_get(*args, **kwargs):
-            raise exception.InstanceNotFound(instance_id='fake')
-
-        self.stubs.Set(volume.api.API, 'get_snapshot', fake_snapshot_get)
-        url = '/v2/fake/snapshots/70f6db34-de8d-4fbd-aafb-4065bdfa6115'
-        res = self._make_request(url)
-
-        self.assertEqual(res.status_int, 404)
-
 
 class ExtendedSnapshotAttributesXmlTest(ExtendedSnapshotAttributesTest):
     content_type = 'application/xml'
