@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -85,19 +83,6 @@ def wsgi_app(inner_app_v2=None, fake_auth=True, fake_auth_context=None,
     return mapper
 
 
-def stub_out_rate_limiting(stubs):
-    def fake_rate_init(self, app):
-        # super(limits.RateLimitingMiddleware, self).__init__(app)
-        self.application = app
-
-    # FIXME(ja): unsure about limits in volumes
-    # stubs.Set(cinder.api.openstack.compute.limits.RateLimitingMiddleware,
-    #     '__init__', fake_rate_init)
-
-    # stubs.Set(cinder.api.openstack.compute.limits.RateLimitingMiddleware,
-    #     '__call__', fake_wsgi)
-
-
 def stub_out_key_pair_funcs(stubs, have_key_pair=True):
     def key_pair(context, user_id):
         return [dict(name='key', public_key='public_key')]
@@ -135,7 +120,12 @@ class HTTPRequest(webob.Request):
 
     @classmethod
     def blank(cls, *args, **kwargs):
-        kwargs['base_url'] = 'http://localhost/v1'
+        if args != None:
+            if args[0].find('v1') == 0:
+                kwargs['base_url'] = 'http://localhost/v1'
+            else:
+                kwargs['base_url'] = 'http://localhost/v2'
+
         use_admin_context = kwargs.pop('use_admin_context', False)
         out = os_wsgi.Request.blank(*args, **kwargs)
         out.environ['cinder.context'] = FakeRequestContext(
