@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -31,10 +29,11 @@ ATOMNS = "{http://www.w3.org/2005/Atom}"
 
 
 class LimiterTest(test.TestCase):
-    """
-    Unit tests for the `cinder.api.common.limited` method which takes
-    in a list of items and, depending on the 'offset' and 'limit' GET params,
-    returns a subset or complete set of the given items.
+    """Unit tests for the `cinder.api.common.limited` method.
+
+    This method takes in a list of items and, depending on the 'offset'
+    and 'limit' GET params, returns a subset or complete set of the given
+    items.
     """
 
     def setUp(self):
@@ -161,9 +160,9 @@ class LimiterTest(test.TestCase):
 
 
 class PaginationParamsTest(test.TestCase):
-    """
-    Unit tests for the `cinder.api.common.get_pagination_params`
-    method which takes in a request object and returns 'marker' and 'limit'
+    """Unit tests for `cinder.api.common.get_pagination_params` method.
+
+    This method takes in a request object and returns 'marker' and 'limit'
     GET params.
     """
 
