@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -28,6 +26,7 @@ from cinder import exception
 from cinder.openstack.common import jsonutils
 from cinder import test
 from cinder.tests.api import fakes
+from cinder.tests.api.v1 import stubs
 
 
 CONF = cfg.CONF
@@ -41,6 +40,15 @@ def return_create_volume_metadata(context, volume_id, metadata, delete):
     return stub_volume_metadata()
 
 
+def return_new_volume_metadata(context, volume_id, metadata, delete):
+    return stub_new_volume_metadata()
+
+
+def return_create_volume_metadata_insensitive(context, snapshot_id,
+                                              metadata, delete):
+    return stub_volume_metadata_insensitive()
+
+
 def return_volume_metadata(context, volume_id):
     if not isinstance(volume_id, str) or not len(volume_id) == 36:
         msg = 'id %s must be a uuid in return volume metadata' % volume_id
@@ -52,6 +60,10 @@ def return_empty_volume_metadata(context, volume_id):
     return {}
 
 
+def return_empty_container_metadata(context, volume_id, metadata, delete):
+    return {}
+
+
 def delete_volume_metadata(context, volume_id, key):
     pass
 
@@ -65,6 +77,25 @@ def stub_volume_metadata():
     return metadata
 
 
+def stub_new_volume_metadata():
+    metadata = {
+        'key10': 'value10',
+        'key99': 'value99',
+        'KEY20': 'value20',
+    }
+    return metadata
+
+
+def stub_volume_metadata_insensitive():
+    metadata = {
+        "key1": "value1",
+        "key2": "value2",
+        "key3": "value3",
+        "KEY4": "value4",
+    }
+    return metadata
+
+
 def stub_max_volume_metadata():
     metadata = {"metadata": {}}
     for num in range(CONF.quota_metadata_items):
@@ -95,6 +126,8 @@ class volumeMetaDataTest(test.TestCase):
         self.stubs.Set(cinder.db, 'volume_get', return_volume)
         self.stubs.Set(cinder.db, 'volume_metadata_get',
                        return_volume_metadata)
+        self.stubs.Set(cinder.db, 'service_get_all_by_topic',
+                       stubs.stub_service_get_all_by_topic)
 
         self.stubs.Set(self.volume_api, 'update_volume_metadata',
                        fake_update_volume_metadata)
@@ -103,8 +136,8 @@ class volumeMetaDataTest(test.TestCase):
         self.ext_mgr.extensions = {}
         self.volume_controller = volumes.VolumeController(self.ext_mgr)
         self.controller = volume_metadata.Controller()
-        self.id = str(uuid.uuid4())
-        self.url = '/v1/fake/volumes/%s/metadata' % self.id
+        self.req_id = str(uuid.uuid4())
+        self.url = '/v1/fake/volumes/%s/metadata' % self.req_id
 
         vol = {"size": 100,
                "display_name": "Volume Test Name",
@@ -117,7 +150,7 @@ class volumeMetaDataTest(test.TestCase):
 
     def test_index(self):
         req = fakes.HTTPRequest.blank(self.url)
-        res_dict = self.controller.index(req, self.id)
+        res_dict = self.controller.index(req, self.req_id)
 
         expected = {
             'metadata': {
@@ -139,13 +172,13 @@ class volumeMetaDataTest(test.TestCase):
         self.stubs.Set(cinder.db, 'volume_metadata_get',
                        return_empty_volume_metadata)
         req = fakes.HTTPRequest.blank(self.url)
-        res_dict = self.controller.index(req, self.id)
+        res_dict = self.controller.index(req, self.req_id)
         expected = {'metadata': {}}
         self.assertEqual(expected, res_dict)
 
     def test_show(self):
         req = fakes.HTTPRequest.blank(self.url + '/key2')
-        res_dict = self.controller.show(req, self.id, 'key2')
+        res_dict = self.controller.show(req, self.req_id, 'key2')
         expected = {'meta': {'key2': 'value2'}}
         self.assertEqual(expected, res_dict)
 
@@ -154,14 +187,14 @@ class volumeMetaDataTest(test.TestCase):
                        return_volume_nonexistent)
         req = fakes.HTTPRequest.blank(self.url + '/key2')
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.show, req, self.id, 'key2')
+                          self.controller.show, req, self.req_id, 'key2')
 
     def test_show_meta_not_found(self):
         self.stubs.Set(cinder.db, 'volume_metadata_get',
                        return_empty_volume_metadata)
         req = fakes.HTTPRequest.blank(self.url + '/key6')
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.show, req, self.id, 'key6')
+                          self.controller.show, req, self.req_id, 'key6')
 
     def test_delete(self):
         self.stubs.Set(cinder.db, 'volume_metadata_get',
@@ -170,7 +203,7 @@ class volumeMetaDataTest(test.TestCase):
                        delete_volume_metadata)
         req = fakes.HTTPRequest.blank(self.url + '/key2')
         req.method = 'DELETE'
-        res = self.controller.delete(req, self.id, 'key2')
+        res = self.controller.delete(req, self.req_id, 'key2')
 
         self.assertEqual(200, res.status_int)
 
@@ -180,7 +213,7 @@ class volumeMetaDataTest(test.TestCase):
         req = fakes.HTTPRequest.blank(self.url + '/key1')
         req.method = 'DELETE'
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.delete, req, self.id, 'key1')
+                          self.controller.delete, req, self.req_id, 'key1')
 
     def test_delete_meta_not_found(self):
         self.stubs.Set(cinder.db, 'volume_metadata_get',
@@ -188,7 +221,7 @@ class volumeMetaDataTest(test.TestCase):
         req = fakes.HTTPRequest.blank(self.url + '/key6')
         req.method = 'DELETE'
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.delete, req, self.id, 'key6')
+                          self.controller.delete, req, self.req_id, 'key6')
 
     def test_create(self):
         self.stubs.Set(cinder.db, 'volume_metadata_get',
@@ -199,11 +232,38 @@ class volumeMetaDataTest(test.TestCase):
         req = fakes.HTTPRequest.blank('/v1/volume_metadata')
         req.method = 'POST'
         req.content_type = "application/json"
-        body = {"metadata": {"key9": "value9"}}
+        body = {"metadata": {"key1": "value1",
+                             "key2": "value2",
+                             "key3": "value3", }}
         req.body = jsonutils.dumps(body)
-        res_dict = self.controller.create(req, self.id, body)
+        res_dict = self.controller.create(req, self.req_id, body)
         self.assertEqual(body, res_dict)
 
+    def test_create_with_keys_in_uppercase_and_lowercase(self):
+        # if the keys in uppercase_and_lowercase, should return the one
+        # which server added
+        self.stubs.Set(cinder.db, 'volume_metadata_get',
+                       return_empty_volume_metadata)
+        self.stubs.Set(cinder.db, 'volume_metadata_update',
+                       return_create_volume_metadata_insensitive)
+
+        req = fakes.HTTPRequest.blank('/v1/volume_metadata')
+        req.method = 'POST'
+        req.content_type = "application/json"
+        body = {"metadata": {"key1": "value1",
+                             "KEY1": "value1",
+                             "key2": "value2",
+                             "KEY2": "value2",
+                             "key3": "value3",
+                             "KEY4": "value4"}}
+        expected = {"metadata": {"key1": "value1",
+                                 "key2": "value2",
+                                 "key3": "value3",
+                                 "KEY4": "value4"}}
+        req.body = jsonutils.dumps(body)
+        res_dict = self.controller.create(req, self.req_id, body)
+        self.assertEqual(expected, res_dict)
+
     def test_create_empty_body(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
                        return_create_volume_metadata)
@@ -212,7 +272,7 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, req, self.id, None)
+                          self.controller.create, req, self.req_id, None)
 
     def test_create_item_empty_key(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -224,7 +284,7 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, req, self.id, body)
+                          self.controller.create, req, self.req_id, body)
 
     def test_create_item_key_too_long(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -237,7 +297,7 @@ class volumeMetaDataTest(test.TestCase):
 
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.controller.create,
-                          req, self.id, body)
+                          req, self.req_id, body)
 
     def test_create_nonexistent_volume(self):
         self.stubs.Set(cinder.db, 'volume_get',
@@ -253,11 +313,11 @@ class volumeMetaDataTest(test.TestCase):
         body = {"metadata": {"key9": "value9"}}
         req.body = jsonutils.dumps(body)
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.create, req, self.id, body)
+                          self.controller.create, req, self.req_id, body)
 
     def test_update_all(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
-                       return_create_volume_metadata)
+                       return_new_volume_metadata)
         req = fakes.HTTPRequest.blank(self.url)
         req.method = 'PUT'
         req.content_type = "application/json"
@@ -265,22 +325,51 @@ class volumeMetaDataTest(test.TestCase):
             'metadata': {
                 'key10': 'value10',
                 'key99': 'value99',
+                'KEY20': 'value20',
             },
         }
         req.body = jsonutils.dumps(expected)
-        res_dict = self.controller.update_all(req, self.id, expected)
+        res_dict = self.controller.update_all(req, self.req_id, expected)
+
+        self.assertEqual(expected, res_dict)
+
+    def test_update_all_with_keys_in_uppercase_and_lowercase(self):
+        self.stubs.Set(cinder.db, 'volume_metadata_get',
+                       return_create_volume_metadata)
+        self.stubs.Set(cinder.db, 'volume_metadata_update',
+                       return_new_volume_metadata)
+        req = fakes.HTTPRequest.blank(self.url)
+        req.method = 'PUT'
+        req.content_type = "application/json"
+        body = {
+            'metadata': {
+                'key10': 'value10',
+                'KEY10': 'value10',
+                'key99': 'value99',
+                'KEY20': 'value20',
+            },
+        }
+        expected = {
+            'metadata': {
+                'key10': 'value10',
+                'key99': 'value99',
+                'KEY20': 'value20',
+            },
+        }
+        req.body = jsonutils.dumps(expected)
+        res_dict = self.controller.update_all(req, self.req_id, body)
 
         self.assertEqual(expected, res_dict)
 
     def test_update_all_empty_container(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
-                       return_create_volume_metadata)
+                       return_empty_container_metadata)
         req = fakes.HTTPRequest.blank(self.url)
         req.method = 'PUT'
         req.content_type = "application/json"
         expected = {'metadata': {}}
         req.body = jsonutils.dumps(expected)
-        res_dict = self.controller.update_all(req, self.id, expected)
+        res_dict = self.controller.update_all(req, self.req_id, expected)
 
         self.assertEqual(expected, res_dict)
 
@@ -294,7 +383,8 @@ class volumeMetaDataTest(test.TestCase):
         req.body = jsonutils.dumps(expected)
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update_all, req, self.id, expected)
+                          self.controller.update_all, req, self.req_id,
+                          expected)
 
     def test_update_all_malformed_data(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -306,7 +396,8 @@ class volumeMetaDataTest(test.TestCase):
         req.body = jsonutils.dumps(expected)
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update_all, req, self.id, expected)
+                          self.controller.update_all, req, self.req_id,
+                          expected)
 
     def test_update_all_nonexistent_volume(self):
         self.stubs.Set(cinder.db, 'volume_get', return_volume_nonexistent)
@@ -327,7 +418,7 @@ class volumeMetaDataTest(test.TestCase):
         body = {"meta": {"key1": "value1"}}
         req.body = jsonutils.dumps(body)
         req.headers["content-type"] = "application/json"
-        res_dict = self.controller.update(req, self.id, 'key1', body)
+        res_dict = self.controller.update(req, self.req_id, 'key1', body)
         expected = {'meta': {'key1': 'value1'}}
         self.assertEqual(expected, res_dict)
 
@@ -341,7 +432,8 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPNotFound,
-                          self.controller.update, req, self.id, 'key1', body)
+                          self.controller.update, req, self.req_id, 'key1',
+                          body)
 
     def test_update_item_empty_body(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -351,7 +443,8 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, self.id, 'key1', None)
+                          self.controller.update, req, self.req_id, 'key1',
+                          None)
 
     def test_update_item_empty_key(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -363,7 +456,7 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, self.id, '', body)
+                          self.controller.update, req, self.req_id, '', body)
 
     def test_update_item_key_too_long(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -376,7 +469,7 @@ class volumeMetaDataTest(test.TestCase):
 
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.controller.update,
-                          req, self.id, ("a" * 260), body)
+                          req, self.req_id, ("a" * 260), body)
 
     def test_update_item_value_too_long(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -389,7 +482,7 @@ class volumeMetaDataTest(test.TestCase):
 
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                           self.controller.update,
-                          req, self.id, "key1", body)
+                          req, self.req_id, "key1", body)
 
     def test_update_item_too_many_keys(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -401,7 +494,8 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, self.id, 'key1', body)
+                          self.controller.update, req, self.req_id, 'key1',
+                          body)
 
     def test_update_item_body_uri_mismatch(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -413,7 +507,8 @@ class volumeMetaDataTest(test.TestCase):
         req.headers["content-type"] = "application/json"
 
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.update, req, self.id, 'bad', body)
+                          self.controller.update, req, self.req_id, 'bad',
+                          body)
 
     def test_invalid_metadata_items_on_create(self):
         self.stubs.Set(cinder.db, 'volume_metadata_update',
@@ -426,16 +521,16 @@ class volumeMetaDataTest(test.TestCase):
         data = {"metadata": {"a" * 260: "value1"}}
         req.body = jsonutils.dumps(data)
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
-                          self.controller.create, req, self.id, data)
+                          self.controller.create, req, self.req_id, data)
 
         #test for long value
         data = {"metadata": {"key": "v" * 260}}
         req.body = jsonutils.dumps(data)
         self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
-                          self.controller.create, req, self.id, data)
+                          self.controller.create, req, self.req_id, data)
 
         #test for empty key.
         data = {"metadata": {"": "value1"}}
         req.body = jsonutils.dumps(data)
         self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, req, self.id, data)
+                          self.controller.create, req, self.req_id, data)
