@@ -18,7 +18,6 @@ import datetime
 
 from lxml import etree
 from oslo.config import cfg
-import urllib
 import webob
 
 from cinder.api import extensions
@@ -29,10 +28,11 @@ from cinder import exception
 from cinder import test
 from cinder.tests.api import fakes
 from cinder.tests.api.v2 import stubs
+from cinder.tests import fake_notifier
 from cinder.tests.image import fake as fake_image
+from cinder import utils
 from cinder.volume import api as volume_api
 
-
 CONF = cfg.CONF
 
 NS = '{http://docs.openstack.org/api/openstack-volume/2.0/content}'
@@ -63,12 +63,19 @@ class VolumeApiTest(test.TestCase):
         fake_image.stub_out_image_service(self.stubs)
         self.controller = volumes.VolumeController(self.ext_mgr)
 
+        self.flags(host='fake',
+                   notification_driver=[fake_notifier.__name__])
+
         self.stubs.Set(db, 'volume_get_all', stubs.stub_volume_get_all)
         self.stubs.Set(volume_api.API, 'delete', stubs.stub_volume_delete)
         self.stubs.Set(db, 'service_get_all_by_topic',
                        stubs.stub_service_get_all_by_topic)
         self.maxDiff = None
 
+    def tearDown(self):
+        super(VolumeApiTest, self).tearDown()
+        fake_notifier.reset()
+
     def test_volume_create(self):
         self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
         self.stubs.Set(volume_api.API, "create", stubs.stub_volume_create)
@@ -82,23 +89,33 @@ class VolumeApiTest(test.TestCase):
         body = {"volume": vol}
         req = fakes.HTTPRequest.blank('/v2/volumes')
         res_dict = self.controller.create(req, body)
-        expected = {
-            'volume': {
-                'name': 'Volume Test Name',
-                'id': '1',
-                'links': [
-                    {
-                        'href': 'http://localhost/v1/fake/volumes/1',
-                        'rel': 'self'
-                    },
-                    {
-                        'href': 'http://localhost/fake/volumes/1',
-                        'rel': 'bookmark'
-                    }
-                ],
-            }
-        }
-        self.assertEqual(res_dict, expected)
+        ex = {'volume': {'attachments':
+                         [{'device': '/',
+                           'host_name': None,
+                           'id': '1',
+                           'server_id': 'fakeuuid',
+                           'volume_id': '1'}],
+                         'availability_zone': 'zone1:host1',
+                         'bootable': 'false',
+                         'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                         'description': 'Volume Test Desc',
+                         'id': '1',
+                         'links':
+                         [{'href': 'http://localhost/v2/fake/volumes/1',
+                           'rel': 'self'},
+                          {'href': 'http://localhost/fake/volumes/1',
+                           'rel': 'bookmark'}],
+                         'metadata': {'attached_mode': 'rw',
+                                      'readonly': 'False'},
+                         'name': 'Volume Test Name',
+                         'size': 100,
+                         'snapshot_id': None,
+                         'source_volid': None,
+                         'status': 'fakestatus',
+                         'user_id': 'fakeuser',
+                         'volume_type': 'vol_type_name',
+                         'encrypted': False}}
+        self.assertEqual(res_dict, ex)
 
     def test_volume_create_with_type(self):
         vol_type = db.volume_type_create(
@@ -176,26 +193,35 @@ class VolumeApiTest(test.TestCase):
                "description": "Volume Test Desc",
                "availability_zone": "nova",
                "imageRef": 'c905cedb-7281-47e4-8a62-f26bc5fc4c77'}
-        expected = {
-            'volume': {
-                'name': 'Volume Test Name',
-                'id': '1',
-                'links': [
-                    {
-                        'href': 'http://localhost/v1/fake/volumes/1',
-                        'rel': 'self'
-                    },
-                    {
-                        'href': 'http://localhost/fake/volumes/1',
-                        'rel': 'bookmark'
-                    }
-                ],
-            }
-        }
+        ex = {'volume': {'attachments': [{'device': '/',
+                         'host_name': None,
+                         'id': '1',
+                         'server_id': 'fakeuuid',
+                         'volume_id': '1'}],
+                         'availability_zone': 'nova',
+                         'bootable': 'false',
+                         'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                         'description': 'Volume Test Desc',
+                         'encrypted': False,
+                         'id': '1',
+                         'links':
+                         [{'href': 'http://localhost/v2/fake/volumes/1',
+                           'rel': 'self'},
+                          {'href': 'http://localhost/fake/volumes/1',
+                           'rel': 'bookmark'}],
+                         'metadata': {'attached_mode': 'rw',
+                                      'readonly': 'False'},
+                         'name': 'Volume Test Name',
+                         'size': '1',
+                         'snapshot_id': None,
+                         'source_volid': None,
+                         'status': 'fakestatus',
+                         'user_id': 'fakeuser',
+                         'volume_type': 'vol_type_name'}}
         body = {"volume": vol}
         req = fakes.HTTPRequest.blank('/v2/volumes')
         res_dict = self.controller.create(req, body)
-        self.assertEqual(res_dict, expected)
+        self.assertEqual(res_dict, ex)
 
     def test_volume_create_with_image_id_is_integer(self):
         self.stubs.Set(volume_api.API, "create", stubs.stub_volume_create)
@@ -240,11 +266,13 @@ class VolumeApiTest(test.TestCase):
         }
         body = {"volume": updates}
         req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 0)
         res_dict = self.controller.update(req, '1', body)
         expected = {
             'volume': {
                 'status': 'fakestatus',
                 'description': 'displaydesc',
+                'encrypted': False,
                 'availability_zone': 'fakeaz',
                 'bootable': 'false',
                 'name': 'Updated Test Name',
@@ -267,7 +295,7 @@ class VolumeApiTest(test.TestCase):
                 'size': 1,
                 'links': [
                     {
-                        'href': 'http://localhost/v1/fake/volumes/1',
+                        'href': 'http://localhost/v2/fake/volumes/1',
                         'rel': 'self'
                     },
                     {
@@ -278,6 +306,114 @@ class VolumeApiTest(test.TestCase):
             }
         }
         self.assertEqual(res_dict, expected)
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
+
+    def test_volume_update_deprecation(self):
+        self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
+        self.stubs.Set(volume_api.API, "update", stubs.stub_volume_update)
+
+        updates = {
+            "display_name": "Updated Test Name",
+            "display_description": "Updated Test Description",
+        }
+        body = {"volume": updates}
+        req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 0)
+        res_dict = self.controller.update(req, '1', body)
+        expected = {
+            'volume': {
+                'status': 'fakestatus',
+                'description': 'Updated Test Description',
+                'encrypted': False,
+                'availability_zone': 'fakeaz',
+                'bootable': 'false',
+                'name': 'Updated Test Name',
+                'attachments': [
+                    {
+                        'id': '1',
+                        'volume_id': '1',
+                        'server_id': 'fakeuuid',
+                        'host_name': None,
+                        'device': '/',
+                    }
+                ],
+                'user_id': 'fakeuser',
+                'volume_type': 'vol_type_name',
+                'snapshot_id': None,
+                'source_volid': None,
+                'metadata': {'attached_mode': 'rw', 'readonly': 'False'},
+                'id': '1',
+                'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                'size': 1,
+                'links': [
+                    {
+                        'href': 'http://localhost/v2/fake/volumes/1',
+                        'rel': 'self'
+                    },
+                    {
+                        'href': 'http://localhost/fake/volumes/1',
+                        'rel': 'bookmark'
+                    }
+                ],
+            }
+        }
+        self.assertEqual(res_dict, expected)
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
+
+    def test_volume_update_deprecation_key_priority(self):
+        """Test current update keys have priority over deprecated keys."""
+        self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
+        self.stubs.Set(volume_api.API, "update", stubs.stub_volume_update)
+
+        updates = {
+            "name": "New Name",
+            "description": "New Description",
+            "display_name": "Not Shown Name",
+            "display_description": "Not Shown Description",
+        }
+        body = {"volume": updates}
+        req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 0)
+        res_dict = self.controller.update(req, '1', body)
+        expected = {
+            'volume': {
+                'status': 'fakestatus',
+                'description': 'New Description',
+                'encrypted': False,
+                'availability_zone': 'fakeaz',
+                'bootable': 'false',
+                'name': 'New Name',
+                'attachments': [
+                    {
+                        'id': '1',
+                        'volume_id': '1',
+                        'server_id': 'fakeuuid',
+                        'host_name': None,
+                        'device': '/',
+                    }
+                ],
+                'user_id': 'fakeuser',
+                'volume_type': 'vol_type_name',
+                'snapshot_id': None,
+                'source_volid': None,
+                'metadata': {'attached_mode': 'rw', 'readonly': 'False'},
+                'id': '1',
+                'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+                'size': 1,
+                'links': [
+                    {
+                        'href': 'http://localhost/v2/fake/volumes/1',
+                        'rel': 'self'
+                    },
+                    {
+                        'href': 'http://localhost/fake/volumes/1',
+                        'rel': 'bookmark'
+                    }
+                ],
+            }
+        }
+        self.assertEqual(res_dict, expected)
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
 
     def test_volume_update_metadata(self):
         self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
@@ -288,10 +424,12 @@ class VolumeApiTest(test.TestCase):
         }
         body = {"volume": updates}
         req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 0)
         res_dict = self.controller.update(req, '1', body)
         expected = {'volume': {
             'status': 'fakestatus',
             'description': 'displaydesc',
+            'encrypted': False,
             'availability_zone': 'fakeaz',
             'bootable': 'false',
             'name': 'displayname',
@@ -314,7 +452,7 @@ class VolumeApiTest(test.TestCase):
             'size': 1,
             'links': [
                 {
-                    'href': 'http://localhost/v1/fake/volumes/1',
+                    'href': 'http://localhost/v2/fake/volumes/1',
                     'rel': 'self'
                 },
                 {
@@ -324,6 +462,7 @@ class VolumeApiTest(test.TestCase):
             ],
         }}
         self.assertEqual(res_dict, expected)
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
 
     def test_volume_update_with_admin_metadata(self):
         self.stubs.Set(volume_api.API, "update", stubs.stub_volume_update)
@@ -340,19 +479,21 @@ class VolumeApiTest(test.TestCase):
                                         False)
 
         updates = {
-            "display_name": "Updated Test Name",
+            "name": "Updated Test Name",
         }
         body = {"volume": updates}
-        req = fakes.HTTPRequest.blank('/v1/volumes/1')
+        req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 0)
         admin_ctx = context.RequestContext('admin', 'fake', True)
         req.environ['cinder.context'] = admin_ctx
         res_dict = self.controller.update(req, '1', body)
         expected = {'volume': {
             'status': 'fakestatus',
             'description': 'displaydesc',
+            'encrypted': False,
             'availability_zone': 'fakeaz',
             'bootable': 'false',
-            'name': 'displayname',
+            'name': 'Updated Test Name',
             'attachments': [{
                 'id': '1',
                 'volume_id': '1',
@@ -371,7 +512,7 @@ class VolumeApiTest(test.TestCase):
             'size': 1,
             'links': [
                 {
-                    'href': 'http://localhost/v1/fake/volumes/1',
+                    'href': 'http://localhost/v2/fake/volumes/1',
                     'rel': 'self'
                 },
                 {
@@ -381,6 +522,7 @@ class VolumeApiTest(test.TestCase):
             ],
         }}
         self.assertEqual(res_dict, expected)
+        self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
 
     def test_update_empty_body(self):
         body = {}
@@ -423,7 +565,7 @@ class VolumeApiTest(test.TestCase):
                     'id': '1',
                     'links': [
                         {
-                            'href': 'http://localhost/v1/fake/volumes/1',
+                            'href': 'http://localhost/v2/fake/volumes/1',
                             'rel': 'self'
                         },
                         {
@@ -450,6 +592,7 @@ class VolumeApiTest(test.TestCase):
                 {
                     'status': 'fakestatus',
                     'description': 'displaydesc',
+                    'encrypted': False,
                     'availability_zone': 'fakeaz',
                     'bootable': 'false',
                     'name': 'displayname',
@@ -472,7 +615,7 @@ class VolumeApiTest(test.TestCase):
                     'size': 1,
                     'links': [
                         {
-                            'href': 'http://localhost/v1/fake/volumes/1',
+                            'href': 'http://localhost/v2/fake/volumes/1',
                             'rel': 'self'
                         },
                         {
@@ -508,6 +651,7 @@ class VolumeApiTest(test.TestCase):
                 {
                     'status': 'fakestatus',
                     'description': 'displaydesc',
+                    'encrypted': False,
                     'availability_zone': 'fakeaz',
                     'bootable': 'false',
                     'name': 'displayname',
@@ -530,7 +674,7 @@ class VolumeApiTest(test.TestCase):
                     'size': 1,
                     'links': [
                         {
-                            'href': 'http://localhost/v1/fakeproject'
+                            'href': 'http://localhost/v2/fakeproject'
                                     '/volumes/1',
                             'rel': 'self'
                         },
@@ -546,7 +690,7 @@ class VolumeApiTest(test.TestCase):
 
     def test_volume_index_with_marker(self):
         def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
+                                           sort_key, sort_dir, filters=None):
             return [
                 stubs.stub_volume(1, display_name='vol1'),
                 stubs.stub_volume(2, display_name='vol2'),
@@ -597,7 +741,7 @@ class VolumeApiTest(test.TestCase):
 
     def test_volume_index_limit_offset(self):
         def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
+                                           sort_key, sort_dir, filters=None):
             return [
                 stubs.stub_volume(1, display_name='vol1'),
                 stubs.stub_volume(2, display_name='vol2'),
@@ -624,7 +768,7 @@ class VolumeApiTest(test.TestCase):
 
     def test_volume_detail_with_marker(self):
         def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
+                                           sort_key, sort_dir, filters=None):
             return [
                 stubs.stub_volume(1, display_name='vol1'),
                 stubs.stub_volume(2, display_name='vol2'),
@@ -675,7 +819,7 @@ class VolumeApiTest(test.TestCase):
 
     def test_volume_detail_limit_offset(self):
         def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
+                                           sort_key, sort_dir, filters=None):
             return [
                 stubs.stub_volume(1, display_name='vol1'),
                 stubs.stub_volume(2, display_name='vol2'),
@@ -707,130 +851,155 @@ class VolumeApiTest(test.TestCase):
                           self.controller.index,
                           req)
 
-    def test_volume_list_by_name(self):
-        def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
-            return [
-                stubs.stub_volume(1, display_name='vol1'),
-                stubs.stub_volume(2, display_name='vol2'),
-                stubs.stub_volume(3, display_name='vol3'),
-            ]
-        self.stubs.Set(db, 'volume_get_all_by_project',
-                       stub_volume_get_all_by_project)
+    def test_volume_with_limit_zero(self):
+        def stub_volume_get_all(context, marker, limit,
+                                sort_key, sort_dir):
+            return []
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all)
+        req = fakes.HTTPRequest.blank('/v2/volumes?limit=0')
+        res_dict = self.controller.index(req)
+        expected = {'volumes': []}
+        self.assertEqual(res_dict, expected)
+
+    def test_volume_default_limit(self):
         self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
 
-        # no name filter
-        req = fakes.HTTPRequest.blank('/v2/volumes')
-        resp = self.controller.index(req)
-        self.assertEqual(len(resp['volumes']), 3)
-        # filter on name
-        req = fakes.HTTPRequest.blank('/v2/volumes?name=vol2')
+        # Number of volumes equals the max, include next link
+        def stub_volume_get_all(context, marker, limit,
+                                sort_key, sort_dir,
+                                filters=None):
+            vols = [stubs.stub_volume(i)
+                    for i in xrange(CONF.osapi_max_limit)]
+            if limit == None or limit >= len(vols):
+                return vols
+            return vols[:limit]
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all)
+        for key in ['volumes', 'volumes/detail']:
+            req = fakes.HTTPRequest.blank('/v2/%s?all_tenants=1' % key,
+                                          use_admin_context=True)
+            res_dict = self.controller.index(req)
+            self.assertEqual(len(res_dict['volumes']), CONF.osapi_max_limit)
+            volumes_links = res_dict['volumes_links']
+            self.assertEqual(volumes_links[0]['rel'], 'next')
+
+        # Number of volumes less then max, do not include
+        def stub_volume_get_all2(context, marker, limit,
+                                 sort_key, sort_dir,
+                                 filters=None):
+            vols = [stubs.stub_volume(i)
+                    for i in xrange(100)]
+            if limit == None or limit >= len(vols):
+                return vols
+            return vols[:limit]
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all2)
+        for key in ['volumes', 'volumes/detail']:
+            req = fakes.HTTPRequest.blank('/v2/%s?all_tenants=1' % key,
+                                          use_admin_context=True)
+            res_dict = self.controller.index(req)
+            self.assertEqual(len(res_dict['volumes']), 100)
+            self.assertFalse('volumes_links' in res_dict)
+
+        # Number of volumes more then the max, include next link
+        def stub_volume_get_all3(context, marker, limit,
+                                 sort_key, sort_dir,
+                                 filters=None):
+            vols = [stubs.stub_volume(i)
+                    for i in xrange(CONF.osapi_max_limit + 100)]
+            if limit == None or limit >= len(vols):
+                return vols
+            return vols[:limit]
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all3)
+        for key in ['volumes', 'volumes/detail']:
+            req = fakes.HTTPRequest.blank('/v2/%s?all_tenants=1' % key,
+                                          use_admin_context=True)
+            res_dict = self.controller.index(req)
+            self.assertEqual(len(res_dict['volumes']), CONF.osapi_max_limit)
+            volumes_links = res_dict['volumes_links']
+            self.assertEqual(volumes_links[0]['rel'], 'next')
+        # Pass a limit that is greater then the max and the total number of
+        # volumes, ensure only the maximum is returned and that the next
+        # link is present
+        for key in ['volumes', 'volumes/detail']:
+            req = fakes.HTTPRequest.blank('/v2/%s?all_tenants=1&limit=%d'
+                                          % (key, CONF.osapi_max_limit * 2),
+                                          use_admin_context=True)
+            res_dict = self.controller.index(req)
+            self.assertEqual(len(res_dict['volumes']), CONF.osapi_max_limit)
+            volumes_links = res_dict['volumes_links']
+            self.assertEqual(volumes_links[0]['rel'], 'next')
+
+    def test_volume_list_default_filters(self):
+        """Tests that the default filters from volume.api.API.get_all are set.
+
+        1. 'no_migration_status'=True for non-admins and get_all_by_project is
+        invoked.
+        2. 'no_migration_status' is not included for admins.
+        3. When 'all_tenants' is not specified, then it is removed and
+        get_all_by_project is invoked for admins.
+        3. When 'all_tenants' is specified, then it is removed and get_all
+        is invoked for admins.
+        """
+        # Non-admin, project function should be called with no_migration_status
+        def stub_volume_get_all_by_project(context, project_id, marker, limit,
+                                           sort_key, sort_dir, filters=None):
+            self.assertEqual(filters['no_migration_targets'], True)
+            self.assertFalse('all_tenants' in filters)
+            return [stubs.stub_volume(1, display_name='vol1')]
+
+        def stub_volume_get_all(context, marker, limit,
+                                sort_key, sort_dir, filters=None):
+            return []
+        self.stubs.Set(db, 'volume_get_all_by_project',
+                       stub_volume_get_all_by_project)
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all)
+
+        # all_tenants does not matter for non-admin
+        for params in ['', '?all_tenants=1']:
+            req = fakes.HTTPRequest.blank('/v2/volumes%s' % params)
+            resp = self.controller.index(req)
+            self.assertEqual(len(resp['volumes']), 1)
+            self.assertEqual(resp['volumes'][0]['name'], 'vol1')
+
+        # Admin, all_tenants is not set, project function should be called
+        # without no_migration_status
+        def stub_volume_get_all_by_project2(context, project_id, marker, limit,
+                                            sort_key, sort_dir, filters=None):
+            self.assertFalse('no_migration_targets' in filters)
+            return [stubs.stub_volume(1, display_name='vol2')]
+
+        def stub_volume_get_all2(context, marker, limit,
+                                 sort_key, sort_dir, filters=None):
+            return []
+        self.stubs.Set(db, 'volume_get_all_by_project',
+                       stub_volume_get_all_by_project2)
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all2)
+
+        req = fakes.HTTPRequest.blank('/v2/volumes', use_admin_context=True)
         resp = self.controller.index(req)
         self.assertEqual(len(resp['volumes']), 1)
         self.assertEqual(resp['volumes'][0]['name'], 'vol2')
-        # filter no match
-        req = fakes.HTTPRequest.blank('/v2/volumes?name=vol4')
-        resp = self.controller.index(req)
-        self.assertEqual(len(resp['volumes']), 0)
 
-    def test_volume_list_by_metadata(self):
-        def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
-            return [
-                stubs.stub_volume(1, display_name='vol1',
-                                  status='available',
-                                  volume_metadata=[{'key': 'key1',
-                                                    'value': 'value1'}]),
-                stubs.stub_volume(2, display_name='vol2',
-                                  status='available',
-                                  volume_metadata=[{'key': 'key1',
-                                                    'value': 'value2'}]),
-                stubs.stub_volume(3, display_name='vol3',
-                                  status='in-use',
-                                  volume_metadata=[{'key': 'key1',
-                                                    'value': 'value2'}]),
-            ]
+        # Admin, all_tenants is set, get_all function should be called
+        # without no_migration_status
+        def stub_volume_get_all_by_project3(context, project_id, marker, limit,
+                                            sort_key, sort_dir, filters=None):
+            return []
+
+        def stub_volume_get_all3(context, marker, limit,
+                                 sort_key, sort_dir, filters=None):
+            self.assertFalse('no_migration_targets' in filters)
+            self.assertFalse('all_tenants' in filters)
+            return [stubs.stub_volume(1, display_name='vol3')]
         self.stubs.Set(db, 'volume_get_all_by_project',
-                       stub_volume_get_all_by_project)
+                       stub_volume_get_all_by_project3)
+        self.stubs.Set(db, 'volume_get_all', stub_volume_get_all3)
 
-        # no metadata filter
-        req = fakes.HTTPRequest.blank('/v2/volumes', use_admin_context=True)
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 3)
-
-        # single match
-        qparams = urllib.urlencode({'metadata': {'key1': 'value1'}})
-        req = fakes.HTTPRequest.blank('/v2/volumes?%s' % qparams,
+        req = fakes.HTTPRequest.blank('/v2/volumes?all_tenants=1',
                                       use_admin_context=True)
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 1)
-        self.assertEqual(resp['volumes'][0]['name'], 'vol1')
-        self.assertEqual(resp['volumes'][0]['metadata']['key1'], 'value1')
-
-        # multiple matches
-        qparams = urllib.urlencode({'metadata': {'key1': 'value2'}})
-        req = fakes.HTTPRequest.blank('/v2/volumes?%s' % qparams,
-                                      use_admin_context=True)
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 2)
-        for volume in resp['volumes']:
-            self.assertEqual(volume['metadata']['key1'], 'value2')
-
-        # multiple filters
-        qparams = urllib.urlencode({'metadata': {'key1': 'value2'}})
-        req = fakes.HTTPRequest.blank('/v2/volumes?status=in-use&%s' % qparams,
-                                      use_admin_context=True)
-        resp = self.controller.detail(req)
+        resp = self.controller.index(req)
         self.assertEqual(len(resp['volumes']), 1)
         self.assertEqual(resp['volumes'][0]['name'], 'vol3')
 
-        # no match
-        qparams = urllib.urlencode({'metadata': {'key1': 'value3'}})
-        req = fakes.HTTPRequest.blank('/v2/volumes?%s' % qparams,
-                                      use_admin_context=True)
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 0)
-
-    def test_volume_list_by_status(self):
-        def stub_volume_get_all_by_project(context, project_id, marker, limit,
-                                           sort_key, sort_dir):
-            return [
-                stubs.stub_volume(1, display_name='vol1', status='available'),
-                stubs.stub_volume(2, display_name='vol2', status='available'),
-                stubs.stub_volume(3, display_name='vol3', status='in-use'),
-            ]
-        self.stubs.Set(db, 'volume_get_all_by_project',
-                       stub_volume_get_all_by_project)
-        self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
-
-        # no status filter
-        req = fakes.HTTPRequest.blank('/v2/volumes/details')
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 3)
-        # single match
-        req = fakes.HTTPRequest.blank('/v2/volumes/details?status=in-use')
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 1)
-        self.assertEqual(resp['volumes'][0]['status'], 'in-use')
-        # multiple match
-        req = fakes.HTTPRequest.blank('/v2/volumes/details/?status=available')
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 2)
-        for volume in resp['volumes']:
-            self.assertEqual(volume['status'], 'available')
-        # multiple filters
-        req = fakes.HTTPRequest.blank('/v2/volumes/details/?status=available&'
-                                      'name=vol1')
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 1)
-        self.assertEqual(resp['volumes'][0]['name'], 'vol1')
-        self.assertEqual(resp['volumes'][0]['status'], 'available')
-        # no match
-        req = fakes.HTTPRequest.blank('/v2/volumes/details?status=in-use&'
-                                      'name=vol1')
-        resp = self.controller.detail(req)
-        self.assertEqual(len(resp['volumes']), 0)
-
     def test_volume_show(self):
         self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
 
@@ -840,6 +1009,7 @@ class VolumeApiTest(test.TestCase):
             'volume': {
                 'status': 'fakestatus',
                 'description': 'displaydesc',
+                'encrypted': False,
                 'availability_zone': 'fakeaz',
                 'bootable': 'false',
                 'name': 'displayname',
@@ -862,7 +1032,7 @@ class VolumeApiTest(test.TestCase):
                 'size': 1,
                 'links': [
                     {
-                        'href': 'http://localhost/v1/fake/volumes/1',
+                        'href': 'http://localhost/v2/fake/volumes/1',
                         'rel': 'self'
                     },
                     {
@@ -888,6 +1058,7 @@ class VolumeApiTest(test.TestCase):
             'volume': {
                 'status': 'fakestatus',
                 'description': 'displaydesc',
+                'encrypted': False,
                 'availability_zone': 'fakeaz',
                 'bootable': 'false',
                 'name': 'displayname',
@@ -902,7 +1073,7 @@ class VolumeApiTest(test.TestCase):
                 'size': 1,
                 'links': [
                     {
-                        'href': 'http://localhost/v1/fake/volumes/1',
+                        'href': 'http://localhost/v2/fake/volumes/1',
                         'rel': 'self'
                     },
                     {
@@ -944,6 +1115,7 @@ class VolumeApiTest(test.TestCase):
             'volume': {
                 'status': 'fakestatus',
                 'description': 'displaydesc',
+                'encrypted': False,
                 'availability_zone': 'fakeaz',
                 'bootable': 'false',
                 'name': 'displayname',
@@ -967,7 +1139,7 @@ class VolumeApiTest(test.TestCase):
                 'size': 1,
                 'links': [
                     {
-                        'href': 'http://localhost/v1/fakeproject/volumes/1',
+                        'href': 'http://localhost/v2/fakeproject/volumes/1',
                         'rel': 'self'
                     },
                     {
@@ -979,6 +1151,26 @@ class VolumeApiTest(test.TestCase):
         }
         self.assertEqual(res_dict, expected)
 
+    def test_volume_show_with_encrypted_volume(self):
+        def stub_volume_get(self, context, volume_id):
+            return stubs.stub_volume(volume_id, encryption_key_id='fake_id')
+
+        self.stubs.Set(volume_api.API, 'get', stub_volume_get)
+
+        req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        res_dict = self.controller.show(req, 1)
+        self.assertEqual(res_dict['volume']['encrypted'], True)
+
+    def test_volume_show_with_unencrypted_volume(self):
+        def stub_volume_get(self, context, volume_id):
+            return stubs.stub_volume(volume_id, encryption_key_id=None)
+
+        self.stubs.Set(volume_api.API, 'get', stub_volume_get)
+
+        req = fakes.HTTPRequest.blank('/v2/volumes/1')
+        res_dict = self.controller.show(req, 1)
+        self.assertEqual(res_dict['volume']['encrypted'], False)
+
     def test_volume_delete(self):
         self.stubs.Set(volume_api.API, 'get', stubs.stub_volume_get)
 
@@ -1071,8 +1263,9 @@ class VolumeApiTest(test.TestCase):
         volume = dict(volume_admin_metadata=admin_metadata,
                       volume_metadata=metadata)
         admin_ctx = context.get_admin_context()
-        self.controller._add_visible_admin_metadata(admin_ctx,
-                                                    volume)
+        utils.add_visible_admin_metadata(admin_ctx, volume,
+                                         self.controller.volume_api)
+
         self.assertEqual(volume['volume_metadata'],
                          [{"key": "key", "value": "value"},
                           {"key": "readonly", "value": "visible"},
@@ -1085,8 +1278,8 @@ class VolumeApiTest(test.TestCase):
         volume = dict(admin_metadata=admin_metadata,
                       metadata=metadata)
         admin_ctx = context.get_admin_context()
-        self.controller._add_visible_admin_metadata(admin_ctx,
-                                                    volume)
+        utils.add_visible_admin_metadata(admin_ctx, volume,
+                                         self.controller.volume_api)
         self.assertEqual(volume['metadata'],
                          {'key': 'value',
                           'attached_mode': 'visible',
