@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,9 +21,6 @@ import mox
 from cinder.brick import exception
 from cinder.brick.initiator import connector
 from cinder.brick.initiator import host_driver
-from cinder.brick.initiator import linuxfc
-from cinder.brick.initiator import linuxscsi
-from cinder.brick.remotefs import remotefs
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import loopingcall
 from cinder.openstack.common import processutils as putils
@@ -158,7 +153,7 @@ class ISCSIConnectorTestCase(ConnectorTestCase):
 
         self.stubs.Set(self.connector, '_execute', initiator_no_file)
         initiator = self.connector.get_initiator()
-        self.assertEqual(initiator, None)
+        self.assertIsNone(initiator)
         self.stubs.Set(self.connector, '_execute', initiator_get_text)
         initiator = self.connector.get_initiator()
         self.assertEqual(initiator, 'iqn.1234-56.foo.bar:01:23456789abc')
@@ -184,8 +179,8 @@ class ISCSIConnectorTestCase(ConnectorTestCase):
                              ('iscsiadm -m node -T %s -p %s --login' %
                               (iqn, location)),
                              ('iscsiadm -m node -T %s -p %s --op update'
-                              ' -n node.startup -v automatic' % (iqn,
-                              location)),
+                              ' -n node.startup -v automatic'
+                              % (iqn, location)),
                              ('iscsiadm -m node --rescan'),
                              ('iscsiadm -m session --rescan'),
                              ('tee -a /sys/block/sdb/device/delete'),
@@ -584,7 +579,8 @@ class RemoteFsConnectorTestCase(ConnectorTestCase):
             'export': self.TEST_DEV,
             'name': '9c592d52-ce47-4263-8c21-4ecf3c029cdb'}
         self.connector = connector.RemoteFsConnector(
-            'nfs', root_helper='sudo', nfs_mount_point_base='/mnt/test')
+            'nfs', root_helper='sudo', nfs_mount_point_base='/mnt/test',
+            nfs_mount_options='vers=3')
 
     def tearDown(self):
         self.mox.VerifyAll()
@@ -599,7 +595,7 @@ class RemoteFsConnectorTestCase(ConnectorTestCase):
                         check_exit_code=0).AndReturn(("", ""))
         client._execute('mkdir', '-p', self.TEST_PATH,
                         check_exit_code=0).AndReturn(("", ""))
-        client._execute('mount', '-t', 'nfs',
+        client._execute('mount', '-t', 'nfs', '-o', 'vers=3',
                         self.TEST_DEV, self.TEST_PATH,
                         root_helper='sudo', run_as_root=True,
                         check_exit_code=0).AndReturn(("", ""))
@@ -623,8 +619,8 @@ class LocalConnectorTestCase(test.TestCase):
         self.connector = connector.LocalConnector(None)
         cprops = self.connection_properties
         dev_info = self.connector.connect_volume(cprops)
-        self.assertTrue(dev_info['type'] == 'local')
-        self.assertTrue(dev_info['path'] == cprops['device_path'])
+        self.assertEqual(dev_info['type'], 'local')
+        self.assertEqual(dev_info['path'], cprops['device_path'])
 
     def test_connect_volume_with_invalid_connection_data(self):
         self.connector = connector.LocalConnector(None)
