@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -21,7 +19,6 @@ import datetime
 import glanceclient.exc
 import glanceclient.v2.client
 from glanceclient.v2.client import Client as glanceclient_v2
-import mock
 from oslo.config import cfg
 
 from cinder import context
@@ -200,7 +197,7 @@ class TestGlanceImageService(test.TestCase):
         num_images = len(self.service.detail(self.context))
         image_id = self.service.create(self.context, fixture)['id']
 
-        self.assertNotEquals(None, image_id)
+        self.assertIsNotNone(image_id)
         self.assertEqual(num_images + 1,
                          len(self.service.detail(self.context)))
 
@@ -208,7 +205,7 @@ class TestGlanceImageService(test.TestCase):
         fixture = self._make_fixture(name='test image')
         image_id = self.service.create(self.context, fixture)['id']
 
-        self.assertNotEquals(None, image_id)
+        self.assertIsNotNone(image_id)
         self.assertRaises(exception.ImageNotFound,
                           self.service.show,
                           self.context,
@@ -548,24 +545,45 @@ class TestGlanceImageService(test.TestCase):
 
 class TestGlanceClientVersion(test.TestCase):
     """Tests the version of the glance client generated."""
+    def setUp(self):
+        super(TestGlanceClientVersion, self).setUp()
 
-    @mock.patch('cinder.image.glance.glanceclient.Client')
-    def test_glance_version_by_flag(self, _mockglanceclient):
+        def fake_get_model(self):
+            return
+
+        self.stubs.Set(glanceclient_v2, '_get_image_model',
+                       fake_get_model)
+
+        try:
+            self.stubs.Set(glanceclient_v2, '_get_member_model',
+                           fake_get_model)
+        except AttributeError:
+            # method requires stubbing only with newer glanceclients.
+            pass
+
+    def test_glance_version_by_flag(self):
         """Test glance version set by flag is honoured."""
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292)
-        self.assertEqual('1', _mockglanceclient.call_args[0][0])
+        client_wrapper_v1 = glance.GlanceClientWrapper('fake', 'fake_host',
+                                                       9292)
+        self.assertEqual(client_wrapper_v1.client.__module__,
+                         'glanceclient.v1.client')
         self.flags(glance_api_version=2)
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292)
-        self.assertEqual('2', _mockglanceclient.call_args[0][0])
+        client_wrapper_v2 = glance.GlanceClientWrapper('fake', 'fake_host',
+                                                       9292)
+        self.assertEqual(client_wrapper_v2.client.__module__,
+                         'glanceclient.v2.client')
         CONF.reset()
 
-    @mock.patch('cinder.image.glance.glanceclient.Client')
-    def test_glance_version_by_arg(self, _mockglanceclient):
+    def test_glance_version_by_arg(self):
         """Test glance version set by arg to GlanceClientWrapper"""
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292, version=1)
-        self.assertEqual('1', _mockglanceclient.call_args[0][0])
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292, version=2)
-        self.assertEqual('2', _mockglanceclient.call_args[0][0])
+        client_wrapper_v1 = glance.GlanceClientWrapper('fake', 'fake_host',
+                                                       9292, version=1)
+        self.assertEqual(client_wrapper_v1.client.__module__,
+                         'glanceclient.v1.client')
+        client_wrapper_v2 = glance.GlanceClientWrapper('fake', 'fake_host',
+                                                       9292, version=2)
+        self.assertEqual(client_wrapper_v2.client.__module__,
+                         'glanceclient.v2.client')
 
 
 def _create_failing_glance_client(info):
