@@ -17,18 +17,22 @@ Tests for Backup code.
 
 """
 
+import mock
 import tempfile
 
+from oslo.config import cfg
+
 from cinder import context
 from cinder import db
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import importutils
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import timeutils
 from cinder import test
 
-FLAGS = flags.FLAGS
+
+CONF = cfg.CONF
+
 LOG = logging.getLogger(__name__)
 
 
@@ -42,12 +46,12 @@ class BackupTestCase(test.TestCase):
     def setUp(self):
         super(BackupTestCase, self).setUp()
         vol_tmpdir = tempfile.mkdtemp()
-        self.flags(connection_type='fake',
-                   volumes_dir=vol_tmpdir)
+        self.flags(volumes_dir=vol_tmpdir)
         self.backup_mgr = \
-            importutils.import_object(FLAGS.backup_manager)
+            importutils.import_object(CONF.backup_manager)
         self.backup_mgr.host = 'testhost'
         self.ctxt = context.get_admin_context()
+        self.backup_mgr.driver.set_initialized()
 
     def tearDown(self):
         super(BackupTestCase, self).tearDown()
@@ -56,11 +60,11 @@ class BackupTestCase(test.TestCase):
                                 display_description='this is a test backup',
                                 container='volumebackups',
                                 status='creating',
-                                size=0,
+                                size=1,
                                 object_count=0,
                                 project_id='fake'):
-        """
-        Create a backup entry in the DB.
+        """Create a backup entry in the DB.
+
         Return the entry ID
         """
         backup = {}
@@ -74,7 +78,7 @@ class BackupTestCase(test.TestCase):
         backup['container'] = container
         backup['status'] = status
         backup['fail_reason'] = ''
-        backup['service'] = FLAGS.backup_service
+        backup['service'] = CONF.backup_driver
         backup['size'] = size
         backup['object_count'] = object_count
         return db.backup_create(self.ctxt, backup)['id']
@@ -83,8 +87,8 @@ class BackupTestCase(test.TestCase):
                                 display_description='this is a test volume',
                                 status='backing-up',
                                 size=1):
-        """
-        Create a volume entry in the DB.
+        """Create a volume entry in the DB.
+
         Return the entry ID
         """
         vol = {}
@@ -98,9 +102,35 @@ class BackupTestCase(test.TestCase):
         vol['attach_status'] = 'detached'
         return db.volume_create(self.ctxt, vol)['id']
 
+    def _create_exported_record_entry(self, vol_size=1):
+        """Create backup metadata export entry."""
+        vol_id = self._create_volume_db_entry(status='available',
+                                              size=vol_size)
+        backup_id = self._create_backup_db_entry(status='available',
+                                                 volume_id=vol_id)
+
+        export = self.backup_mgr.export_record(self.ctxt, backup_id)
+        return export
+
+    def _create_export_record_db_entry(self,
+                                       volume_id='0000',
+                                       status='creating',
+                                       project_id='fake'):
+        """Create a backup entry in the DB.
+
+        Return the entry ID
+        """
+        backup = {}
+        backup['volume_id'] = volume_id
+        backup['user_id'] = 'fake'
+        backup['project_id'] = project_id
+        backup['status'] = status
+        return db.backup_create(self.ctxt, backup)['id']
+
     def test_init_host(self):
         """Make sure stuck volumes and backups are reset to correct
-        states when backup_manager.init_host() is called"""
+        states when backup_manager.init_host() is called
+        """
         vol1_id = self._create_volume_db_entry(status='backing-up')
         vol2_id = self._create_volume_db_entry(status='restoring-backup')
         backup1_id = self._create_backup_db_entry(status='creating')
@@ -109,14 +139,14 @@ class BackupTestCase(test.TestCase):
 
         self.backup_mgr.init_host()
         vol1 = db.volume_get(self.ctxt, vol1_id)
-        self.assertEquals(vol1['status'], 'available')
+        self.assertEqual(vol1['status'], 'available')
         vol2 = db.volume_get(self.ctxt, vol2_id)
-        self.assertEquals(vol2['status'], 'error_restoring')
+        self.assertEqual(vol2['status'], 'error_restoring')
 
         backup1 = db.backup_get(self.ctxt, backup1_id)
-        self.assertEquals(backup1['status'], 'error')
+        self.assertEqual(backup1['status'], 'error')
         backup2 = db.backup_get(self.ctxt, backup2_id)
-        self.assertEquals(backup2['status'], 'available')
+        self.assertEqual(backup2['status'], 'available')
         self.assertRaises(exception.BackupNotFound,
                           db.backup_get,
                           self.ctxt,
@@ -124,7 +154,8 @@ class BackupTestCase(test.TestCase):
 
     def test_create_backup_with_bad_volume_status(self):
         """Test error handling when creating a backup from a volume
-        with a bad status"""
+        with a bad status
+        """
         vol_id = self._create_volume_db_entry(status='available', size=1)
         backup_id = self._create_backup_db_entry(volume_id=vol_id)
         self.assertRaises(exception.InvalidVolume,
@@ -134,7 +165,8 @@ class BackupTestCase(test.TestCase):
 
     def test_create_backup_with_bad_backup_status(self):
         """Test error handling when creating a backup with a backup
-        with a bad status"""
+        with a bad status
+        """
         vol_id = self._create_volume_db_entry(size=1)
         backup_id = self._create_backup_db_entry(status='available',
                                                  volume_id=vol_id)
@@ -143,48 +175,42 @@ class BackupTestCase(test.TestCase):
                           self.ctxt,
                           backup_id)
 
-    def test_create_backup_with_error(self):
-        """Test error handling when an error occurs during backup creation"""
+    @mock.patch('%s.%s' % (CONF.volume_driver, 'backup_volume'))
+    def test_create_backup_with_error(self, _mock_volume_backup):
+        """Test error handling when error occurs during backup creation."""
         vol_id = self._create_volume_db_entry(size=1)
         backup_id = self._create_backup_db_entry(volume_id=vol_id)
 
-        def fake_backup_volume(context, backup, backup_service):
-            raise FakeBackupException('fake')
-
-        self.stubs.Set(self.backup_mgr.driver, 'backup_volume',
-                       fake_backup_volume)
-
+        _mock_volume_backup.side_effect = FakeBackupException('fake')
         self.assertRaises(FakeBackupException,
                           self.backup_mgr.create_backup,
                           self.ctxt,
                           backup_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'available')
+        self.assertEqual(vol['status'], 'available')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'error')
+        self.assertEqual(backup['status'], 'error')
+        self.assertTrue(_mock_volume_backup.called)
 
-    def test_create_backup(self):
-        """Test normal backup creation"""
+    @mock.patch('%s.%s' % (CONF.volume_driver, 'backup_volume'))
+    def test_create_backup(self, _mock_volume_backup):
+        """Test normal backup creation."""
         vol_size = 1
         vol_id = self._create_volume_db_entry(size=vol_size)
         backup_id = self._create_backup_db_entry(volume_id=vol_id)
 
-        def fake_backup_volume(context, backup, backup_service):
-            pass
-
-        self.stubs.Set(self.backup_mgr.driver, 'backup_volume',
-                       fake_backup_volume)
-
         self.backup_mgr.create_backup(self.ctxt, backup_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'available')
+        self.assertEqual(vol['status'], 'available')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'available')
         self.assertEqual(backup['size'], vol_size)
+        self.assertTrue(_mock_volume_backup.called)
 
     def test_restore_backup_with_bad_volume_status(self):
         """Test error handling when restoring a backup to a volume
-        with a bad status"""
+        with a bad status.
+        """
         vol_id = self._create_volume_db_entry(status='available', size=1)
         backup_id = self._create_backup_db_entry(volume_id=vol_id)
         self.assertRaises(exception.InvalidVolume,
@@ -193,11 +219,12 @@ class BackupTestCase(test.TestCase):
                           backup_id,
                           vol_id)
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'available')
 
     def test_restore_backup_with_bad_backup_status(self):
         """Test error handling when restoring a backup with a backup
-        with a bad status"""
+        with a bad status.
+        """
         vol_id = self._create_volume_db_entry(status='restoring-backup',
                                               size=1)
         backup_id = self._create_backup_db_entry(status='available',
@@ -208,47 +235,39 @@ class BackupTestCase(test.TestCase):
                           backup_id,
                           vol_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'error')
+        self.assertEqual(vol['status'], 'error')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'error')
+        self.assertEqual(backup['status'], 'error')
 
-    def test_restore_backup_with_driver_error(self):
-        """Test error handling when an error occurs during backup restore"""
+    @mock.patch('%s.%s' % (CONF.volume_driver, 'restore_backup'))
+    def test_restore_backup_with_driver_error(self, _mock_volume_restore):
+        """Test error handling when an error occurs during backup restore."""
         vol_id = self._create_volume_db_entry(status='restoring-backup',
                                               size=1)
         backup_id = self._create_backup_db_entry(status='restoring',
                                                  volume_id=vol_id)
 
-        def fake_restore_backup(context, backup, volume, backup_service):
-            raise FakeBackupException('fake')
-
-        self.stubs.Set(self.backup_mgr.driver, 'restore_backup',
-                       fake_restore_backup)
-
+        _mock_volume_restore.side_effect = FakeBackupException('fake')
         self.assertRaises(FakeBackupException,
                           self.backup_mgr.restore_backup,
                           self.ctxt,
                           backup_id,
                           vol_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'error_restoring')
+        self.assertEqual(vol['status'], 'error_restoring')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'available')
+        self.assertTrue(_mock_volume_restore.called)
 
     def test_restore_backup_with_bad_service(self):
         """Test error handling when attempting a restore of a backup
-        with a different service to that used to create the backup"""
+        with a different service to that used to create the backup.
+        """
         vol_id = self._create_volume_db_entry(status='restoring-backup',
                                               size=1)
         backup_id = self._create_backup_db_entry(status='restoring',
                                                  volume_id=vol_id)
 
-        def fake_restore_backup(context, backup, volume, backup_service):
-            pass
-
-        self.stubs.Set(self.backup_mgr.driver, 'restore_backup',
-                       fake_restore_backup)
-
         service = 'cinder.tests.backup.bad_service'
         db.backup_update(self.ctxt, backup_id, {'service': service})
         self.assertRaises(exception.InvalidBackup,
@@ -257,33 +276,30 @@ class BackupTestCase(test.TestCase):
                           backup_id,
                           vol_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'error')
+        self.assertEqual(vol['status'], 'error')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'available')
 
-    def test_restore_backup(self):
-        """Test normal backup restoration"""
+    @mock.patch('%s.%s' % (CONF.volume_driver, 'restore_backup'))
+    def test_restore_backup(self, _mock_volume_restore):
+        """Test normal backup restoration."""
         vol_size = 1
         vol_id = self._create_volume_db_entry(status='restoring-backup',
                                               size=vol_size)
         backup_id = self._create_backup_db_entry(status='restoring',
                                                  volume_id=vol_id)
 
-        def fake_restore_backup(context, backup, volume, backup_service):
-            pass
-
-        self.stubs.Set(self.backup_mgr.driver, 'restore_backup',
-                       fake_restore_backup)
-
         self.backup_mgr.restore_backup(self.ctxt, backup_id, vol_id)
         vol = db.volume_get(self.ctxt, vol_id)
-        self.assertEquals(vol['status'], 'available')
+        self.assertEqual(vol['status'], 'available')
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'available')
+        self.assertTrue(_mock_volume_restore.called)
 
     def test_delete_backup_with_bad_backup_status(self):
         """Test error handling when deleting a backup with a backup
-        with a bad status"""
+        with a bad status.
+        """
         vol_id = self._create_volume_db_entry(size=1)
         backup_id = self._create_backup_db_entry(status='available',
                                                  volume_id=vol_id)
@@ -292,7 +308,7 @@ class BackupTestCase(test.TestCase):
                           self.ctxt,
                           backup_id)
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'error')
+        self.assertEqual(backup['status'], 'error')
 
     def test_delete_backup_with_error(self):
         """Test error handling when an error occurs during backup deletion."""
@@ -305,11 +321,12 @@ class BackupTestCase(test.TestCase):
                           self.ctxt,
                           backup_id)
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'error')
+        self.assertEqual(backup['status'], 'error')
 
     def test_delete_backup_with_bad_service(self):
         """Test error handling when attempting a delete of a backup
-        with a different service to that used to create the backup"""
+        with a different service to that used to create the backup.
+        """
         vol_id = self._create_volume_db_entry(size=1)
         backup_id = self._create_backup_db_entry(status='deleting',
                                                  volume_id=vol_id)
@@ -320,10 +337,20 @@ class BackupTestCase(test.TestCase):
                           self.ctxt,
                           backup_id)
         backup = db.backup_get(self.ctxt, backup_id)
-        self.assertEquals(backup['status'], 'available')
+        self.assertEqual(backup['status'], 'error')
+
+    def test_delete_backup_with_no_service(self):
+        """Test error handling when attempting a delete of a backup
+        with no service defined for that backup, relates to bug #1162908
+        """
+        vol_id = self._create_volume_db_entry(size=1)
+        backup_id = self._create_backup_db_entry(status='deleting',
+                                                 volume_id=vol_id)
+        db.backup_update(self.ctxt, backup_id, {'service': None})
+        self.backup_mgr.delete_backup(self.ctxt, backup_id)
 
     def test_delete_backup(self):
-        """Test normal backup deletion"""
+        """Test normal backup deletion."""
         vol_id = self._create_volume_db_entry(size=1)
         backup_id = self._create_backup_db_entry(status='deleting',
                                                  volume_id=vol_id)
@@ -336,7 +363,7 @@ class BackupTestCase(test.TestCase):
         ctxt_read_deleted = context.get_admin_context('yes')
         backup = db.backup_get(ctxt_read_deleted, backup_id)
         self.assertEqual(backup.deleted, True)
-        self.assertTrue(timeutils.utcnow() > backup.deleted_at)
+        self.assertGreaterEqual(timeutils.utcnow(), backup.deleted_at)
         self.assertEqual(backup.status, 'deleted')
 
     def test_list_backup(self):
@@ -351,7 +378,8 @@ class BackupTestCase(test.TestCase):
 
     def test_backup_get_all_by_project_with_deleted(self):
         """Test deleted backups don't show up in backup_get_all_by_project.
-           Unless context.read_deleted is 'yes'"""
+           Unless context.read_deleted is 'yes'.
+        """
         backups = db.backup_get_all_by_project(self.ctxt, 'fake')
         self.assertEqual(len(backups), 0)
 
@@ -369,7 +397,8 @@ class BackupTestCase(test.TestCase):
 
     def test_backup_get_all_by_host_with_deleted(self):
         """Test deleted backups don't show up in backup_get_all_by_project.
-           Unless context.read_deleted is 'yes'"""
+           Unless context.read_deleted is 'yes'
+        """
         backups = db.backup_get_all_by_host(self.ctxt, 'testhost')
         self.assertEqual(len(backups), 0)
 
@@ -384,3 +413,187 @@ class BackupTestCase(test.TestCase):
         ctxt_read_deleted = context.get_admin_context('yes')
         backups = db.backup_get_all_by_host(ctxt_read_deleted, 'testhost')
         self.assertEqual(len(backups), 2)
+
+    def test_backup_manager_driver_name(self):
+        """"Test mapping between backup services and backup drivers."""
+
+        old_setting = CONF.backup_driver
+        setattr(cfg.CONF, 'backup_driver', "cinder.backup.services.swift")
+        backup_mgr = \
+            importutils.import_object(CONF.backup_manager)
+        self.assertEqual('cinder.backup.drivers.swift',
+                         backup_mgr.driver_name)
+        setattr(cfg.CONF, 'backup_driver', old_setting)
+
+    def test_export_record_with_bad_service(self):
+        """Test error handling when attempting an export of a backup
+        record with a different service to that used to create the backup.
+        """
+        vol_id = self._create_volume_db_entry(size=1)
+        backup_id = self._create_backup_db_entry(status='available',
+                                                 volume_id=vol_id)
+        service = 'cinder.tests.backup.bad_service'
+        db.backup_update(self.ctxt, backup_id, {'service': service})
+        self.assertRaises(exception.InvalidBackup,
+                          self.backup_mgr.export_record,
+                          self.ctxt,
+                          backup_id)
+
+    def test_export_record_with_bad_backup_status(self):
+        """Test error handling when exporting a backup record with a backup
+        with a bad status.
+        """
+        vol_id = self._create_volume_db_entry(status='available',
+                                              size=1)
+        backup_id = self._create_backup_db_entry(status='error',
+                                                 volume_id=vol_id)
+        self.assertRaises(exception.InvalidBackup,
+                          self.backup_mgr.export_record,
+                          self.ctxt,
+                          backup_id)
+
+    def test_export_record(self):
+        """Test normal backup record export."""
+        vol_size = 1
+        vol_id = self._create_volume_db_entry(status='available',
+                                              size=vol_size)
+        backup_id = self._create_backup_db_entry(status='available',
+                                                 volume_id=vol_id)
+
+        export = self.backup_mgr.export_record(self.ctxt, backup_id)
+        self.assertEqual(export['backup_service'], CONF.backup_driver)
+        self.assertTrue('backup_url' in export)
+
+    def test_import_record_with_verify_not_implemented(self):
+        """Test normal backup record import.
+
+        Test the case when import succeeds for the case that the
+        driver does not support verify.
+        """
+        vol_size = 1
+        export = self._create_exported_record_entry(vol_size=vol_size)
+        imported_record = self._create_export_record_db_entry()
+        backup_hosts = []
+        backup_driver = self.backup_mgr.service.get_backup_driver(self.ctxt)
+        _mock_backup_verify_class = ('%s.%s.%s' %
+                                     (backup_driver.__module__,
+                                      backup_driver.__class__.__name__,
+                                      'verify'))
+        with mock.patch(_mock_backup_verify_class) as _mock_record_verify:
+            _mock_record_verify.side_effect = NotImplementedError()
+            self.backup_mgr.import_record(self.ctxt,
+                                          imported_record,
+                                          export['backup_service'],
+                                          export['backup_url'],
+                                          backup_hosts)
+        backup = db.backup_get(self.ctxt, imported_record)
+        self.assertEqual(backup['status'], 'available')
+        self.assertEqual(backup['size'], vol_size)
+
+    def test_import_record_with_verify(self):
+        """Test normal backup record import.
+
+        Test the case when import succeeds for the case that the
+        driver implements verify.
+        """
+        vol_size = 1
+        export = self._create_exported_record_entry(vol_size=vol_size)
+        imported_record = self._create_export_record_db_entry()
+        backup_hosts = []
+        backup_driver = self.backup_mgr.service.get_backup_driver(self.ctxt)
+        _mock_backup_verify_class = ('%s.%s.%s' %
+                                     (backup_driver.__module__,
+                                      backup_driver.__class__.__name__,
+                                      'verify'))
+        with mock.patch(_mock_backup_verify_class) as _mock_record_verify:
+            self.backup_mgr.import_record(self.ctxt,
+                                          imported_record,
+                                          export['backup_service'],
+                                          export['backup_url'],
+                                          backup_hosts)
+        backup = db.backup_get(self.ctxt, imported_record)
+        self.assertEqual(backup['status'], 'available')
+        self.assertEqual(backup['size'], vol_size)
+
+    def test_import_record_with_bad_service(self):
+        """Test error handling when attempting an import of a backup
+        record with a different service to that used to create the backup.
+        """
+        export = self._create_exported_record_entry()
+        export['backup_service'] = 'cinder.tests.backup.bad_service'
+        imported_record = self._create_export_record_db_entry()
+
+        #Test the case where the additional hosts list is empty
+        backup_hosts = []
+        self.assertRaises(exception.ServiceNotFound,
+                          self.backup_mgr.import_record,
+                          self.ctxt,
+                          imported_record,
+                          export['backup_service'],
+                          export['backup_url'],
+                          backup_hosts)
+
+        #Test that the import backup keeps calling other hosts to find a
+        #suitable host for the backup service
+        backup_hosts = ['fake1', 'fake2']
+        BackupAPI_import = 'cinder.backup.rpcapi.BackupAPI.import_record'
+        with mock.patch(BackupAPI_import) as _mock_backup_import:
+            self.backup_mgr.import_record(self.ctxt,
+                                          imported_record,
+                                          export['backup_service'],
+                                          export['backup_url'],
+                                          backup_hosts)
+            self.assertTrue(_mock_backup_import.called)
+
+    def test_import_record_with_invalid_backup(self):
+        """Test error handling when attempting an import of a backup
+        record where the backup driver returns an exception.
+        """
+        export = self._create_exported_record_entry()
+        backup_driver = self.backup_mgr.service.get_backup_driver(self.ctxt)
+        _mock_record_import_class = ('%s.%s.%s' %
+                                     (backup_driver.__module__,
+                                      backup_driver.__class__.__name__,
+                                      'import_record'))
+        imported_record = self._create_export_record_db_entry()
+        backup_hosts = []
+        with mock.patch(_mock_record_import_class) as _mock_record_import:
+            _mock_record_import.side_effect = FakeBackupException('fake')
+            self.assertRaises(exception.InvalidBackup,
+                              self.backup_mgr.import_record,
+                              self.ctxt,
+                              imported_record,
+                              export['backup_service'],
+                              export['backup_url'],
+                              backup_hosts)
+            self.assertTrue(_mock_record_import.called)
+        backup = db.backup_get(self.ctxt, imported_record)
+        self.assertEqual(backup['status'], 'error')
+
+    def test_import_record_with_verify_invalid_backup(self):
+        """Test error handling when attempting an import of a backup
+        record where the backup driver returns an exception.
+        """
+        vol_size = 1
+        export = self._create_exported_record_entry(vol_size=vol_size)
+        imported_record = self._create_export_record_db_entry()
+        backup_hosts = []
+        backup_driver = self.backup_mgr.service.get_backup_driver(self.ctxt)
+        _mock_backup_verify_class = ('%s.%s.%s' %
+                                     (backup_driver.__module__,
+                                      backup_driver.__class__.__name__,
+                                      'verify'))
+        with mock.patch(_mock_backup_verify_class) as _mock_record_verify:
+            _mock_record_verify.side_effect = \
+                exception.InvalidBackup(reason='fake')
+
+            self.assertRaises(exception.InvalidBackup,
+                              self.backup_mgr.import_record,
+                              self.ctxt,
+                              imported_record,
+                              export['backup_service'],
+                              export['backup_url'],
+                              backup_hosts)
+            self.assertTrue(_mock_record_verify.called)
+        backup = db.backup_get(self.ctxt, imported_record)
+        self.assertEqual(backup['status'], 'error')
