@@ -1,3 +1,4 @@
+#    Copyright 2014 IBM Corp.
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -68,15 +69,18 @@ class ModelsObjectComparatorMixin(object):
         obj1 = self._dict_from_object(obj1, ignored_keys)
         obj2 = self._dict_from_object(obj2, ignored_keys)
 
-        self.assertEqual(len(obj1), len(obj2))
+        self.assertEqual(
+            len(obj1), len(obj2),
+            "Keys mismatch: %s" % str(set(obj1.keys()) ^ set(obj2.keys())))
         for key, value in obj1.iteritems():
             self.assertEqual(value, obj2[key])
 
     def _assertEqualListsOfObjects(self, objs1, objs2, ignored_keys=None):
-        self.assertEqual(len(objs1), len(objs2))
-        objs2 = dict([(o['id'], o) for o in objs2])
-        for o1 in objs1:
-            self._assertEqualObjects(o1, objs2[o1['id']], ignored_keys)
+        obj_to_dict = lambda o: self._dict_from_object(o, ignored_keys)
+        sort_key = lambda d: [d[k] for k in sorted(d)]
+        conv_and_sort = lambda obj: sorted(map(obj_to_dict, obj), key=sort_key)
+
+        self.assertEqual(conv_and_sort(objs1), conv_and_sort(objs2))
 
     def _assertEqualListsOfPrimitivesAsSets(self, primitives1, primitives2):
         self.assertEqual(len(primitives1), len(primitives2))
@@ -252,7 +256,7 @@ class DBAPIVolumeTestCase(BaseTest):
         self.assertEqual(volume.host, 'host1')
 
     def test_volume_allocate_iscsi_target_no_more_targets(self):
-        self.assertRaises(db.NoMoreTargets,
+        self.assertRaises(exception.NoMoreTargets,
                           db.volume_allocate_iscsi_target,
                           self.ctxt, 42, 'host1')
 
@@ -279,7 +283,7 @@ class DBAPIVolumeTestCase(BaseTest):
         self.assertEqual(volume['mountpoint'], '/tmp')
         self.assertEqual(volume['attach_status'], 'attached')
         self.assertEqual(volume['instance_uuid'], instance_uuid)
-        self.assertEqual(volume['attached_host'], None)
+        self.assertIsNone(volume['attached_host'])
 
     def test_volume_attached_to_host(self):
         volume = db.volume_create(self.ctxt, {'host': 'host1'})
@@ -290,7 +294,7 @@ class DBAPIVolumeTestCase(BaseTest):
         self.assertEqual(volume['status'], 'in-use')
         self.assertEqual(volume['mountpoint'], '/tmp')
         self.assertEqual(volume['attach_status'], 'attached')
-        self.assertEqual(volume['instance_uuid'], None)
+        self.assertIsNone(volume['instance_uuid'])
         self.assertEqual(volume['attached_host'], host_name)
 
     def test_volume_data_get_for_host(self):
@@ -392,6 +396,10 @@ class DBAPIVolumeTestCase(BaseTest):
                                             db.volume_get_all_by_instance_uuid(
                                             self.ctxt, instance_uuids[i]))
 
+    def test_volume_get_all_by_instance_uuid_empty(self):
+        self.assertEqual([], db.volume_get_all_by_instance_uuid(self.ctxt,
+                                                                'empty'))
+
     def test_volume_get_all_by_project(self):
         volumes = []
         for i in xrange(3):
@@ -403,6 +411,257 @@ class DBAPIVolumeTestCase(BaseTest):
                                             self.ctxt, 'p%d' % i, None,
                                             None, 'host', None))
 
+    def test_volume_get_by_name(self):
+        db.volume_create(self.ctxt, {'display_name': 'vol1'})
+        db.volume_create(self.ctxt, {'display_name': 'vol2'})
+        db.volume_create(self.ctxt, {'display_name': 'vol3'})
+
+        # no name filter
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc')
+        self.assertEqual(len(volumes), 3)
+        # filter on name
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'display_name': 'vol2'})
+        self.assertEqual(len(volumes), 1)
+        self.assertEqual(volumes[0]['display_name'], 'vol2')
+        # filter no match
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'display_name': 'vol4'})
+        self.assertEqual(len(volumes), 0)
+
+    def test_volume_list_by_status(self):
+        db.volume_create(self.ctxt, {'display_name': 'vol1',
+                                     'status': 'available'})
+        db.volume_create(self.ctxt, {'display_name': 'vol2',
+                                     'status': 'available'})
+        db.volume_create(self.ctxt, {'display_name': 'vol3',
+                                     'status': 'in-use'})
+
+        # no status filter
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc')
+        self.assertEqual(len(volumes), 3)
+        # single match
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'status': 'in-use'})
+        self.assertEqual(len(volumes), 1)
+        self.assertEqual(volumes[0]['status'], 'in-use')
+        # multiple match
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'status': 'available'})
+        self.assertEqual(len(volumes), 2)
+        for volume in volumes:
+            self.assertEqual(volume['status'], 'available')
+        # multiple filters
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'status': 'available',
+                                            'display_name': 'vol1'})
+        self.assertEqual(len(volumes), 1)
+        self.assertEqual(volumes[0]['display_name'], 'vol1')
+        self.assertEqual(volumes[0]['status'], 'available')
+        # no match
+        volumes = db.volume_get_all(self.ctxt, None, None, 'created_at',
+                                    'asc', {'status': 'in-use',
+                                            'display_name': 'vol1'})
+        self.assertEqual(len(volumes), 0)
+
+    def _assertEqualsVolumeOrderResult(self, correct_order, limit=None,
+                                       sort_key='created_at', sort_dir='asc',
+                                       filters=None, project_id=None,
+                                       match_keys=['id', 'display_name',
+                                                   'volume_metadata',
+                                                   'created_at']):
+        """"Verifies that volumes are returned in the correct order."""
+        if project_id:
+            result = db.volume_get_all_by_project(self.ctxt, project_id, None,
+                                                  limit, sort_key,
+                                                  sort_dir, filters=filters)
+        else:
+            result = db.volume_get_all(self.ctxt, None, limit, sort_key,
+                                       sort_dir, filters=filters)
+        self.assertEqual(len(correct_order), len(result))
+        self.assertEqual(len(result), len(correct_order))
+        for vol1, vol2 in zip(result, correct_order):
+            for key in match_keys:
+                val1 = vol1.get(key)
+                val2 = vol2.get(key)
+                # metadata is a list, compare the 'key' and 'value' of each
+                if key == 'volume_metadata':
+                    self.assertEqual(len(val1), len(val2))
+                    for m1, m2 in zip(val1, val2):
+                        self.assertEqual(m1.get('key'), m2.get('key'))
+                        self.assertEqual(m1.get('value'), m2.get('value'))
+                else:
+                    self.assertEqual(val1, val2)
+
+    def test_volume_get_by_filter(self):
+        """Verifies that all filtering is done at the DB layer."""
+        vols = []
+        vols.extend([db.volume_create(self.ctxt,
+                                      {'project_id': 'g1',
+                                       'display_name': 'name_%d' % i,
+                                       'size': 1})
+                     for i in xrange(2)])
+        vols.extend([db.volume_create(self.ctxt,
+                                      {'project_id': 'g1',
+                                       'display_name': 'name_%d' % i,
+                                       'size': 2})
+                     for i in xrange(2)])
+        vols.extend([db.volume_create(self.ctxt,
+                                      {'project_id': 'g1',
+                                       'display_name': 'name_%d' % i})
+                     for i in xrange(2)])
+        vols.extend([db.volume_create(self.ctxt,
+                                      {'project_id': 'g2',
+                                       'display_name': 'name_%d' % i,
+                                       'size': 1})
+                     for i in xrange(2)])
+
+        # By project, filter on size and name
+        filters = {'size': '1'}
+        correct_order = [vols[0], vols[1]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters,
+                                            project_id='g1')
+        filters = {'size': '1', 'display_name': 'name_1'}
+        correct_order = [vols[1]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters,
+                                            project_id='g1')
+
+        # Remove project scope
+        filters = {'size': '1'}
+        correct_order = [vols[0], vols[1], vols[6], vols[7]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters)
+        filters = {'size': '1', 'display_name': 'name_1'}
+        correct_order = [vols[1], vols[7]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters)
+
+        # Remove size constraint
+        filters = {'display_name': 'name_1'}
+        correct_order = [vols[1], vols[3], vols[5]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters,
+                                            project_id='g1')
+        correct_order = [vols[1], vols[3], vols[5], vols[7]]
+        self._assertEqualsVolumeOrderResult(correct_order, filters=filters)
+
+        # Verify bogus values return nothing
+        filters = {'display_name': 'name_1', 'bogus_value': 'foo'}
+        self._assertEqualsVolumeOrderResult([], filters=filters,
+                                            project_id='g1')
+        self._assertEqualsVolumeOrderResult([], project_id='bogus')
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+        self._assertEqualsVolumeOrderResult([], filters={'metadata':
+                                                         'not valid'})
+        self._assertEqualsVolumeOrderResult([], filters={'metadata':
+                                                         ['not', 'valid']})
+
+        # Verify that relationship property keys return nothing, these
+        # exist on the Volumes model but are not columns
+        filters = {'volume_type': 'bogus_type'}
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+
+    def test_volume_get_all_filters_limit(self):
+        vol1 = db.volume_create(self.ctxt, {'display_name': 'test1'})
+        vol2 = db.volume_create(self.ctxt, {'display_name': 'test2'})
+        vol3 = db.volume_create(self.ctxt, {'display_name': 'test2',
+                                            'metadata': {'key1': 'val1'}})
+        vol4 = db.volume_create(self.ctxt, {'display_name': 'test3',
+                                            'metadata': {'key1': 'val1',
+                                                         'key2': 'val2'}})
+        vol5 = db.volume_create(self.ctxt, {'display_name': 'test3',
+                                            'metadata': {'key2': 'val2',
+                                                         'key3': 'val3'},
+                                            'host': 'host5'})
+        vols = [vol1, vol2, vol3, vol4, vol5]
+
+        # Ensure we have 5 total instances
+        self._assertEqualsVolumeOrderResult(vols)
+
+        # No filters, test limit
+        self._assertEqualsVolumeOrderResult(vols[:1], limit=1)
+        self._assertEqualsVolumeOrderResult(vols[:4], limit=4)
+
+        # Just the test2 volumes
+        filters = {'display_name': 'test2'}
+        self._assertEqualsVolumeOrderResult([vol2, vol3], filters=filters)
+        self._assertEqualsVolumeOrderResult([vol2], limit=1,
+                                            filters=filters)
+        self._assertEqualsVolumeOrderResult([vol2, vol3], limit=2,
+                                            filters=filters)
+        self._assertEqualsVolumeOrderResult([vol2, vol3], limit=100,
+                                            filters=filters)
+
+        # metdata filters
+        filters = {'metadata': {'key1': 'val1'}}
+        self._assertEqualsVolumeOrderResult([vol3, vol4], filters=filters)
+        self._assertEqualsVolumeOrderResult([vol3], limit=1,
+                                            filters=filters)
+        self._assertEqualsVolumeOrderResult([vol3, vol4], limit=10,
+                                            filters=filters)
+
+        filters = {'metadata': {'key1': 'val1',
+                                'key2': 'val2'}}
+        self._assertEqualsVolumeOrderResult([vol4], filters=filters)
+        self._assertEqualsVolumeOrderResult([vol4], limit=1,
+                                            filters=filters)
+
+        # No match
+        filters = {'metadata': {'key1': 'val1',
+                                'key2': 'val2',
+                                'key3': 'val3'}}
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+        filters = {'metadata': {'key1': 'val1',
+                                'key2': 'bogus'}}
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+        filters = {'metadata': {'key1': 'val1',
+                                'key2': 'val1'}}
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+
+        # Combination
+        filters = {'display_name': 'test2',
+                   'metadata': {'key1': 'val1'}}
+        self._assertEqualsVolumeOrderResult([vol3], filters=filters)
+        self._assertEqualsVolumeOrderResult([vol3], limit=1,
+                                            filters=filters)
+        self._assertEqualsVolumeOrderResult([vol3], limit=100,
+                                            filters=filters)
+        filters = {'display_name': 'test3',
+                   'metadata': {'key2': 'val2',
+                                'key3': 'val3'},
+                   'host': 'host5'}
+        self._assertEqualsVolumeOrderResult([vol5], filters=filters)
+        self._assertEqualsVolumeOrderResult([vol5], limit=1,
+                                            filters=filters)
+
+    def test_volume_get_no_migration_targets(self):
+        """Verifies the unique 'no_migration_targets'=True filter.
+
+        This filter returns volumes with either a NULL 'migration_status'
+        or a non-NULL value that does not start with 'target:'.
+        """
+        vol1 = db.volume_create(self.ctxt, {'display_name': 'test1'})
+        vol2 = db.volume_create(self.ctxt, {'display_name': 'test2',
+                                            'migration_status': 'bogus'})
+        vol3 = db.volume_create(self.ctxt, {'display_name': 'test3',
+                                            'migration_status': 'btarget:'})
+        vol4 = db.volume_create(self.ctxt, {'display_name': 'test4',
+                                            'migration_status': 'target:'})
+        vols = [vol1, vol2, vol3, vol4]
+
+        # Ensure we have 4 total instances
+        self._assertEqualsVolumeOrderResult(vols)
+
+        # Apply the unique filter
+        filters = {'no_migration_targets': True}
+        self._assertEqualsVolumeOrderResult([vol1, vol2, vol3],
+                                            filters=filters)
+        self._assertEqualsVolumeOrderResult([vol1, vol2], limit=2,
+                                            filters=filters)
+
+        filters = {'no_migration_targets': True,
+                   'display_name': 'test4'}
+        self._assertEqualsVolumeOrderResult([], filters=filters)
+
     def test_volume_get_iscsi_target_num(self):
         target = db.iscsi_target_create_safe(self.ctxt, {'volume_id': 42,
                                                          'target_num': 43})
@@ -414,7 +673,8 @@ class DBAPIVolumeTestCase(BaseTest):
 
     def test_volume_update(self):
         volume = db.volume_create(self.ctxt, {'host': 'h1'})
-        db.volume_update(self.ctxt, volume['id'], {'host': 'h2'})
+        db.volume_update(self.ctxt, volume['id'],
+                         {'host': 'h2', 'metadata': {'m1': 'v1'}})
         volume = db.volume_get(self.ctxt, volume['id'])
         self.assertEqual('h2', volume['host'])
 
@@ -434,9 +694,9 @@ class DBAPIVolumeTestCase(BaseTest):
         should_be = {'a': '3', 'c': '2', 'd': '5'}
 
         db.volume_create(self.ctxt, {'id': 1, 'metadata': metadata1})
-        db.volume_metadata_update(self.ctxt, 1, metadata2, False)
+        db_meta = db.volume_metadata_update(self.ctxt, 1, metadata2, False)
 
-        self.assertEqual(should_be, db.volume_metadata_get(self.ctxt, 1))
+        self.assertEqual(should_be, db_meta)
 
     def test_volume_metadata_update_delete(self):
         metadata1 = {'a': '1', 'c': '2'}
@@ -444,12 +704,41 @@ class DBAPIVolumeTestCase(BaseTest):
         should_be = metadata2
 
         db.volume_create(self.ctxt, {'id': 1, 'metadata': metadata1})
-        db.volume_metadata_update(self.ctxt, 1, metadata2, True)
+        db_meta = db.volume_metadata_update(self.ctxt, 1, metadata2, True)
 
-        self.assertEqual(should_be, db.volume_metadata_get(self.ctxt, 1))
+        self.assertEqual(should_be, db_meta)
+
+    def test_volume_metadata_delete(self):
+        metadata = {'a': 'b', 'c': 'd'}
+        db.volume_create(self.ctxt, {'id': 1, 'metadata': metadata})
+        db.volume_metadata_delete(self.ctxt, 1, 'c')
+        metadata.pop('c')
+        self.assertEqual(metadata, db.volume_metadata_get(self.ctxt, 1))
 
 
 class DBAPISnapshotTestCase(BaseTest):
+
+    """Tests for cinder.db.api.snapshot_*."""
+
+    def test_snapshot_data_get_for_project(self):
+        actual = db.snapshot_data_get_for_project(self.ctxt, 'project1')
+        self.assertEqual(actual, (0, 0))
+        db.volume_create(self.ctxt, {'id': 1,
+                                     'project_id': 'project1',
+                                     'size': 42})
+        snapshot = db.snapshot_create(self.ctxt, {'id': 1, 'volume_id': 1,
+                                                  'project_id': 'project1',
+                                                  'volume_size': 42})
+        actual = db.snapshot_data_get_for_project(self.ctxt, 'project1')
+        self.assertEqual(actual, (1, 42))
+
+    def test_snapshot_get_all(self):
+        db.volume_create(self.ctxt, {'id': 1})
+        snapshot = db.snapshot_create(self.ctxt, {'id': 1, 'volume_id': 1})
+        self._assertEqualListsOfObjects([snapshot],
+                                        db.snapshot_get_all(self.ctxt),
+                                        ignored_keys=['metadata', 'volume'])
+
     def test_snapshot_metadata_get(self):
         metadata = {'a': 'b', 'c': 'd'}
         db.volume_create(self.ctxt, {'id': 1})
@@ -466,9 +755,9 @@ class DBAPISnapshotTestCase(BaseTest):
         db.volume_create(self.ctxt, {'id': 1})
         db.snapshot_create(self.ctxt,
                            {'id': 1, 'volume_id': 1, 'metadata': metadata1})
-        db.snapshot_metadata_update(self.ctxt, 1, metadata2, False)
+        db_meta = db.snapshot_metadata_update(self.ctxt, 1, metadata2, False)
 
-        self.assertEqual(should_be, db.snapshot_metadata_get(self.ctxt, 1))
+        self.assertEqual(should_be, db_meta)
 
     def test_snapshot_metadata_update_delete(self):
         metadata1 = {'a': '1', 'c': '2'}
@@ -478,9 +767,9 @@ class DBAPISnapshotTestCase(BaseTest):
         db.volume_create(self.ctxt, {'id': 1})
         db.snapshot_create(self.ctxt,
                            {'id': 1, 'volume_id': 1, 'metadata': metadata1})
-        db.snapshot_metadata_update(self.ctxt, 1, metadata2, True)
+        db_meta = db.snapshot_metadata_update(self.ctxt, 1, metadata2, True)
 
-        self.assertEqual(should_be, db.snapshot_metadata_get(self.ctxt, 1))
+        self.assertEqual(should_be, db_meta)
 
     def test_snapshot_metadata_delete(self):
         metadata = {'a': '1', 'c': '2'}
@@ -494,6 +783,26 @@ class DBAPISnapshotTestCase(BaseTest):
         self.assertEqual(should_be, db.snapshot_metadata_get(self.ctxt, 1))
 
 
+class DBAPIVolumeTypeTestCase(BaseTest):
+
+    """Tests for the db.api.volume_type_* methods."""
+
+    def setUp(self):
+        self.ctxt = context.get_admin_context()
+        super(DBAPIVolumeTypeTestCase, self).setUp()
+
+    def test_volume_type_create_exists(self):
+        vt = db.volume_type_create(self.ctxt, {'name': 'n1'})
+        self.assertRaises(exception.VolumeTypeExists,
+                          db.volume_type_create,
+                          self.ctxt,
+                          {'name': 'n1'})
+        self.assertRaises(exception.VolumeTypeExists,
+                          db.volume_type_create,
+                          self.ctxt,
+                          {'name': 'n2', 'id': vt['id']})
+
+
 class DBAPIEncryptionTestCase(BaseTest):
 
     """Tests for the db.api.volume_type_encryption_* methods."""
@@ -508,11 +817,11 @@ class DBAPIEncryptionTestCase(BaseTest):
     def setUp(self):
         super(DBAPIEncryptionTestCase, self).setUp()
         self.created = \
-            [db.volume_type_encryption_update_or_create(self.ctxt, 'fake_type',
-                                                        values)
+            [db.volume_type_encryption_create(self.ctxt,
+                                              values['volume_type_id'], values)
              for values in self._get_values()]
 
-    def _get_values(self, one=False):
+    def _get_values(self, one=False, updated=False):
         base_values = {
             'cipher': 'fake_cipher',
             'key_size': 256,
@@ -520,21 +829,43 @@ class DBAPIEncryptionTestCase(BaseTest):
             'volume_type_id': 'fake_type',
             'control_location': 'front-end',
         }
+        updated_values = {
+            'cipher': 'fake_updated_cipher',
+            'key_size': 512,
+            'provider': 'fake_updated_provider',
+            'volume_type_id': 'fake_type',
+            'control_location': 'front-end',
+        }
+
         if one:
             return base_values
 
+        if updated:
+            values = updated_values
+        else:
+            values = base_values
+
         def compose(val, step):
             if isinstance(val, str):
                 step = str(step)
             return val + step
 
-        return [dict([(k, compose(v, i)) for k, v in base_values.items()])
+        return [dict([(k, compose(v, i)) for k, v in values.items()])
                 for i in range(1, 4)]
 
-    def test_volume_type_encryption_update_or_create(self):
+    def test_volume_type_encryption_create(self):
         values = self._get_values()
         for i, encryption in enumerate(self.created):
-            self._assertEqualObjects(values[i], encryption,
+            self._assertEqualObjects(values[i], encryption, self._ignored_keys)
+
+    def test_volume_type_encryption_update(self):
+        update_values = self._get_values(updated=True)
+        self.updated = \
+            [db.volume_type_encryption_update(self.ctxt,
+                                              values['volume_type_id'], values)
+             for values in update_values]
+        for i, encryption in enumerate(self.updated):
+            self._assertEqualObjects(update_values[i], encryption,
                                      self._ignored_keys)
 
     def test_volume_type_encryption_get(self):
@@ -545,6 +876,13 @@ class DBAPIEncryptionTestCase(BaseTest):
             self._assertEqualObjects(encryption, encryption_get,
                                      self._ignored_keys)
 
+    def test_volume_type_update_with_no_create(self):
+        self.assertRaises(exception.VolumeTypeEncryptionNotFound,
+                          db.volume_type_encryption_update,
+                          self.ctxt,
+                          'fake_no_create_type',
+                          {'cipher': 'fake_updated_cipher'})
+
     def test_volume_type_encryption_delete(self):
         values = {
             'cipher': 'fake_cipher',
@@ -554,9 +892,8 @@ class DBAPIEncryptionTestCase(BaseTest):
             'control_location': 'front-end',
         }
 
-        encryption = db.volume_type_encryption_update_or_create(self.ctxt,
-                                                                'fake_type',
-                                                                values)
+        encryption = db.volume_type_encryption_create(self.ctxt, 'fake_type',
+                                                      values)
         self._assertEqualObjects(values, encryption, self._ignored_keys)
 
         db.volume_type_encryption_delete(self.ctxt,
@@ -564,7 +901,7 @@ class DBAPIEncryptionTestCase(BaseTest):
         encryption_get = \
             db.volume_type_encryption_get(self.ctxt,
                                           encryption['volume_type_id'])
-        self.assertEqual(None, encryption_get)
+        self.assertIsNone(encryption_get)
 
 
 class DBAPIReservationTestCase(BaseTest):
@@ -652,6 +989,16 @@ class DBAPIReservationTestCase(BaseTest):
                              self.ctxt,
                              'project1'))
 
+    def test_reservation_get_all_by_project(self):
+        reservations = _quota_reserve(self.ctxt, 'project1')
+        r1 = db.reservation_get(self.ctxt, reservations[0])
+        r2 = db.reservation_get(self.ctxt, reservations[1])
+        expected = {'project_id': 'project1',
+                    r1['resource']: {r1['uuid']: r1['delta']},
+                    r2['resource']: {r2['uuid']: r2['delta']}}
+        self.assertEqual(expected, db.reservation_get_all_by_project(
+            self.ctxt, 'project1'))
+
     def test_reservation_expire(self):
         self.values['expire'] = datetime.datetime.utcnow() + \
             datetime.timedelta(days=1)
@@ -666,6 +1013,60 @@ class DBAPIReservationTestCase(BaseTest):
                              self.ctxt,
                              'project1'))
 
+    def test_reservation_destroy(self):
+        reservations = _quota_reserve(self.ctxt, 'project1')
+        r1 = db.reservation_get(self.ctxt, reservations[0])
+        db.reservation_destroy(self.ctxt, reservations[1])
+        expected = {'project_id': 'project1',
+                    r1['resource']: {r1['uuid']: r1['delta']}}
+        self.assertEqual(expected, db.reservation_get_all_by_project(
+            self.ctxt, 'project1'))
+
+
+class DBAPIQuotaClassTestCase(BaseTest):
+
+    """Tests for db.api.quota_class_* methods."""
+
+    def setUp(self):
+        super(DBAPIQuotaClassTestCase, self).setUp()
+        self.sample_qc = db.quota_class_create(self.ctxt, 'test_qc',
+                                               'test_resource', 42)
+
+    def test_quota_class_get(self):
+        qc = db.quota_class_get(self.ctxt, 'test_qc', 'test_resource')
+        self._assertEqualObjects(self.sample_qc, qc)
+
+    def test_quota_class_destroy(self):
+        db.quota_class_destroy(self.ctxt, 'test_qc', 'test_resource')
+        self.assertRaises(exception.QuotaClassNotFound,
+                          db.quota_class_get, self.ctxt,
+                          'test_qc', 'test_resource')
+
+    def test_quota_class_get_not_found(self):
+        self.assertRaises(exception.QuotaClassNotFound,
+                          db.quota_class_get, self.ctxt, 'nonexistent',
+                          'nonexistent')
+
+    def test_quota_class_get_all_by_name(self):
+        sample1 = db.quota_class_create(self.ctxt, 'test2', 'res1', 43)
+        sample2 = db.quota_class_create(self.ctxt, 'test2', 'res2', 44)
+        self.assertEqual({'class_name': 'test_qc', 'test_resource': 42},
+                         db.quota_class_get_all_by_name(self.ctxt, 'test_qc'))
+        self.assertEqual({'class_name': 'test2', 'res1': 43, 'res2': 44},
+                         db.quota_class_get_all_by_name(self.ctxt, 'test2'))
+
+    def test_quota_class_update(self):
+        db.quota_class_update(self.ctxt, 'test_qc', 'test_resource', 43)
+        updated = db.quota_class_get(self.ctxt, 'test_qc', 'test_resource')
+        self.assertEqual(43, updated['hard_limit'])
+
+    def test_quota_class_destroy_all_by_name(self):
+        sample1 = db.quota_class_create(self.ctxt, 'test2', 'res1', 43)
+        sample2 = db.quota_class_create(self.ctxt, 'test2', 'res2', 44)
+        db.quota_class_destroy_all_by_name(self.ctxt, 'test2')
+        self.assertEqual({'class_name': 'test2'},
+                         db.quota_class_get_all_by_name(self.ctxt, 'test2'))
+
 
 class DBAPIQuotaTestCase(BaseTest):
 
@@ -725,6 +1126,13 @@ class DBAPIQuotaTestCase(BaseTest):
             self.assertIn(reservation.resource, res_names)
             res_names.remove(reservation.resource)
 
+    def test_quota_destroy(self):
+        db.quota_create(self.ctxt, 'project1', 'resource1', 41)
+        self.assertIsNone(db.quota_destroy(self.ctxt, 'project1',
+                                           'resource1'))
+        self.assertRaises(exception.ProjectQuotaNotFound, db.quota_get,
+                          self.ctxt, 'project1', 'resource1')
+
     def test_quota_destroy_all_by_project(self):
         reservations = _quota_reserve(self.ctxt, 'project1')
         db.quota_destroy_all_by_project(self.ctxt, 'project1')
@@ -855,6 +1263,11 @@ class DBAPIBackupTestCase(BaseTest):
                                               self.created[1]['project_id'])
         self._assertEqualObjects(self.created[1], byproj[0])
 
+    def test_backup_update_nonexistent(self):
+        self.assertRaises(exception.BackupNotFound,
+                          db.backup_update,
+                          self.ctxt, 'nonexistent', {})
+
     def test_backup_update(self):
         updated_values = self._get_values(one=True)
         update_id = self.created[1]['id']
