@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2013 Red Hat, Inc.
 # All Rights Reserved.
 #
@@ -16,8 +14,9 @@
 #    under the License.
 """Unit tests for the GlusterFS driver module."""
 
+import contextlib
 import errno
-import json
+import mock
 import os
 import tempfile
 
@@ -26,17 +25,18 @@ from mox import IgnoreArg
 from mox import IsA
 from mox import stubout
 
-from cinder import compute
+from cinder import brick
 from cinder import context
 from cinder import db
 from cinder import exception
 from cinder.image import image_utils
+from cinder.openstack.common import imageutils
 from cinder.openstack.common import processutils as putils
 from cinder import test
-from cinder.tests.compute import test_nova
 from cinder import units
 from cinder import utils
 from cinder.volume import configuration as conf
+from cinder.volume import driver as base_driver
 from cinder.volume.drivers import glusterfs
 
 
@@ -50,6 +50,17 @@ class DumbVolume(object):
         return self.fields[item]
 
 
+class FakeDb(object):
+    msg = "Tests are broken: mock this out."
+
+    def volume_get(self, *a, **kw):
+        raise Exception(self.msg)
+
+    def snapshot_get_all_for_volume(self, *a, **kw):
+        """Mock this if you want results from it."""
+        return []
+
+
 class GlusterFsDriverTestCase(test.TestCase):
     """Test case for GlusterFS driver."""
 
@@ -81,7 +92,8 @@ class GlusterFsDriverTestCase(test.TestCase):
 
         self.stubs = stubout.StubOutForTesting()
         self._driver =\
-            glusterfs.GlusterfsDriver(configuration=self._configuration)
+            glusterfs.GlusterfsDriver(configuration=self._configuration,
+                                      db=FakeDb())
         self._driver.shares = {}
 
     def tearDown(self):
@@ -94,6 +106,23 @@ class GlusterFsDriverTestCase(test.TestCase):
         stub = mox_lib.MockObject(attr_to_replace)
         self.stubs.Set(obj, attr_name, stub)
 
+    def test_set_execute(self):
+        mox = self._mox
+        drv = self._driver
+
+        rfsclient = brick.remotefs.remotefs.RemoteFsClient
+
+        mox.StubOutWithMock(rfsclient, 'set_execute')
+
+        def my_execute(*a, **k):
+            pass
+
+        rfsclient.set_execute(my_execute)
+
+        mox.ReplayAll()
+
+        drv.set_execute(my_execute)
+
     def test_local_path(self):
         """local_path common use case."""
         glusterfs.CONF.glusterfs_mount_point_base = self.TEST_MNT_POINT_BASE
@@ -194,14 +223,22 @@ class GlusterFsDriverTestCase(test.TestCase):
                          drv._get_hash_str(self.TEST_EXPORT1))
 
     def test_get_mount_point_for_share(self):
-        """_get_mount_point_for_share should calculate correct value."""
+        """_get_mount_point_for_share should call RemoteFsClient."""
+        mox = self._mox
         drv = self._driver
+        hashed_path = '/mnt/test/abcdefabcdef'
+
+        mox.StubOutWithMock(brick.remotefs.remotefs.RemoteFsClient,
+                            'get_mount_point')
 
         glusterfs.CONF.glusterfs_mount_point_base = self.TEST_MNT_POINT_BASE
 
-        self.assertEqual('/mnt/test/ab03ab34eaca46a5fb81878f7e9b91fc',
-                         drv._get_mount_point_for_share(
-                             self.TEST_EXPORT1))
+        brick.remotefs.remotefs.RemoteFsClient.\
+            get_mount_point(self.TEST_EXPORT1).AndReturn(hashed_path)
+
+        mox.ReplayAll()
+
+        drv._get_mount_point_for_share(self.TEST_EXPORT1)
 
     def test_get_available_capacity_with_df(self):
         """_get_available_capacity should calculate correct value."""
@@ -235,50 +272,6 @@ class GlusterFsDriverTestCase(test.TestCase):
 
         delattr(glusterfs.CONF, 'glusterfs_disk_util')
 
-    def test_get_available_capacity_with_du(self):
-        """_get_available_capacity should calculate correct value."""
-        mox = self._mox
-        drv = self._driver
-
-        old_value = self._configuration.glusterfs_disk_util
-        self._configuration.glusterfs_disk_util = 'du'
-
-        df_total_size = 2620544
-        df_used_size = 996864
-        df_avail_size = 1490560
-        df_title = 'Filesystem 1-blocks Used Available Use% Mounted on\n'
-        df_mnt_data = 'glusterfs-host:/export %d %d %d 41%% /mnt' % \
-                      (df_total_size,
-                       df_used_size,
-                       df_avail_size)
-        df_output = df_title + df_mnt_data
-
-        du_used = 490560
-        du_output = '%d /mnt' % du_used
-
-        mox.StubOutWithMock(drv, '_get_mount_point_for_share')
-        drv._get_mount_point_for_share(self.TEST_EXPORT1).\
-            AndReturn(self.TEST_MNT_POINT)
-
-        mox.StubOutWithMock(drv, '_execute')
-        drv._execute('df', '--portability', '--block-size', '1',
-                     self.TEST_MNT_POINT,
-                     run_as_root=True).\
-            AndReturn((df_output, None))
-        drv._execute('du', '-sb', '--apparent-size',
-                     '--exclude', '*snapshot*',
-                     self.TEST_MNT_POINT,
-                     run_as_root=True).AndReturn((du_output, None))
-
-        mox.ReplayAll()
-
-        self.assertEqual((df_total_size - du_used, df_total_size),
-                         drv._get_available_capacity(self.TEST_EXPORT1))
-
-        mox.VerifyAll()
-
-        self._configuration.glusterfs_disk_util = old_value
-
     def test_load_shares_config(self):
         mox = self._mox
         drv = self._driver
@@ -467,7 +460,8 @@ class GlusterFsDriverTestCase(test.TestCase):
 
         drv._mounted_shares = []
 
-        self.assertRaises(exception.NotFound, drv._find_share,
+        self.assertRaises(exception.GlusterfsNoSharesMounted,
+                          drv._find_share,
                           self.TEST_SIZE_IN_GB)
 
     def test_find_share(self):
@@ -753,6 +747,31 @@ class GlusterFsDriverTestCase(test.TestCase):
 
         mox.VerifyAll()
 
+    @mock.patch('os.remove')
+    @mock.patch('os.path.exists')
+    def test_delete_volume_with_info_file(self, mock_path_exists, mock_remove):
+        mock_path_exists.return_value = True
+        info_file = self.TEST_LOCAL_PATH + '.info'
+        volume = self._simple_volume()
+
+        with contextlib.nested(
+                mock.patch.object(self._driver, '_ensure_share_mounted'),
+                mock.patch.object(self._driver, 'local_path'),
+                mock.patch.object(self._driver, '_execute')
+        ) as (mock_ensure_share_mounted, mock_local_path, mock_execute):
+            mock_local_path.return_value = self.TEST_LOCAL_PATH
+
+            self._driver.delete_volume(volume)
+
+            mock_ensure_share_mounted.assert_called_once_with(
+                volume['provider_location'])
+            mock_local_path.assert_called_once_with(volume)
+            mock_execute.assert_called_once_with('rm', '-f',
+                                                 self.TEST_LOCAL_PATH,
+                                                 run_as_root=True)
+            mock_path_exists.assert_called_once_with(info_file)
+            mock_remove.assert_called_once_with(info_file)
+
     def test_create_snapshot(self):
         (mox, drv) = self._mox, self._driver
 
@@ -857,7 +876,7 @@ class GlusterFsDriverTestCase(test.TestCase):
 
         drv._ensure_share_writable(volume_dir)
 
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
         image_utils.qemu_img_info(snap_path_2).AndReturn(img_info)
 
         info_file_dict = {'active': snap_file_2,
@@ -969,7 +988,7 @@ class GlusterFsDriverTestCase(test.TestCase):
         drv._read_info_file(info_path, empty_if_missing=True).\
             AndReturn(info_file_dict)
 
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output_snap_1)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output_snap_1)
         image_utils.qemu_img_info(snap_path).AndReturn(img_info)
 
         snap_ref = {'name': 'test snap',
@@ -1082,7 +1101,7 @@ class GlusterFsDriverTestCase(test.TestCase):
         disk size: 473K
         """ % self.VOLUME_UUID
 
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
 
         mox.StubOutWithMock(drv, '_execute')
         mox.StubOutWithMock(drv, 'get_active_image_from_info')
@@ -1272,14 +1291,14 @@ class GlusterFsDriverTestCase(test.TestCase):
         disk size: 173K
         backing file: %s
         """ % (snap_file, volume_file)
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
 
         vol_qemu_img_info_output = """image: %s
         file format: raw
         virtual size: 1.0G (1073741824 bytes)
         disk size: 173K
         """ % volume_file
-        volume_img_info = image_utils.QemuImgInfo(vol_qemu_img_info_output)
+        volume_img_info = imageutils.QemuImgInfo(vol_qemu_img_info_output)
 
         image_utils.qemu_img_info(snap_path).AndReturn(img_info)
 
@@ -1375,14 +1394,14 @@ class GlusterFsDriverTestCase(test.TestCase):
         disk size: 173K
         backing file: %s
         """ % (snap_file, volume_file)
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
 
         vol_qemu_img_info_output = """image: %s
         file format: raw
         virtual size: 1.0G (1073741824 bytes)
         disk size: 173K
         """ % volume_file
-        volume_img_info = image_utils.QemuImgInfo(vol_qemu_img_info_output)
+        volume_img_info = imageutils.QemuImgInfo(vol_qemu_img_info_output)
 
         image_utils.qemu_img_info(snap_path).AndReturn(img_info)
 
@@ -1468,7 +1487,7 @@ class GlusterFsDriverTestCase(test.TestCase):
         disk size: 173K
         backing file: %s
         """ % (snap_file, volume_file)
-        img_info = image_utils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
 
         image_utils.qemu_img_info(snap_path).AndReturn(img_info)
 
@@ -1548,9 +1567,9 @@ class GlusterFsDriverTestCase(test.TestCase):
         qemu_img_output_3 = qemu_img_output % {'image_name': vol_filename_3,
                                                'backing_file': vol_filename_2}
 
-        info_1 = image_utils.QemuImgInfo(qemu_img_output_1)
-        info_2 = image_utils.QemuImgInfo(qemu_img_output_2)
-        info_3 = image_utils.QemuImgInfo(qemu_img_output_3)
+        info_1 = imageutils.QemuImgInfo(qemu_img_output_1)
+        info_2 = imageutils.QemuImgInfo(qemu_img_output_2)
+        info_3 = imageutils.QemuImgInfo(qemu_img_output_3)
 
         drv._local_volume_dir(volume).AndReturn(vol_dir)
         image_utils.qemu_img_info(vol_path_3).\
@@ -1618,7 +1637,7 @@ class GlusterFsDriverTestCase(test.TestCase):
         disk size: 173K
         backing file: %s
         """ % (snap_file, src_volume['name'])
-        img_info = image_utils.QemuImgInfo(qemu_img_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         image_utils.qemu_img_info(snap_path).AndReturn(img_info)
 
@@ -1677,7 +1696,7 @@ class GlusterFsDriverTestCase(test.TestCase):
         virtual size: 1.0G (1073741824 bytes)
         disk size: 173K
         """ % volume['name']
-        img_info = image_utils.QemuImgInfo(qemu_img_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         mox.StubOutWithMock(drv, 'get_active_image_from_info')
         mox.StubOutWithMock(image_utils, 'qemu_img_info')
@@ -1692,3 +1711,177 @@ class GlusterFsDriverTestCase(test.TestCase):
         self.assertEqual(conn_info['data']['format'], 'raw')
         self.assertEqual(conn_info['driver_volume_type'], 'glusterfs')
         self.assertEqual(conn_info['data']['name'], volume['name'])
+        self.assertEqual(conn_info['mount_point_base'],
+                         self.TEST_MNT_POINT_BASE)
+
+    def test_get_mount_point_base(self):
+        (mox, drv) = self._mox, self._driver
+
+        self.assertEqual(drv._get_mount_point_base(),
+                         self.TEST_MNT_POINT_BASE)
+
+    def test_backup_volume(self):
+        """Backup a volume with no snapshots."""
+
+        (mox, drv) = self._mox, self._driver
+
+        mox.StubOutWithMock(drv, '_qemu_img_info')
+        mox.StubOutWithMock(drv.db, 'volume_get')
+        mox.StubOutWithMock(base_driver.VolumeDriver, 'backup_volume')
+        mox.StubOutWithMock(drv, '_read_info_file')
+        mox.StubOutWithMock(drv, 'get_active_image_from_info')
+
+        ctxt = context.RequestContext('fake_user', 'fake_project')
+        volume = self._simple_volume()
+        backup = {'volume_id': volume['id']}
+
+        drv._read_info_file(IgnoreArg(), empty_if_missing=True).AndReturn({})
+        drv.get_active_image_from_info(IgnoreArg()).AndReturn('/some/path')
+
+        info = imageutils.QemuImgInfo()
+        info.file_format = 'raw'
+
+        drv.db.volume_get(ctxt, volume['id']).AndReturn(volume)
+        drv._qemu_img_info(IgnoreArg()).AndReturn(info)
+
+        base_driver.VolumeDriver.backup_volume(IgnoreArg(),
+                                               IgnoreArg(),
+                                               IgnoreArg())
+
+        mox.ReplayAll()
+
+        drv.backup_volume(ctxt, backup, IgnoreArg())
+
+    def test_backup_volume_previous_snap(self):
+        """Backup a volume that previously had a snapshot.
+
+           Snapshot was deleted, snap_info is different from above.
+        """
+
+        (mox, drv) = self._mox, self._driver
+
+        mox.StubOutWithMock(drv, '_qemu_img_info')
+        mox.StubOutWithMock(drv.db, 'volume_get')
+        mox.StubOutWithMock(drv, '_read_info_file')
+        mox.StubOutWithMock(drv, 'get_active_image_from_info')
+        mox.StubOutWithMock(base_driver.VolumeDriver, 'backup_volume')
+
+        ctxt = context.RequestContext('fake_user', 'fake_project')
+        volume = self._simple_volume()
+        backup = {'volume_id': volume['id']}
+
+        drv._read_info_file(IgnoreArg(), empty_if_missing=True).AndReturn(
+            {'active': 'file2'})
+        drv.get_active_image_from_info(IgnoreArg()).AndReturn('/some/file2')
+
+        info = imageutils.QemuImgInfo()
+        info.file_format = 'raw'
+
+        drv.db.volume_get(ctxt, volume['id']).AndReturn(volume)
+        drv._qemu_img_info(IgnoreArg()).AndReturn(info)
+
+        base_driver.VolumeDriver.backup_volume(IgnoreArg(),
+                                               IgnoreArg(),
+                                               IgnoreArg())
+
+        mox.ReplayAll()
+
+        drv.backup_volume(ctxt, backup, IgnoreArg())
+
+    def test_backup_snap_failure_1(self):
+        """Backup fails if snapshot exists (database)."""
+
+        (mox, drv) = self._mox, self._driver
+        mox.StubOutWithMock(drv.db, 'snapshot_get_all_for_volume')
+        mox.StubOutWithMock(base_driver.VolumeDriver, 'backup_volume')
+
+        ctxt = context.RequestContext('fake_user', 'fake_project')
+        volume = self._simple_volume()
+        backup = {'volume_id': volume['id']}
+
+        drv.db.snapshot_get_all_for_volume(ctxt, volume['id']).AndReturn(
+            [{'snap1': 'a'}, {'snap2': 'b'}])
+
+        base_driver.VolumeDriver.backup_volume(IgnoreArg(),
+                                               IgnoreArg(),
+                                               IgnoreArg())
+
+        mox.ReplayAll()
+
+        self.assertRaises(exception.InvalidVolume,
+                          drv.backup_volume,
+                          ctxt, backup, IgnoreArg())
+
+    def test_backup_snap_failure_2(self):
+        """Backup fails if snapshot exists (on-disk)."""
+
+        (mox, drv) = self._mox, self._driver
+        mox.StubOutWithMock(drv, '_read_info_file')
+        mox.StubOutWithMock(drv.db, 'volume_get')
+        mox.StubOutWithMock(drv, 'get_active_image_from_info')
+        mox.StubOutWithMock(drv, '_qemu_img_info')
+        mox.StubOutWithMock(base_driver.VolumeDriver, 'backup_volume')
+
+        ctxt = context.RequestContext('fake_user', 'fake_project')
+        volume = self._simple_volume()
+        backup = {'volume_id': volume['id']}
+
+        drv.db.volume_get(ctxt, volume['id']).AndReturn(volume)
+
+        drv._read_info_file(IgnoreArg(), empty_if_missing=True).AndReturn(
+            {'id1': 'file1',
+             'id2': 'file2',
+             'active': 'file2'})
+
+        drv.get_active_image_from_info(IgnoreArg()).\
+            AndReturn('/some/path/file2')
+
+        info = imageutils.QemuImgInfo()
+        info.file_format = 'raw'
+        info.backing_file = 'file1'
+
+        drv._qemu_img_info(IgnoreArg()).AndReturn(info)
+
+        base_driver.VolumeDriver.backup_volume(IgnoreArg(),
+                                               IgnoreArg(),
+                                               IgnoreArg())
+
+        mox.ReplayAll()
+
+        self.assertRaises(exception.InvalidVolume,
+                          drv.backup_volume,
+                          ctxt, backup, IgnoreArg())
+
+    def test_backup_failure_unsupported_format(self):
+        """Attempt to backup a volume with a qcow2 base."""
+
+        (mox, drv) = self._mox, self._driver
+
+        mox.StubOutWithMock(drv, '_qemu_img_info')
+        mox.StubOutWithMock(drv.db, 'volume_get')
+        mox.StubOutWithMock(base_driver.VolumeDriver, 'backup_volume')
+        mox.StubOutWithMock(drv, '_read_info_file')
+        mox.StubOutWithMock(drv, 'get_active_image_from_info')
+
+        ctxt = context.RequestContext('fake_user', 'fake_project')
+        volume = self._simple_volume()
+        backup = {'volume_id': volume['id']}
+
+        drv._read_info_file(IgnoreArg(), empty_if_missing=True).AndReturn({})
+        drv.get_active_image_from_info(IgnoreArg()).AndReturn('/some/path')
+
+        info = imageutils.QemuImgInfo()
+        info.file_format = 'qcow2'
+
+        drv.db.volume_get(ctxt, volume['id']).AndReturn(volume)
+        drv._qemu_img_info(IgnoreArg()).AndReturn(info)
+
+        base_driver.VolumeDriver.backup_volume(IgnoreArg(),
+                                               IgnoreArg(),
+                                               IgnoreArg())
+
+        mox.ReplayAll()
+
+        self.assertRaises(exception.InvalidVolume,
+                          drv.backup_volume,
+                          ctxt, backup, IgnoreArg())
