@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2012 NetApp, Inc.
 # All Rights Reserved.
@@ -22,12 +21,15 @@ Tests for NetApp volume driver
 import BaseHTTPServer
 import httplib
 from lxml import etree
-import StringIO
+
+import six
 
 from cinder import exception
 from cinder.openstack.common import log as logging
 from cinder import test
 from cinder.volume import configuration as conf
+from cinder.volume.drivers.netapp.api import NaElement
+from cinder.volume.drivers.netapp.api import NaServer
 from cinder.volume.drivers.netapp import common
 from cinder.volume.drivers.netapp.options import netapp_7mode_opts
 from cinder.volume.drivers.netapp.options import netapp_basicauth_opts
@@ -36,7 +38,6 @@ from cinder.volume.drivers.netapp.options import netapp_connection_opts
 from cinder.volume.drivers.netapp.options import netapp_provisioning_opts
 from cinder.volume.drivers.netapp.options import netapp_transport_opts
 from cinder.volume.drivers.netapp import ssc_utils
-from cinder.volume.drivers.netapp import utils
 
 
 LOG = logging.getLogger("cinder.volume.driver")
@@ -61,10 +62,10 @@ class FakeHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
 
 class FakeHttplibSocket(object):
-    """A fake socket implementation for httplib.HTTPResponse"""
+    """A fake socket implementation for httplib.HTTPResponse."""
     def __init__(self, value):
-        self._rbuffer = StringIO.StringIO(value)
-        self._wbuffer = StringIO.StringIO('')
+        self._rbuffer = six.StringIO(value)
+        self._wbuffer = six.StringIO('')
         oldclose = self._wbuffer.close
 
         def newclose():
@@ -93,11 +94,11 @@ RESPONSE_SUFFIX_DIRECT = """</netapp>"""
 
 
 class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
-    """HTTP handler that fakes enough stuff to allow the driver to run"""
+    """HTTP handler that fakes enough stuff to allow the driver to run."""
 
     def do_GET(s):
         """Respond to a GET request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
@@ -111,7 +112,7 @@ class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
 
     def do_POST(s):
         """Respond to a POST request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
@@ -342,6 +343,12 @@ class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
                 body = """<results status="passed"><lun-id-assigned>1
                 </lun-id-assigned>
                 </results>"""
+        elif 'lun-get-geometry' == api:
+                body = """<results status="passed"><bytes-per-sector>256
+                </bytes-per-sector><cylinders>512</cylinders><max-resize-size>
+                3221225472</max-resize-size><sectors-per-track>512
+                </sectors-per-track><size>2147483648</size>
+                <tracks-per-cylinder>256</tracks-per-cylinder></results>"""
         elif 'iscsi-service-get-iter' == api:
                 body = """<results status="passed"><attributes-list>
                 <iscsi-service-info>
@@ -433,7 +440,7 @@ class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
 
     @staticmethod
     def _get_child_content(self, name):
-        """Get the content of the child"""
+        """Get the content of the child."""
         for child in self.iterchildren():
             if child.tag == name or etree.QName(child.tag).localname == name:
                 return child.text
@@ -459,7 +466,7 @@ class FakeDirectCmodeHTTPConnection(object):
         if data:
             req_str += '\r\n%s' % data
 
-        # NOTE(vish): normally the http transport normailizes from unicode
+        # NOTE(vish): normally the http transport normalizes from unicode
         sock = FakeHttplibSocket(req_str.decode("latin-1").encode("utf-8"))
         # NOTE(vish): stop the server from trying to look up address from
         #             the fake socket
@@ -638,6 +645,18 @@ class NetAppDirectCmodeISCSIDriverTestCase(test.TestCase):
         self.driver.delete_snapshot(self.snapshot)
         self.driver.delete_volume(self.volume)
 
+    def test_extend_vol_same_size(self):
+        self.driver.create_volume(self.volume)
+        self.driver.extend_volume(self.volume, self.volume['size'])
+
+    def test_extend_vol_direct_resize(self):
+        self.driver.create_volume(self.volume)
+        self.driver.extend_volume(self.volume, 3)
+
+    def test_extend_vol_sub_lun_clone(self):
+        self.driver.create_volume(self.volume)
+        self.driver.extend_volume(self.volume, 4)
+
 
 class NetAppDriverNegativeTestCase(test.TestCase):
     """Test case for NetAppDriver"""
@@ -680,11 +699,11 @@ class NetAppDriverNegativeTestCase(test.TestCase):
 
 
 class FakeDirect7MODEServerHandler(FakeHTTPRequestHandler):
-    """HTTP handler that fakes enough stuff to allow the driver to run"""
+    """HTTP handler that fakes enough stuff to allow the driver to run."""
 
     def do_GET(s):
         """Respond to a GET request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
@@ -698,7 +717,7 @@ class FakeDirect7MODEServerHandler(FakeHTTPRequestHandler):
 
     def do_POST(s):
         """Respond to a POST request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
@@ -1186,3 +1205,132 @@ class NetAppDirect7modeISCSIDriverTestCase_WV(
         configuration.netapp_server_port = '80'
         configuration.netapp_vfiler = 'openstack'
         return configuration
+
+
+class NetAppApiElementTransTests(test.TestCase):
+    """Test case for NetApp api element translations."""
+
+    def setUp(self):
+        super(NetAppApiElementTransTests, self).setUp()
+
+    def test_translate_struct_dict_unique_key(self):
+        """Tests if dict gets properly converted to NaElements."""
+        root = NaElement('root')
+        child = {'e1': 'v1', 'e2': 'v2', 'e3': 'v3'}
+        root.translate_struct(child)
+        self.assertEqual(len(root.get_children()), 3)
+        self.assertEqual(root.get_child_content('e1'), 'v1')
+        self.assertEqual(root.get_child_content('e2'), 'v2')
+        self.assertEqual(root.get_child_content('e3'), 'v3')
+
+    def test_translate_struct_dict_nonunique_key(self):
+        """Tests if list/dict gets properly converted to NaElements."""
+        root = NaElement('root')
+        child = [{'e1': 'v1', 'e2': 'v2'}, {'e1': 'v3'}]
+        root.translate_struct(child)
+        self.assertEqual(len(root.get_children()), 3)
+        children = root.get_children()
+        for c in children:
+            if c.get_name() == 'e1':
+                self.assertIn(c.get_content(), ['v1', 'v3'])
+            else:
+                self.assertEqual(c.get_content(), 'v2')
+
+    def test_translate_struct_list(self):
+        """Tests if list gets properly converted to NaElements."""
+        root = NaElement('root')
+        child = ['e1', 'e2']
+        root.translate_struct(child)
+        self.assertEqual(len(root.get_children()), 2)
+        self.assertIsNone(root.get_child_content('e1'))
+        self.assertIsNone(root.get_child_content('e2'))
+
+    def test_translate_struct_tuple(self):
+        """Tests if tuple gets properly converted to NaElements."""
+        root = NaElement('root')
+        child = ('e1', 'e2')
+        root.translate_struct(child)
+        self.assertEqual(len(root.get_children()), 2)
+        self.assertIsNone(root.get_child_content('e1'))
+        self.assertIsNone(root.get_child_content('e2'))
+
+    def test_translate_invalid_struct(self):
+        """Tests if invalid data structure raises exception."""
+        root = NaElement('root')
+        child = 'random child element'
+        self.assertRaises(ValueError, root.translate_struct, child)
+
+    def test_setter_builtin_types(self):
+        """Tests str, int, float get converted to NaElement."""
+        root = NaElement('root')
+        root['e1'] = 'v1'
+        root['e2'] = 1
+        root['e3'] = 2.0
+        root['e4'] = 8l
+        self.assertEqual(len(root.get_children()), 4)
+        self.assertEqual(root.get_child_content('e1'), 'v1')
+        self.assertEqual(root.get_child_content('e2'), '1')
+        self.assertEqual(root.get_child_content('e3'), '2.0')
+        self.assertEqual(root.get_child_content('e4'), '8')
+
+    def test_setter_na_element(self):
+        """Tests na_element gets appended as child."""
+        root = NaElement('root')
+        root['e1'] = NaElement('nested')
+        self.assertEqual(len(root.get_children()), 1)
+        e1 = root.get_child_by_name('e1')
+        self.assertIsInstance(e1, NaElement)
+        self.assertIsInstance(e1.get_child_by_name('nested'), NaElement)
+
+    def test_setter_child_dict(self):
+        """Tests dict is appended as child to root."""
+        root = NaElement('root')
+        root['d'] = {'e1': 'v1', 'e2': 'v2'}
+        e1 = root.get_child_by_name('d')
+        self.assertIsInstance(e1, NaElement)
+        sub_ch = e1.get_children()
+        self.assertEqual(len(sub_ch), 2)
+        for c in sub_ch:
+            self.assertIn(c.get_name(), ['e1', 'e2'])
+            if c.get_name() == 'e1':
+                self.assertEqual(c.get_content(), 'v1')
+            else:
+                self.assertEqual(c.get_content(), 'v2')
+
+    def test_setter_child_list_tuple(self):
+        """Tests list/tuple are appended as child to root."""
+        root = NaElement('root')
+        root['l'] = ['l1', 'l2']
+        root['t'] = ('t1', 't2')
+        l = root.get_child_by_name('l')
+        self.assertIsInstance(l, NaElement)
+        t = root.get_child_by_name('t')
+        self.assertIsInstance(t, NaElement)
+        for le in l.get_children():
+            self.assertIn(le.get_name(), ['l1', 'l2'])
+        for te in t.get_children():
+            self.assertIn(te.get_name(), ['t1', 't2'])
+
+    def test_setter_no_value(self):
+        """Tests key with None value."""
+        root = NaElement('root')
+        root['k'] = None
+        self.assertIsNone(root.get_child_content('k'))
+
+    def test_setter_invalid_value(self):
+        """Tests invalid value raises exception."""
+        root = NaElement('root')
+        try:
+            root['k'] = NaServer('localhost')
+        except Exception as e:
+            if not isinstance(e, TypeError):
+                self.fail(_('Error not a TypeError.'))
+
+    def test_setter_invalid_key(self):
+        """Tests invalid value raises exception."""
+        root = NaElement('root')
+        try:
+            root[None] = 'value'
+        except Exception as e:
+            if not isinstance(e, KeyError):
+                self.fail(_('Error not a KeyError.'))
