@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright (c) 2012 NetApp, Inc.
 # All Rights Reserved.
@@ -20,16 +19,12 @@ import BaseHTTPServer
 import copy
 import httplib
 from lxml import etree
-import mox
 from mox import IgnoreArg
-from mox import IsA
-from mox import MockObject
-import StringIO
 
-from cinder import context
+import six
+
 from cinder import exception
 from cinder import test
-from cinder.volume import configuration as conf
 from cinder.volume.drivers.netapp import api
 from cinder.volume.drivers.netapp import ssc_utils
 
@@ -44,8 +39,8 @@ class FakeHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 class FakeHttplibSocket(object):
     """A fake socket implementation for httplib.HTTPResponse."""
     def __init__(self, value):
-        self._rbuffer = StringIO.StringIO(value)
-        self._wbuffer = StringIO.StringIO('')
+        self._rbuffer = six.StringIO(value)
+        self._wbuffer = six.StringIO('')
         oldclose = self._wbuffer.close
 
         def newclose():
@@ -75,7 +70,7 @@ class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
 
     def do_GET(s):
         """Respond to a GET request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
@@ -89,7 +84,7 @@ class FakeDirectCMODEServerHandler(FakeHTTPRequestHandler):
 
     def do_POST(s):
         """Respond to a POST request."""
-        if '/servlets/netapp.servlets.admin.XMLrequest_filer' != s.path:
+        if '/servlets/netapp.servlets.admin.XMLrequest_filer' not in s.path:
             s.send_response(404)
             s.end_headers
             return
