@@ -1,6 +1,3 @@
-#!/usr/bin/env python
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-#
 # Copyright (c) 2012 Rackspace Hosting
 # All Rights Reserved.
 #
@@ -68,4 +65,4 @@ class VolumeConfigurationTest(test.TestCase):
 
     def test_safe_get(self):
         c = configuration.Configuration(volume_opts, config_group='foo')
-        self.assertEqual(c.safe_get('none_opt'), None)
+        self.assertIsNone(c.safe_get('none_opt'))
