@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2013 OpenStack Foundation
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -30,7 +28,7 @@ LOG = logging.getLogger(__name__)
 
 
 class VolumeTransferTestCase(test.TestCase):
-    """Test cases for volume type code."""
+    """Test cases for volume transfer code."""
     def setUp(self):
         super(VolumeTransferTestCase, self).setUp()
         self.ctxt = context.RequestContext(user_id='user_id',
@@ -39,8 +37,8 @@ class VolumeTransferTestCase(test.TestCase):
 
     def test_transfer_volume_create_delete(self):
         tx_api = transfer_api.API()
-        volume = utils.create_volume(self.ctxt, id='1',
-                                     updated_at=self.updated_at)
+        utils.create_volume(self.ctxt, id='1',
+                            updated_at=self.updated_at)
         response = tx_api.create(self.ctxt, '1', 'Description')
         volume = db.volume_get(self.ctxt, '1')
         self.assertEqual('awaiting-transfer', volume['status'],
@@ -52,8 +50,8 @@ class VolumeTransferTestCase(test.TestCase):
 
     def test_transfer_invalid_volume(self):
         tx_api = transfer_api.API()
-        volume = utils.create_volume(self.ctxt, id='1', status='in-use',
-                                     updated_at=self.updated_at)
+        utils.create_volume(self.ctxt, id='1', status='in-use',
+                            updated_at=self.updated_at)
         self.assertRaises(exception.InvalidVolume,
                           tx_api.create,
                           self.ctxt, '1', 'Description')
@@ -62,8 +60,8 @@ class VolumeTransferTestCase(test.TestCase):
 
     def test_transfer_accept(self):
         tx_api = transfer_api.API()
-        volume = utils.create_volume(self.ctxt, id='1',
-                                     updated_at=self.updated_at)
+        utils.create_volume(self.ctxt, id='1',
+                            updated_at=self.updated_at)
         transfer = tx_api.create(self.ctxt, '1', 'Description')
         volume = db.volume_get(self.ctxt, '1')
         self.assertEqual('awaiting-transfer', volume['status'],
@@ -112,6 +110,7 @@ class VolumeTransferTestCase(test.TestCase):
 
         nctxt = context.RequestContext(user_id='new_user_id',
                                        project_id='new_project_id')
+        utils.create_volume(nctxt, id='2', updated_at=self.updated_at)
         self.assertRaises(exception.TransferNotFound,
                           tx_api.get,
                           nctxt,
