@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2011 Zadara Storage Inc.
 # Copyright (c) 2011 OpenStack Foundation
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -64,7 +62,7 @@ class VolumeTypeTestCase(test.TestCase):
 
         for k, v in self.vol_type1_specs.iteritems():
             self.assertEqual(v, new['extra_specs'][k],
-                             'one of fields doesnt match')
+                             'one of fields does not match')
 
         new_all_vtypes = volume_types.get_all_types(self.ctxt)
         self.assertEqual(len(prev_all_vtypes) + 1,
@@ -213,9 +211,8 @@ class VolumeTypeTestCase(test.TestCase):
             'control_location': 'front-end',
             'provider': 'fake_provider',
         }
-        db_api.volume_type_encryption_update_or_create(self.ctxt,
-                                                       volume_type_id,
-                                                       encryption)
+        db_api.volume_type_encryption_create(self.ctxt, volume_type_id,
+                                             encryption)
         self.assertTrue(volume_types.is_encrypted(self.ctxt, volume_type_id))
 
     def test_get_volume_type_qos_specs(self):
@@ -225,7 +222,7 @@ class VolumeTypeTestCase(test.TestCase):
         type_ref = volume_types.create(self.ctxt, "type1", {"key2": "val2",
                                                   "key3": "val3"})
         res = volume_types.get_volume_type_qos_specs(type_ref['id'])
-        self.assertEqual(res['qos_specs'], None)
+        self.assertIsNone(res['qos_specs'])
         qos_specs.associate_qos_with_type(self.ctxt,
                                           qos_ref['id'],
                                           type_ref['id'])
@@ -239,3 +236,68 @@ class VolumeTypeTestCase(test.TestCase):
                                       'k3': 'v3'}}}
         res = volume_types.get_volume_type_qos_specs(type_ref['id'])
         self.assertDictMatch(expected, res)
+
+    def test_volume_types_diff(self):
+        #type_ref 1 and 2 have the same extra_specs, while 3 has different
+        keyvals1 = {"key1": "val1", "key2": "val2"}
+        keyvals2 = {"key1": "val0", "key2": "val2"}
+        type_ref1 = volume_types.create(self.ctxt, "type1", keyvals1)
+        type_ref2 = volume_types.create(self.ctxt, "type2", keyvals1)
+        type_ref3 = volume_types.create(self.ctxt, "type3", keyvals2)
+
+        # Check equality with only extra_specs
+        diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
+                                                    type_ref2['id'])
+        self.assertEqual(same, True)
+        self.assertEqual(diff['extra_specs']['key1'], ('val1', 'val1'))
+        diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
+                                                    type_ref3['id'])
+        self.assertEqual(same, False)
+        self.assertEqual(diff['extra_specs']['key1'], ('val1', 'val0'))
+
+        #qos_ref 1 and 2 have the same specs, while 3 has different
+        qos_keyvals1 = {'k1': 'v1', 'k2': 'v2', 'k3': 'v3'}
+        qos_keyvals2 = {'k1': 'v0', 'k2': 'v2', 'k3': 'v3'}
+        qos_ref1 = qos_specs.create(self.ctxt, 'qos-specs-1', qos_keyvals1)
+        qos_ref2 = qos_specs.create(self.ctxt, 'qos-specs-2', qos_keyvals1)
+        qos_ref3 = qos_specs.create(self.ctxt, 'qos-specs-3', qos_keyvals2)
+
+        # Check equality with qos specs too
+        qos_specs.associate_qos_with_type(self.ctxt, qos_ref1['id'],
+                                          type_ref1['id'])
+        qos_specs.associate_qos_with_type(self.ctxt, qos_ref2['id'],
+                                          type_ref2['id'])
+        diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
+                                                    type_ref2['id'])
+        self.assertEqual(same, True)
+        self.assertEqual(diff['extra_specs']['key1'], ('val1', 'val1'))
+        self.assertEqual(diff['qos_specs']['k1'], ('v1', 'v1'))
+        qos_specs.disassociate_qos_specs(self.ctxt, qos_ref2['id'],
+                                         type_ref2['id'])
+        qos_specs.associate_qos_with_type(self.ctxt, qos_ref3['id'],
+                                          type_ref2['id'])
+        diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
+                                                    type_ref2['id'])
+        self.assertEqual(same, False)
+        self.assertEqual(diff['extra_specs']['key1'], ('val1', 'val1'))
+        self.assertEqual(diff['qos_specs']['k1'], ('v1', 'v0'))
+        qos_specs.disassociate_qos_specs(self.ctxt, qos_ref3['id'],
+                                         type_ref2['id'])
+        qos_specs.associate_qos_with_type(self.ctxt, qos_ref2['id'],
+                                          type_ref2['id'])
+
+        # And add encryption for good measure
+        enc_keyvals1 = {'cipher': 'c1', 'key_size': 256, 'provider': 'p1',
+                        'control_location': 'front-end'}
+        enc_keyvals2 = {'cipher': 'c1', 'key_size': 128, 'provider': 'p1',
+                        'control_location': 'front-end'}
+        db.volume_type_encryption_create(self.ctxt, type_ref1['id'],
+                                         enc_keyvals1)
+        db.volume_type_encryption_create(self.ctxt, type_ref2['id'],
+                                         enc_keyvals2)
+        diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
+                                                    type_ref2['id'])
+        self.assertEqual(same, False)
+        self.assertEqual(diff['extra_specs']['key1'], ('val1', 'val1'))
+        self.assertEqual(diff['qos_specs']['k1'], ('v1', 'v1'))
+        self.assertEqual(diff['encryption']['key_size'], (256, 128))
