@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2011 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -18,20 +16,19 @@
 
 """Unit tests for `cinder.wsgi`."""
 
+import mock
 import os.path
-import ssl
 import tempfile
-import unittest
 import urllib2
 
 from oslo.config import cfg
+import testtools
 import webob
 import webob.dec
 
-from cinder.api.middleware import fault
 from cinder import exception
+from cinder.openstack.common import gettextutils
 from cinder import test
-from cinder import utils
 import cinder.wsgi
 
 CONF = cfg.CONF
@@ -54,7 +51,7 @@ class TestLoaderNothingExists(test.TestCase):
         )
 
 
-class TestLoaderNormalFilesystem(unittest.TestCase):
+class TestLoaderNormalFilesystem(test.TestCase):
     """Loader tests with normal filesystem (unmodified os.path module)."""
 
     _paste_config = """
@@ -64,14 +61,16 @@ document_root = /tmp
     """
 
     def setUp(self):
+        super(TestLoaderNormalFilesystem, self).setUp()
         self.config = tempfile.NamedTemporaryFile(mode="w+t")
         self.config.write(self._paste_config.lstrip())
         self.config.seek(0)
         self.config.flush()
         self.loader = cinder.wsgi.Loader(self.config.name)
+        self.addCleanup(self.config.close)
 
     def test_config_found(self):
-        self.assertEquals(self.config.name, self.loader.config_path)
+        self.assertEqual(self.config.name, self.loader.config_path)
 
     def test_app_not_found(self):
         self.assertRaises(
@@ -82,27 +81,21 @@ document_root = /tmp
 
     def test_app_found(self):
         url_parser = self.loader.load_app("test_app")
-        self.assertEquals("/tmp", url_parser.directory)
-
-    def tearDown(self):
-        self.config.close()
+        self.assertEqual("/tmp", url_parser.directory)
 
 
-class TestWSGIServer(unittest.TestCase):
+class TestWSGIServer(test.TestCase):
     """WSGI server tests."""
     def _ipv6_configured():
         try:
-            out, err = utils.execute('cat', '/proc/net/if_inet6')
-        except exception.ProcessExecutionError:
+            with file('/proc/net/if_inet6') as f:
+                return len(f.read()) > 0
+        except IOError:
             return False
 
-        if not out:
-            return False
-        return True
-
     def test_no_app(self):
         server = cinder.wsgi.Server("test_app", None)
-        self.assertEquals("test_app", server.name)
+        self.assertEqual("test_app", server.name)
 
     def test_start_random_port(self):
         server = cinder.wsgi.Server("test_random_port", None, host="127.0.0.1")
@@ -112,8 +105,8 @@ class TestWSGIServer(unittest.TestCase):
         server.stop()
         server.wait()
 
-    @test.skip_if(not _ipv6_configured(),
-                  "Test requires an IPV6 configured interface")
+    @testtools.skipIf(not _ipv6_configured(),
+                      "Test requires an IPV6 configured interface")
     def test_start_random_port_with_ipv6(self):
         server = cinder.wsgi.Server("test_random_port",
                                     None,
@@ -139,7 +132,7 @@ class TestWSGIServer(unittest.TestCase):
         server.start()
 
         response = urllib2.urlopen('http://127.0.0.1:%d/' % server.port)
-        self.assertEquals(greetings, response.read())
+        self.assertEqual(greetings, response.read())
 
         server.stop()
 
@@ -159,12 +152,12 @@ class TestWSGIServer(unittest.TestCase):
         server.start()
 
         response = urllib2.urlopen('https://127.0.0.1:%d/' % server.port)
-        self.assertEquals(greetings, response.read())
+        self.assertEqual(greetings, response.read())
 
         server.stop()
 
-    @test.skip_if(not _ipv6_configured(),
-                  "Test requires an IPV6 configured interface")
+    @testtools.skipIf(not _ipv6_configured(),
+                      "Test requires an IPV6 configured interface")
     def test_app_using_ipv6_and_ssl(self):
         CONF.set_default("ssl_cert_file",
                          os.path.join(TEST_VAR_DIR, 'certificate.crt'))
@@ -184,7 +177,7 @@ class TestWSGIServer(unittest.TestCase):
         server.start()
 
         response = urllib2.urlopen('https://[::1]:%d/' % server.port)
-        self.assertEquals(greetings, response.read())
+        self.assertEqual(greetings, response.read())
 
         server.stop()
 
@@ -192,6 +185,13 @@ class TestWSGIServer(unittest.TestCase):
 class ExceptionTest(test.TestCase):
 
     def _wsgi_app(self, inner_app):
+        # NOTE(luisg): In order to test localization, we need to
+        # make sure the lazy _() is installed in the 'fault' module
+        # also we don't want to install the _() system-wide and
+        # potentially break other test cases, so we do it here for this
+        # test suite only.
+        gettextutils.install('', lazy=True)
+        from cinder.api.middleware import fault
         return fault.FaultWrapper(inner_app)
 
     def _do_test_exception_safety_reflected_in_faults(self, expose):
@@ -204,11 +204,11 @@ class ExceptionTest(test.TestCase):
 
         api = self._wsgi_app(fail)
         resp = webob.Request.blank('/').get_response(api)
-        self.assertTrue('{"computeFault' in resp.body, resp.body)
+        self.assertIn('{"computeFault', resp.body)
         expected = ('ExceptionWithSafety: some explanation' if expose else
                     'The server has either erred or is incapable '
                     'of performing the requested operation.')
-        self.assertTrue(expected in resp.body, resp.body)
+        self.assertIn(expected, resp.body)
         self.assertEqual(resp.status_int, 500, resp.body)
 
     def test_safe_exceptions_are_described_in_faults(self):
@@ -224,13 +224,13 @@ class ExceptionTest(test.TestCase):
 
         api = self._wsgi_app(fail)
         resp = webob.Request.blank('/').get_response(api)
-        self.assertTrue(msg in resp.body, resp.body)
+        self.assertIn(msg, resp.body)
         self.assertEqual(resp.status_int, exception_type.code, resp.body)
 
         if hasattr(exception_type, 'headers'):
             for (key, value) in exception_type.headers.iteritems():
-                self.assertTrue(key in resp.headers)
-                self.assertEquals(resp.headers[key], value)
+                self.assertIn(key, resp.headers)
+                self.assertEqual(resp.headers[key], value)
 
     def test_quota_error_mapping(self):
         self._do_test_exception_mapping(exception.QuotaError, 'too many used')
@@ -262,3 +262,41 @@ class ExceptionTest(test.TestCase):
         api = self._wsgi_app(fail)
         resp = webob.Request.blank('/').get_response(api)
         self.assertEqual(500, resp.status_int)
+
+    @mock.patch('cinder.openstack.common.gettextutils.translate')
+    def test_cinder_exception_with_localized_explanation(self, mock_t9n):
+        msg = 'My Not Found'
+        msg_translation = 'Mi No Encontrado'
+        message = gettextutils.Message(msg, '')
+
+        @webob.dec.wsgify
+        def fail(req):
+            class MyVolumeNotFound(exception.NotFound):
+                def __init__(self):
+                    self.msg = message
+                    self.safe = True
+            raise MyVolumeNotFound()
+
+        # Test response without localization
+        def mock_get_non_localized_message(msgid, locale):
+            return msg
+
+        mock_t9n.side_effect = mock_get_non_localized_message
+
+        api = self._wsgi_app(fail)
+        resp = webob.Request.blank('/').get_response(api)
+        self.assertEqual(404, resp.status_int)
+        self.assertIn(msg, resp.body)
+
+        # Test response with localization
+        def mock_translate(msgid, locale):
+            if isinstance(msgid, gettextutils.Message):
+                return msg_translation
+            return msgid
+
+        mock_t9n.side_effect = mock_translate
+
+        api = self._wsgi_app(fail)
+        resp = webob.Request.blank('/').get_response(api)
+        self.assertEqual(404, resp.status_int)
+        self.assertIn(msg_translation, resp.body)
