@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # Copyright 2011 Justin Santa Barbara
@@ -32,23 +30,19 @@ import shutil
 import stat
 import sys
 import tempfile
-import time
+
+from eventlet import pools
+from oslo.config import cfg
+import paramiko
+import six
 from xml.dom import minidom
 from xml.parsers import expat
 from xml import sax
 from xml.sax import expatreader
 from xml.sax import saxutils
 
-from eventlet import event
-from eventlet import greenthread
-from eventlet import pools
-from oslo.config import cfg
-import paramiko
-
 from cinder.brick.initiator import connector
 from cinder import exception
-from cinder.openstack.common import excutils
-from cinder.openstack.common import gettextutils
 from cinder.openstack.common import importutils
 from cinder.openstack.common import lockutils
 from cinder.openstack.common import log as logging
@@ -158,12 +152,12 @@ def check_ssh_injection(cmd_list):
             if quoted:
                 if (re.match('[\'"]', quoted) or
                         re.search('[^\\\\][\'"]', quoted)):
-                    raise exception.SSHInjectionThreat(command=str(cmd_list))
+                    raise exception.SSHInjectionThreat(command=cmd_list)
         else:
             # We only allow spaces within quoted arguments, and that
             # is the only special character allowed within quotes
             if len(arg.split()) > 1:
-                raise exception.SSHInjectionThreat(command=str(cmd_list))
+                raise exception.SSHInjectionThreat(command=cmd_list)
 
         # Second, check whether danger character in command. So the shell
         # special operator must be a single argument.
@@ -220,7 +214,7 @@ class SSHPool(pools.Pool):
                 raise exception.CinderException(msg)
 
             # Paramiko by default sets the socket timeout to 0.1 seconds,
-            # ignoring what we set thru the sshclient. This doesn't help for
+            # ignoring what we set through the sshclient. This doesn't help for
             # keeping long lived connections. Hence we have to bypass it, by
             # overriding it after the transport is initialized. We are setting
             # the sockettimeout to None and setting a keepalive packet so that,
@@ -237,11 +231,12 @@ class SSHPool(pools.Pool):
             raise paramiko.SSHException(msg)
 
     def get(self):
-        """
-        Return an item from the pool, when one is available.  This may
-        cause the calling greenthread to block. Check if a connection is active
-        before returning it. For dead connections create and return a new
-        connection.
+        """Return an item from the pool, when one is available.
+
+        This may cause the calling greenthread to block. Check if a
+        connection is active before returning it.
+
+        For dead connections create and return a new connection.
         """
         conn = super(SSHPool, self).get()
         if conn:
@@ -434,66 +429,6 @@ class LazyPluggable(object):
         return getattr(backend, key)
 
 
-class LoopingCallDone(Exception):
-    """Exception to break out and stop a LoopingCall.
-
-    The poll-function passed to LoopingCall can raise this exception to
-    break out of the loop normally. This is somewhat analogous to
-    StopIteration.
-
-    An optional return-value can be included as the argument to the exception;
-    this return-value will be returned by LoopingCall.wait()
-
-    """
-
-    def __init__(self, retvalue=True):
-        """:param retvalue: Value that LoopingCall.wait() should return."""
-        self.retvalue = retvalue
-
-
-class LoopingCall(object):
-    def __init__(self, f=None, *args, **kw):
-        self.args = args
-        self.kw = kw
-        self.f = f
-        self._running = False
-
-    def start(self, interval, initial_delay=None):
-        self._running = True
-        done = event.Event()
-
-        def _inner():
-            if initial_delay:
-                greenthread.sleep(initial_delay)
-
-            try:
-                while self._running:
-                    self.f(*self.args, **self.kw)
-                    if not self._running:
-                        break
-                    greenthread.sleep(interval)
-            except LoopingCallDone as e:
-                self.stop()
-                done.send(e.retvalue)
-            except Exception:
-                LOG.exception(_('in looping call'))
-                done.send_exception(*sys.exc_info())
-                return
-            else:
-                done.send(True)
-
-        self.done = done
-
-        greenthread.spawn(_inner)
-        return self.done
-
-    def stop(self):
-        self._running = False
-
-    def wait(self):
-        return self.done.wait()
-
-
 class ProtectedExpatParser(expatreader.ExpatParser):
     """An expat parser which disables DTD's and entities by default."""
 
@@ -667,7 +602,7 @@ def make_dev_path(dev, partition=None, base='/dev'):
 
 
 def total_seconds(td):
-    """Local total_seconds implementation for compatibility with python 2.6"""
+    """Local total_seconds implementation for compatibility with python 2.6."""
     if hasattr(td, 'total_seconds'):
         return td.total_seconds()
     else:
@@ -761,11 +696,11 @@ def tempdir(**kwargs):
         try:
             shutil.rmtree(tmpdir)
         except OSError as e:
-            LOG.debug(_('Could not remove tmpdir: %s'), str(e))
+            LOG.debug(_('Could not remove tmpdir: %s'), e)
 
 
 def walk_class_hierarchy(clazz, encountered=None):
-    """Walk class hierarchy, yielding most derived classes first"""
+    """Walk class hierarchy, yielding most derived classes first."""
     if not encountered:
         encountered = []
     for subclass in clazz.__subclasses__():
@@ -811,16 +746,6 @@ def brick_get_connector(protocol, driver=None,
                                                 *args, **kwargs)
 
 
-def get_file_mode(path):
-    """This primarily exists to make unit testing easier."""
-    return stat.S_IMODE(os.stat(path).st_mode)
-
-
-def get_file_gid(path):
-    """This primarily exists to make unit testing easier."""
-    return os.stat(path).st_gid
-
-
 def require_driver_initialized(driver):
     """Verifies if `driver` is initialized
 
@@ -834,3 +759,89 @@ def require_driver_initialized(driver):
         driver_name = driver.__class__.__name__
         LOG.error(_("Volume driver %s not initialized") % driver_name)
         raise exception.DriverNotInitialized()
+
+
+def get_file_mode(path):
+    """This primarily exists to make unit testing easier."""
+    return stat.S_IMODE(os.stat(path).st_mode)
+
+
+def get_file_gid(path):
+    """This primarily exists to make unit testing easier."""
+    return os.stat(path).st_gid
+
+
+def check_string_length(value, name, min_length=0, max_length=None):
+    """Check the length of specified string
+    :param value: the value of the string
+    :param name: the name of the string
+    :param min_length: the min_length of the string
+    :param max_length: the max_length of the string
+    """
+    if not isinstance(value, six.string_types):
+        msg = _("%s is not a string or unicode") % name
+        raise exception.InvalidInput(message=msg)
+
+    if len(value) < min_length:
+        msg = _("%(name)s has a minimum character requirement of "
+                "%(min_length)s.") % {'name': name, 'min_length': min_length}
+        raise exception.InvalidInput(message=msg)
+
+    if max_length and len(value) > max_length:
+        msg = _("%(name)s has more than %(max_length)s "
+                "characters.") % {'name': name, 'max_length': max_length}
+        raise exception.InvalidInput(message=msg)
+
+_visible_admin_metadata_keys = ['readonly', 'attached_mode']
+
+
+def add_visible_admin_metadata(context, volume, volume_api):
+    """Add user-visible admin metadata to regular metadata.
+
+    Extracts the admin metadata keys that are to be made visible to
+    non-administrators, and adds them to the regular metadata structure for the
+    passed-in volume.
+    """
+    if context is None:
+        return
+
+    visible_admin_meta = {}
+
+    if context.is_admin:
+        volume_tmp = volume
+    else:
+        try:
+            volume_tmp = volume_api.get(context.elevated(), volume['id'])
+        except Exception:
+            return
+
+    if volume_tmp.get('volume_admin_metadata'):
+        for item in volume_tmp['volume_admin_metadata']:
+            if item['key'] in _visible_admin_metadata_keys:
+                visible_admin_meta[item['key']] = item['value']
+    # avoid circular ref when volume is a Volume instance
+    elif (volume_tmp.get('admin_metadata') and
+            isinstance(volume_tmp.get('admin_metadata'), dict)):
+        for key in _visible_admin_metadata_keys:
+            if key in volume_tmp['admin_metadata'].keys():
+                visible_admin_meta[key] = volume_tmp['admin_metadata'][key]
+
+    if not visible_admin_meta:
+        return
+
+    # NOTE(zhiyan): update visible administration metadata to
+    # volume metadata, administration metadata will rewrite existing key.
+    if volume.get('volume_metadata'):
+        orig_meta = list(volume.get('volume_metadata'))
+        for item in orig_meta:
+            if item['key'] in visible_admin_meta.keys():
+                item['value'] = visible_admin_meta.pop(item['key'])
+        for key, value in visible_admin_meta.iteritems():
+            orig_meta.append({'key': key, 'value': value})
+        volume['volume_metadata'] = orig_meta
+    # avoid circular ref when vol is a Volume instance
+    elif (volume.get('metadata') and
+            isinstance(volume.get('metadata'), dict)):
+        volume['metadata'].update(visible_admin_meta)
+    else:
+        volume['metadata'] = visible_admin_meta
