@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -34,18 +32,18 @@ from cinder import keymgr
 from cinder.openstack.common import excutils
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import timeutils
+from cinder.openstack.common import uuidutils
 import cinder.policy
 from cinder import quota
+from cinder import quota_utils
 from cinder.scheduler import rpcapi as scheduler_rpcapi
-from cinder import units
 from cinder import utils
-from cinder.volume.flows import create_volume
+from cinder.volume.flows.api import create_volume
+from cinder.volume import qos_specs
 from cinder.volume import rpcapi as volume_rpcapi
 from cinder.volume import utils as volume_utils
 from cinder.volume import volume_types
 
-from cinder.taskflow import states
-
 
 volume_host_opt = cfg.BoolOpt('snapshot_same_host',
                               default=True,
@@ -101,7 +99,7 @@ class API(base.Base):
         self.key_manager = keymgr.API()
         super(API, self).__init__(db_driver)
 
-    def _valid_availabilty_zone(self, availability_zone):
+    def _valid_availability_zone(self, availability_zone):
         #NOTE(bcwaldon): This approach to caching fails to handle the case
         # that an availability zone is disabled/removed.
         if availability_zone in self.availability_zone_names:
@@ -138,55 +136,62 @@ class API(base.Base):
                availability_zone=None, source_volume=None,
                scheduler_hints=None, backup_source_volume=None):
 
+        if source_volume and volume_type:
+            if volume_type['id'] != source_volume['volume_type_id']:
+                msg = _("Invalid volume_type provided (requested type "
+                        "must match source volume, or be omitted). "
+                        "You should omit the argument.")
+                raise exception.InvalidInput(reason=msg)
+
+        if snapshot and volume_type:
+            if volume_type['id'] != snapshot['volume_type_id']:
+                msg = _("Invalid volume_type provided (requested type "
+                        "must match source snapshot, or be omitted). "
+                        "You should omit the argument.")
+                raise exception.InvalidInput(reason=msg)
+
         def check_volume_az_zone(availability_zone):
             try:
-                return self._valid_availabilty_zone(availability_zone)
+                return self._valid_availability_zone(availability_zone)
             except exception.CinderException:
                 LOG.exception(_("Unable to query if %s is in the "
                                 "availability zone set"), availability_zone)
                 return False
 
         create_what = {
-            'size': size,
+            'context': context,
+            'raw_size': size,
             'name': name,
             'description': description,
             'snapshot': snapshot,
             'image_id': image_id,
-            'volume_type': volume_type,
+            'raw_volume_type': volume_type,
             'metadata': metadata,
-            'availability_zone': availability_zone,
+            'raw_availability_zone': availability_zone,
             'source_volume': source_volume,
             'scheduler_hints': scheduler_hints,
             'key_manager': self.key_manager,
             'backup_source_volume': backup_source_volume,
         }
-        (flow, uuid) = create_volume.get_api_flow(self.scheduler_rpcapi,
-                                                  self.volume_rpcapi,
-                                                  self.db,
-                                                  self.image_service,
-                                                  check_volume_az_zone,
-                                                  create_what)
 
-        assert flow, _('Create volume flow not retrieved')
-        flow.run(context)
-        if flow.state != states.SUCCESS:
-            raise exception.CinderException(_("Failed to successfully complete"
-                                              " create volume workflow"))
-
-        # Extract the volume information from the task uuid that was specified
-        # to produce said information.
-        volume = None
         try:
-            volume = flow.results[uuid]['volume']
-        except KeyError:
-            pass
+            flow_engine = create_volume.get_flow(self.scheduler_rpcapi,
+                                                 self.volume_rpcapi,
+                                                 self.db,
+                                                 self.image_service,
+                                                 check_volume_az_zone,
+                                                 create_what)
+        except Exception:
+            LOG.exception(_("Failed to create api volume flow"))
+            raise exception.CinderException(
+                _("Failed to create api volume flow"))
 
-        # Raise an error, nobody provided it??
-        assert volume, _('Expected volume result not found')
+        flow_engine.run()
+        volume = flow_engine.storage.fetch('volume')
         return volume
 
     @wrap_check_policy
-    def delete(self, context, volume, force=False):
+    def delete(self, context, volume, force=False, unmanage_only=False):
         if context.is_admin and context.project_id != volume['project_id']:
             project_id = volume['project_id']
         else:
@@ -228,7 +233,7 @@ class API(base.Base):
             # Volume is still attached, need to detach first
             raise exception.VolumeAttached(volume_id=volume_id)
 
-        if volume['migration_status'] != None:
+        if volume['migration_status'] is not None:
             # Volume is migrating, wait until done
             msg = _("Volume cannot be deleted while migrating")
             raise exception.InvalidVolume(reason=msg)
@@ -249,7 +254,7 @@ class API(base.Base):
         self.db.volume_update(context, volume_id, {'status': 'deleting',
                                                    'terminated_at': now})
 
-        self.volume_rpcapi.delete_volume(context, volume)
+        self.volume_rpcapi.delete_volume(context, volume, unmanage_only)
 
     @wrap_check_policy
     def update(self, context, volume, fields):
@@ -262,8 +267,10 @@ class API(base.Base):
         return volume
 
     def get_all(self, context, marker=None, limit=None, sort_key='created_at',
-                sort_dir='desc', filters={}):
+                sort_dir='desc', filters=None):
         check_policy(context, 'get_all')
+        if filters == None:
+            filters = {}
 
         try:
             if limit is not None:
@@ -275,60 +282,27 @@ class API(base.Base):
             msg = _('limit param must be an integer')
             raise exception.InvalidInput(reason=msg)
 
-        if (context.is_admin and 'all_tenants' in filters):
-            # Need to remove all_tenants to pass the filtering below.
-            del filters['all_tenants']
-            volumes = self.db.volume_get_all(context, marker, limit, sort_key,
-                                             sort_dir)
-        else:
-            volumes = self.db.volume_get_all_by_project(context,
-                                                        context.project_id,
-                                                        marker, limit,
-                                                        sort_key, sort_dir)
-
-        # Non-admin shouldn't see temporary target of a volume migration
+        # Non-admin shouldn't see temporary target of a volume migration, add
+        # unique filter data to reflect that only volumes with a NULL
+        # 'migration_status' or a 'migration_status' that does not start with
+        # 'target:' should be returned (processed in db/sqlalchemy/api.py)
         if not context.is_admin:
             filters['no_migration_targets'] = True
 
         if filters:
             LOG.debug(_("Searching by: %s") % str(filters))
 
-            def _check_metadata_match(volume, searchdict):
-                volume_metadata = {}
-                for i in volume.get('volume_metadata'):
-                    volume_metadata[i['key']] = i['value']
-
-                for k, v in searchdict.iteritems():
-                    if (k not in volume_metadata.keys() or
-                            volume_metadata[k] != v):
-                        return False
-                return True
-
-            def _check_migration_target(volume, searchdict):
-                status = volume['migration_status']
-                if status and status.startswith('target:'):
-                    return False
-                return True
-
-            # search_option to filter_name mapping.
-            filter_mapping = {'metadata': _check_metadata_match,
-                              'no_migration_targets': _check_migration_target}
-
-            result = []
-            not_found = object()
-            for volume in volumes:
-                # go over all filters in the list
-                for opt, values in filters.iteritems():
-                    try:
-                        filter_func = filter_mapping[opt]
-                    except KeyError:
-                        def filter_func(volume, value):
-                            return volume.get(opt, not_found) == value
-                    if not filter_func(volume, values):
-                        break  # volume doesn't match this filter
-                else:  # did not break out loop
-                    result.append(volume)  # volume matches all filters
-            volumes = result
+        if (context.is_admin and 'all_tenants' in filters):
+            # Need to remove all_tenants to pass the filtering below.
+            del filters['all_tenants']
+            volumes = self.db.volume_get_all(context, marker, limit, sort_key,
+                                             sort_dir, filters=filters)
+        else:
+            volumes = self.db.volume_get_all_by_project(context,
+                                                        context.project_id,
+                                                        marker, limit,
+                                                        sort_key, sort_dir,
+                                                        filters=filters)
 
         return volumes
 
@@ -356,7 +330,7 @@ class API(base.Base):
                 context, context.project_id)
 
         if search_opts:
-            LOG.debug(_("Searching by: %s") % str(search_opts))
+            LOG.debug(_("Searching by: %s") % search_opts)
 
             results = []
             not_found = object()
@@ -370,7 +344,7 @@ class API(base.Base):
         return snapshots
 
     @wrap_check_policy
-    def check_attach(self, context, volume):
+    def check_attach(self, volume):
         # TODO(vish): abstract status checking?
         if volume['status'] != "available":
             msg = _("status must be available")
@@ -380,7 +354,7 @@ class API(base.Base):
             raise exception.InvalidVolume(reason=msg)
 
     @wrap_check_policy
-    def check_detach(self, context, volume):
+    def check_detach(self, volume):
         # TODO(vish): abstract status checking?
         if volume['status'] != "in-use":
             msg = _("status must be in-use to detach")
@@ -468,9 +442,9 @@ class API(base.Base):
                          force=False, metadata=None):
         check_policy(context, 'create_snapshot', volume)
 
-        if volume['migration_status'] != None:
+        if volume['migration_status'] is not None:
             # Volume is migrating, wait until done
-            msg = _("Volume cannot be deleted while migrating")
+            msg = _("Snapshot cannot be created while volume is migrating")
             raise exception.InvalidVolume(reason=msg)
 
         if ((not force) and (volume['status'] != "available")):
@@ -503,7 +477,10 @@ class API(base.Base):
                                     's_size': volume['size'],
                                     'd_consumed': _consumed(over),
                                     'd_quota': quotas[over]})
-                    raise exception.VolumeSizeExceedsAvailableQuota()
+                    raise exception.VolumeSizeExceedsAvailableQuota(
+                        requested=volume['size'],
+                        consumed=_consumed('gigabytes'),
+                        quota=quotas['gigabytes'])
                 elif 'snapshots' in over:
                     msg = _("Quota exceeded for %(s_pid)s, tried to create "
                             "snapshot (%(d_consumed)d snapshots "
@@ -514,7 +491,7 @@ class API(base.Base):
                     raise exception.SnapshotLimitExceeded(
                         allowed=quotas[over])
 
-        self._check_metadata_properties(context, metadata)
+        self._check_metadata_properties(metadata)
         options = {'volume_id': volume['id'],
                    'user_id': context.user_id,
                    'project_id': context.project_id,
@@ -578,7 +555,7 @@ class API(base.Base):
         """Delete the given metadata item from a volume."""
         self.db.volume_metadata_delete(context, volume['id'], key)
 
-    def _check_metadata_properties(self, context, metadata=None):
+    def _check_metadata_properties(self, metadata=None):
         if not metadata:
             metadata = {}
 
@@ -604,21 +581,21 @@ class API(base.Base):
         `metadata` argument will be deleted.
 
         """
-        orig_meta = self.get_volume_metadata(context, volume)
         if delete:
             _metadata = metadata
         else:
+            orig_meta = self.get_volume_metadata(context, volume)
             _metadata = orig_meta.copy()
             _metadata.update(metadata)
 
-        self._check_metadata_properties(context, _metadata)
+        self._check_metadata_properties(_metadata)
 
-        self.db.volume_metadata_update(context, volume['id'],
-                                       _metadata, delete)
+        db_meta = self.db.volume_metadata_update(context, volume['id'],
+                                                 _metadata, delete)
 
         # TODO(jdg): Implement an RPC call for drivers that may use this info
 
-        return _metadata
+        return db_meta
 
     def get_volume_metadata_value(self, volume, key):
         """Get value of particular metadata key."""
@@ -649,14 +626,14 @@ class API(base.Base):
         `metadata` argument will be deleted.
 
         """
-        orig_meta = self.get_volume_admin_metadata(context, volume)
         if delete:
             _metadata = metadata
         else:
+            orig_meta = self.get_volume_admin_metadata(context, volume)
             _metadata = orig_meta.copy()
             _metadata.update(metadata)
 
-        self._check_metadata_properties(context, _metadata)
+        self._check_metadata_properties(_metadata)
 
         self.db.volume_admin_metadata_update(context, volume['id'],
                                              _metadata, delete)
@@ -683,23 +660,23 @@ class API(base.Base):
         `metadata` argument will be deleted.
 
         """
-        orig_meta = self.get_snapshot_metadata(context, snapshot)
         if delete:
             _metadata = metadata
         else:
+            orig_meta = self.get_snapshot_metadata(context, snapshot)
             _metadata = orig_meta.copy()
             _metadata.update(metadata)
 
-        self._check_metadata_properties(context, _metadata)
+        self._check_metadata_properties(_metadata)
 
-        self.db.snapshot_metadata_update(context,
-                                         snapshot['id'],
-                                         _metadata,
-                                         True)
+        db_meta = self.db.snapshot_metadata_update(context,
+                                                   snapshot['id'],
+                                                   _metadata,
+                                                   True)
 
         # TODO(jdg): Implement an RPC call for drivers that may use this info
 
-        return _metadata
+        return db_meta
 
     def get_snapshot_metadata_value(self, snapshot, key):
         pass
@@ -720,7 +697,7 @@ class API(base.Base):
             (meta_entry.key, meta_entry.value) for meta_entry in db_data
         )
 
-    def _check_volume_availability(self, context, volume, force):
+    def _check_volume_availability(self, volume, force):
         """Check if the volume can be used."""
         if volume['status'] not in ['available', 'in-use']:
             msg = _('Volume status must be available/in-use.')
@@ -732,7 +709,7 @@ class API(base.Base):
     @wrap_check_policy
     def copy_volume_to_image(self, context, volume, metadata, force):
         """Create a new image from the specified volume."""
-        self._check_volume_availability(context, volume, force)
+        self._check_volume_availability(volume, force)
 
         recv_metadata = self.image_service.create(context, metadata)
         self.update(context, volume, {'status': 'uploading'})
@@ -766,8 +743,30 @@ class API(base.Base):
                                                    'size': volume['size']})
             raise exception.InvalidInput(reason=msg)
 
+        try:
+            reservations = QUOTAS.reserve(context, gigabytes=+size_increase)
+        except exception.OverQuota as exc:
+            usages = exc.kwargs['usages']
+            quotas = exc.kwargs['quotas']
+
+            def _consumed(name):
+                return (usages[name]['reserved'] + usages[name]['in_use'])
+
+            msg = _("Quota exceeded for %(s_pid)s, tried to extend volume by "
+                    "%(s_size)sG, (%(d_consumed)dG of %(d_quota)dG already "
+                    "consumed).")
+            LOG.error(msg % {'s_pid': context.project_id,
+                             's_size': size_increase,
+                             'd_consumed': _consumed('gigabytes'),
+                             'd_quota': quotas['gigabytes']})
+            raise exception.VolumeSizeExceedsAvailableQuota(
+                requested=size_increase,
+                consumed=_consumed('gigabytes'),
+                quota=quotas['gigabytes'])
+
         self.update(context, volume, {'status': 'extending'})
-        self.volume_rpcapi.extend_volume(context, volume, new_size)
+        self.volume_rpcapi.extend_volume(context, volume, new_size,
+                                         reservations)
 
     @wrap_check_policy
     def migrate_volume(self, context, volume, host, force_host_copy):
@@ -780,7 +779,7 @@ class API(base.Base):
             raise exception.InvalidVolume(reason=msg)
 
         # Make sure volume is not part of a migration
-        if volume['migration_status'] != None:
+        if volume['migration_status'] is not None:
             msg = _("Volume is already part of an active migration")
             raise exception.InvalidVolume(reason=msg)
 
@@ -861,6 +860,137 @@ class API(base.Base):
         self.update_volume_admin_metadata(context.elevated(), volume,
                                           {'readonly': str(flag)})
 
+    @wrap_check_policy
+    def retype(self, context, volume, new_type, migration_policy=None):
+        """Attempt to modify the type associated with an existing volume."""
+        if volume['status'] not in ['available', 'in-use']:
+            msg = _('Unable to update type due to incorrect status '
+                    'on volume: %s') % volume['id']
+            LOG.error(msg)
+            raise exception.InvalidVolume(reason=msg)
+
+        if volume['migration_status'] is not None:
+            msg = (_("Volume %s is already part of an active migration.")
+                   % volume['id'])
+            LOG.error(msg)
+            raise exception.InvalidVolume(reason=msg)
+
+        if migration_policy and migration_policy not in ['on-demand', 'never']:
+            msg = _('migration_policy must be \'on-demand\' or \'never\', '
+                    'passed: %s') % new_type
+            LOG.error(msg)
+            raise exception.InvalidInput(reason=msg)
+
+        # Support specifying volume type by ID or name
+        try:
+            if uuidutils.is_uuid_like(new_type):
+                vol_type = volume_types.get_volume_type(context, new_type)
+            else:
+                vol_type = volume_types.get_volume_type_by_name(context,
+                                                                new_type)
+        except exception.InvalidVolumeType:
+            msg = _('Invalid volume_type passed: %s') % new_type
+            LOG.error(msg)
+            raise exception.InvalidInput(reason=msg)
+
+        vol_type_id = vol_type['id']
+        vol_type_qos_id = vol_type['qos_specs_id']
+
+        old_vol_type = None
+        old_vol_type_id = volume['volume_type_id']
+        old_vol_type_qos_id = None
+
+        # Error if the original and new type are the same
+        if volume['volume_type_id'] == vol_type_id:
+            msg = (_('New volume_type same as original: %s') % new_type)
+            LOG.error(msg)
+            raise exception.InvalidInput(reason=msg)
+
+        if volume['volume_type_id']:
+            old_vol_type = volume_types.get_volume_type(
+                context, old_vol_type_id)
+            old_vol_type_qos_id = old_vol_type['qos_specs_id']
+
+        # We don't support changing encryption requirements yet
+        old_enc = volume_types.get_volume_type_encryption(context,
+                                                          old_vol_type_id)
+        new_enc = volume_types.get_volume_type_encryption(context,
+                                                          vol_type_id)
+        if old_enc != new_enc:
+            msg = _('Retype cannot change encryption requirements')
+            raise exception.InvalidInput(reason=msg)
+
+        # We don't support changing QoS at the front-end yet for in-use volumes
+        # TODO(avishay): Call Nova to change QoS setting (libvirt has support
+        # - virDomainSetBlockIoTune() - Nova does not have support yet).
+        if (volume['status'] != 'available' and
+                old_vol_type_qos_id != vol_type_qos_id):
+            for qos_id in [old_vol_type_qos_id, vol_type_qos_id]:
+                if qos_id:
+                    specs = qos_specs.get_qos_specs(context.elevated(), qos_id)
+                    if specs['qos_specs']['consumer'] != 'back-end':
+                        msg = _('Retype cannot change front-end qos specs for '
+                                'in-use volumes')
+                        raise exception.InvalidInput(reason=msg)
+
+        self.update(context, volume, {'status': 'retyping'})
+
+        # We're checking here in so that we can report any quota issues as
+        # early as possible, but won't commit until we change the type. We
+        # pass the reservations onward in case we need to roll back.
+        reservations = quota_utils.get_volume_type_reservation(context, volume,
+                                                               vol_type_id)
+        request_spec = {'volume_properties': volume,
+                        'volume_id': volume['id'],
+                        'volume_type': vol_type,
+                        'migration_policy': migration_policy,
+                        'quota_reservations': reservations}
+
+        self.scheduler_rpcapi.retype(context, CONF.volume_topic, volume['id'],
+                                     request_spec=request_spec,
+                                     filter_properties={})
+
+    def manage_existing(self, context, host, ref, name=None, description=None,
+                        volume_type=None, metadata=None,
+                        availability_zone=None):
+        if availability_zone is None:
+            elevated = context.elevated()
+            try:
+                service = self.db.service_get_by_host_and_topic(
+                    elevated, host, CONF.volume_topic)
+            except exception.ServiceNotFound:
+                with excutils.save_and_reraise_exception():
+                    LOG.error(_('Unable to find service for given host.'))
+            availability_zone = service.get('availability_zone')
+
+        volume_type_id = volume_type['id'] if volume_type else None
+        volume_properties = {
+            'size': 0,
+            'user_id': context.user_id,
+            'project_id': context.project_id,
+            'status': 'creating',
+            'attach_status': 'detached',
+            # Rename these to the internal name.
+            'display_description': description,
+            'display_name': name,
+            'host': host,
+            'availability_zone': availability_zone,
+            'volume_type_id': volume_type_id,
+            'metadata': metadata
+        }
+
+        # Call the scheduler to ensure that the host exists and that it can
+        # accept the volume
+        volume = self.db.volume_create(context, volume_properties)
+        request_spec = {'volume_properties': volume,
+                        'volume_type': volume_type,
+                        'volume_id': volume['id'],
+                        'ref': ref}
+        self.scheduler_rpcapi.manage_existing(context, CONF.volume_topic,
+                                              volume['id'],
+                                              request_spec=request_spec)
+        return volume
+
 
 class HostAPI(base.Base):
     def __init__(self):
