@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2013 Red Hat, Inc.
 # All Rights Reserved.
 #
@@ -19,18 +17,19 @@ import errno
 import hashlib
 import json
 import os
-import re
 import stat
 import tempfile
 import time
 
 from oslo.config import cfg
 
+from cinder.brick.remotefs import remotefs
 from cinder import compute
 from cinder import db
 from cinder import exception
 from cinder.image import image_utils
 from cinder.openstack.common import log as logging
+from cinder.openstack.common import processutils
 from cinder import units
 from cinder import utils
 from cinder.volume.drivers import nfs
@@ -41,9 +40,6 @@ volume_opts = [
     cfg.StrOpt('glusterfs_shares_config',
                default='/etc/cinder/glusterfs_shares',
                help='File with the list of available gluster shares'),
-    cfg.StrOpt('glusterfs_disk_util',
-               default='df',
-               help='Use du or df for free space calculation'),
     cfg.BoolOpt('glusterfs_sparsed_volumes',
                 default=True,
                 help=('Create volumes as sparsed files which take no space.'
@@ -74,12 +70,25 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
     driver_volume_type = 'glusterfs'
     driver_prefix = 'glusterfs'
     volume_backend_name = 'GlusterFS'
-    VERSION = '1.1.0'
+    VERSION = '1.1.1'
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, execute=processutils.execute, *args, **kwargs):
+        self._remotefsclient = None
         super(GlusterfsDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(volume_opts)
         self._nova = None
+        self.base = getattr(self.configuration,
+                            'glusterfs_mount_point_base',
+                            CONF.glusterfs_mount_point_base)
+        self._remotefsclient = remotefs.RemoteFsClient(
+            'glusterfs',
+            execute,
+            glusterfs_mount_point_base=self.base)
+
+    def set_execute(self, execute):
+        super(GlusterfsDriver, self).set_execute(execute)
+        if self._remotefsclient:
+            self._remotefsclient.set_execute(execute)
 
     def do_setup(self, context):
         """Any initialization the volume driver does while starting."""
@@ -288,6 +297,10 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
 
         self._execute('rm', '-f', mounted_path, run_as_root=True)
 
+        info_path = mounted_path + '.info'
+        if os.path.exists(info_path):
+            os.remove(info_path)
+
     @utils.synchronized('glusterfs', external=False)
     def create_snapshot(self, snapshot):
         """Apply locking to the create snapshot operation."""
@@ -492,6 +505,8 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
                    new_snap_path]
         self._execute(*command, run_as_root=True)
 
+        self._set_rw_permissions_for_all(new_snap_path)
+
     def _create_snapshot_offline(self, snapshot, path_to_disk):
         """Create snapshot (offline case)."""
 
@@ -790,7 +805,7 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
         # An update of progress = '90%' means that Nova is done
         seconds_elapsed = 0
         increment = 1
-        timeout = 600
+        timeout = 7200
         while True:
             s = db.snapshot_get(context, snapshot['id'])
 
@@ -912,7 +927,8 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
 
         return {
             'driver_volume_type': 'glusterfs',
-            'data': data
+            'data': data,
+            'mount_point_base': self._get_mount_point_base()
         }
 
     def terminate_connection(self, volume, connector, **kwargs):
@@ -1024,7 +1040,7 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
             except Exception as exc:
                 LOG.warning(_('Exception during mounting %s') % (exc,))
 
-        LOG.debug(_('Available shares: %s') % str(self._mounted_shares))
+        LOG.debug(_('Available shares: %s') % self._mounted_shares)
 
     def _ensure_share_writable(self, path):
         """Ensure that the Cinder user can write to the share.
@@ -1101,8 +1117,7 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
         """Return mount point for share.
         :param glusterfs_share: example 172.18.194.100:/var/glusterfs
         """
-        return os.path.join(self.configuration.glusterfs_mount_point_base,
-                            self._get_hash_str(glusterfs_share))
+        return self._remotefsclient.get_mount_point(glusterfs_share)
 
     def _get_available_capacity(self, glusterfs_share):
         """Calculate available space on the GlusterFS share.
@@ -1114,17 +1129,8 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
                                mount_point, run_as_root=True)
         out = out.splitlines()[1]
 
-        available = 0
-
         size = int(out.split()[1])
-        if self.configuration.glusterfs_disk_util == 'df':
-            available = int(out.split()[3])
-        else:
-            out, _ = self._execute('du', '-sb', '--apparent-size',
-                                   '--exclude', '*snapshot*', mount_point,
-                                   run_as_root=True)
-            used = int(out.split()[0])
-            available = size - used
+        available = int(out.split()[3])
 
         return available, size
 
@@ -1142,3 +1148,47 @@ class GlusterfsDriver(nfs.RemoteFsDriver):
             command.extend(self.shares[glusterfs_share].split())
 
         self._do_mount(command, ensure, glusterfs_share)
+
+    def _get_mount_point_base(self):
+        return self.base
+
+    def backup_volume(self, context, backup, backup_service):
+        """Create a new backup from an existing volume.
+
+        Allow a backup to occur only if no snapshots exist.
+        Check both Cinder and the file on-disk.  The latter is only
+        a safety mechanism to prevent further damage if the snapshot
+        information is already inconsistent.
+        """
+
+        snapshots = self.db.snapshot_get_all_for_volume(context,
+                                                        backup['volume_id'])
+        snap_error_msg = _('Backup is not supported for GlusterFS '
+                           'volumes with snapshots.')
+        if len(snapshots) > 0:
+            raise exception.InvalidVolume(snap_error_msg)
+
+        volume = self.db.volume_get(context, backup['volume_id'])
+
+        volume_dir = self._local_volume_dir(volume)
+        active_file_path = os.path.join(
+            volume_dir,
+            self.get_active_image_from_info(volume))
+
+        info = self._qemu_img_info(active_file_path)
+
+        if info.backing_file is not None:
+            msg = _('No snapshots found in database, but '
+                    '%(path)s has backing file '
+                    '%(backing_file)s!') % {'path': active_file_path,
+                                            'backing_file': info.backing_file}
+            LOG.error(msg)
+            raise exception.InvalidVolume(snap_error_msg)
+
+        if info.file_format != 'raw':
+            msg = _('Backup is only supported for raw-formatted '
+                    'GlusterFS volumes.')
+            raise exception.InvalidVolume(msg)
+
+        return super(GlusterfsDriver, self).backup_volume(
+            context, backup, backup_service)
