@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2013 Huawei Technologies Co., Ltd.
 # Copyright (c) 2013 OpenStack Foundation
 # All Rights Reserved.
@@ -52,6 +50,10 @@ class HuaweiHVSISCSIDriver(driver.ISCSIDriver):
         """Create a clone of the specified volume."""
         self.common.create_cloned_volume(volume, src_vref)
 
+    def extend_volume(self, volume, new_size):
+        """Extend a volume."""
+        self.common.extend_volume(volume, new_size)
+
     def delete_volume(self, volume):
         """Delete a volume."""
         self.common.delete_volume(volume)
@@ -123,6 +125,10 @@ class HuaweiHVSFCDriver(driver.FibreChannelDriver):
         """Create a clone of the specified volume."""
         self.common.create_cloned_volume(volume, src_vref)
 
+    def extend_volume(self, volume, new_size):
+        """Extend a volume."""
+        self.common.extend_volume(volume, new_size)
+
     def delete_volume(self, volume):
         """Delete a volume."""
         self.common.delete_volume(volume)
