@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2013 Huawei Technologies Co., Ltd.
 # Copyright (c) 2012 OpenStack Foundation
 # All Rights Reserved.
@@ -25,6 +23,7 @@ import time
 from cinder import exception
 from cinder.openstack.common import log as logging
 from cinder.volume import driver
+from cinder.volume.drivers.huawei import huawei_utils
 from cinder.volume.drivers.huawei import ssh_common
 
 
@@ -68,6 +67,10 @@ class HuaweiTISCSIDriver(driver.ISCSIDriver):
         volume_id = self.common.create_cloned_volume(volume, src_vref)
         return {'provider_location': volume_id}
 
+    def extend_volume(self, volume, new_size):
+        """Extend a volume."""
+        self.common.extend_volume(volume, new_size)
+
     def delete_volume(self, volume):
         """Delete a volume."""
         self.common.delete_volume(volume)
@@ -106,7 +109,7 @@ class HuaweiTISCSIDriver(driver.ISCSIDriver):
             self._get_iscsi_params(connector['initiator'])
 
         # First, add a host if not added before.
-        host_id = self.common.add_host(connector['host'],
+        host_id = self.common.add_host(connector['host'], connector['ip'],
                                        connector['initiator'])
 
         # Then, add the iSCSI port to the host.
@@ -175,7 +178,7 @@ class HuaweiTISCSIDriver(driver.ISCSIDriver):
         """
 
         iscsiinfo = {}
-        root = ssh_common.parse_xml_file(filename)
+        root = huawei_utils.parse_xml_file(filename)
 
         default_ip = root.findtext('iSCSI/DefaultTargetIP')
         if default_ip:
@@ -398,6 +401,10 @@ class HuaweiTFCDriver(driver.FibreChannelDriver):
         volume_id = self.common.create_cloned_volume(volume, src_vref)
         return {'provider_location': volume_id}
 
+    def extend_volume(self, volume, new_size):
+        """Extend a volume."""
+        self.common.extend_volume(volume, new_size)
+
     def delete_volume(self, volume):
         """Delete a volume."""
         self.common.delete_volume(volume)
@@ -441,7 +448,7 @@ class HuaweiTFCDriver(driver.FibreChannelDriver):
 
         self.common._update_login_info()
         # First, add a host if it is not added before.
-        host_id = self.common.add_host(connector['host'])
+        host_id = self.common.add_host(connector['host'], connector['ip'])
         # Then, add free FC ports to the host.
         ini_wwns = connector['wwpns']
         free_wwns = self._get_connected_free_wwns()
