@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -20,21 +18,20 @@ Driver for Linux servers running LVM.
 
 """
 
+import math
 import os
-import re
 import socket
 
 from oslo.config import cfg
 
 from cinder.brick import exception as brick_exception
-from cinder.brick.iscsi import iscsi
-from cinder.brick.iser import iser
 from cinder.brick.local_dev import lvm as lvm
 from cinder import exception
 from cinder.image import image_utils
 from cinder.openstack.common import fileutils
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import processutils
+from cinder import units
 from cinder import utils
 from cinder.volume import driver
 from cinder.volume import utils as volutils
@@ -45,10 +42,6 @@ volume_opts = [
     cfg.StrOpt('volume_group',
                default='cinder-volumes',
                help='Name for the VG that will contain exported volumes'),
-    cfg.StrOpt('pool_size',
-               default=None,
-               help='Size of thin provisioning pool '
-                    '(None uses entire cinder VG)'),
     cfg.IntOpt('lvm_mirrors',
                default=0,
                help='If set, create lvms with multiple mirrors. Note that '
@@ -130,15 +123,53 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
     def _delete_volume(self, volume, is_snapshot=False):
         """Deletes a logical volume."""
+        if self.configuration.volume_clear != 'none' and \
+                self.configuration.lvm_type != 'thin':
+            self._clear_volume(volume, is_snapshot)
 
-        # zero out old volumes to prevent data leaking between users
-        # TODO(ja): reclaiming space should be done lazy and low priority
-        self.clear_volume(volume, is_snapshot)
         name = volume['name']
         if is_snapshot:
             name = self._escape_snapshot(volume['name'])
         self.vg.delete(name)
 
+    def _clear_volume(self, volume, is_snapshot=False):
+        # zero out old volumes to prevent data leaking between users
+        # TODO(ja): reclaiming space should be done lazy and low priority
+        if is_snapshot:
+            # if the volume to be cleared is a snapshot of another volume
+            # we need to clear out the volume using the -cow instead of the
+            # directly volume path.  We need to skip this if we are using
+            # thin provisioned LVs.
+            # bug# lp1191812
+            dev_path = self.local_path(volume) + "-cow"
+        else:
+            dev_path = self.local_path(volume)
+
+        # TODO(jdg): Maybe we could optimize this for snaps by looking at
+        # the cow table and only overwriting what's necessary?
+        # for now we're still skipping on snaps due to hang issue
+        if not os.path.exists(dev_path):
+            msg = (_('Volume device file path %s does not exist.')
+                   % dev_path)
+            LOG.error(msg)
+            raise exception.VolumeBackendAPIException(data=msg)
+
+        size_in_g = volume.get('size', volume.get('volume_size', None))
+        if size_in_g is None:
+            msg = (_("Size for volume: %s not found, "
+                     "cannot secure delete.") % volume['id'])
+            LOG.error(msg)
+            raise exception.InvalidParameterValue(msg)
+
+        # clear_volume expects sizes in MiB, we store integer GiB
+        # be sure to convert before passing in
+        vol_sz_in_meg = size_in_g * units.KiB
+
+        volutils.clear_volume(
+            vol_sz_in_meg, dev_path,
+            volume_clear=self.configuration.volume_clear,
+            volume_clear_size=self.configuration.volume_clear_size)
+
     def _escape_snapshot(self, snapshot_name):
         # Linux LVM reserves name that starts with snapshot, so that
         # such volume name can't be created. Mangle it.
@@ -175,9 +206,12 @@ class LVMVolumeDriver(driver.VolumeDriver):
         # ThinLVM snapshot LVs.
         self.vg.activate_lv(snapshot['name'], is_snapshot=True)
 
+       # copy_volume expects sizes in MiB, we store integer GiB
+       # be sure to convert before passing in
         volutils.copy_volume(self.local_path(snapshot),
                              self.local_path(volume),
-                             snapshot['volume_size'] * 1024,
+                             snapshot['volume_size'] * units.KiB,
+                             self.configuration.volume_dd_blocksize,
                              execute=self._execute)
 
     def delete_volume(self, volume):
@@ -198,60 +232,6 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
         self._delete_volume(volume)
 
-    def clear_volume(self, volume, is_snapshot=False):
-        """unprovision old volumes to prevent data leaking between users."""
-
-        # NOTE(jdg): Don't write the blocks of thin provisioned
-        # volumes
-        if self.configuration.volume_clear == 'none' or \
-                self.configuration.lvm_type == 'thin':
-            return
-
-        if is_snapshot:
-            # if the volume to be cleared is a snapshot of another volume
-            # we need to clear out the volume using the -cow instead of the
-            # directly volume path.  We need to skip this if we are using
-            # thin provisioned LVs.
-            # bug# lp1191812
-            dev_path = self.local_path(volume) + "-cow"
-        else:
-            dev_path = self.local_path(volume)
-
-        if not os.path.exists(dev_path):
-            msg = (_('Volume device file path %s does not exist.') % dev_path)
-            LOG.error(msg)
-            raise exception.VolumeBackendAPIException(data=msg)
-
-        size_in_g = volume.get('size', volume.get('volume_size', None))
-        if size_in_g is None:
-            msg = (_("Size for volume: %s not found, "
-                     "cannot secure delete.") % volume['id'])
-            LOG.error(msg)
-            raise exception.InvalidParameterValue(msg)
-        size_in_m = self.configuration.volume_clear_size
-
-        LOG.info(_("Performing secure delete on volume: %s") % volume['id'])
-
-        if self.configuration.volume_clear == 'zero':
-            if size_in_m == 0:
-                return volutils.copy_volume('/dev/zero',
-                                            dev_path, size_in_g * 1024,
-                                            sync=True,
-                                            execute=self._execute)
-            else:
-                clear_cmd = ['shred', '-n0', '-z', '-s%dMiB' % size_in_m]
-        elif self.configuration.volume_clear == 'shred':
-            clear_cmd = ['shred', '-n3']
-            if size_in_m:
-                clear_cmd.append('-s%dMiB' % size_in_m)
-        else:
-            raise exception.InvalidConfigurationValue(
-                option='volume_clear',
-                value=self.configuration.volume_clear)
-
-        clear_cmd.append(dev_path)
-        self._execute(*clear_cmd, run_as_root=True)
-
     def create_snapshot(self, snapshot):
         """Creates a snapshot."""
 
@@ -284,7 +264,9 @@ class LVMVolumeDriver(driver.VolumeDriver):
         image_utils.fetch_to_raw(context,
                                  image_service,
                                  image_id,
-                                 self.local_path(volume), size=volume['size'])
+                                 self.local_path(volume),
+                                 self.configuration.volume_dd_blocksize,
+                                 size=volume['size'])
 
     def copy_volume_to_image(self, context, volume, image_service, image_meta):
         """Copy the volume to the specified image."""
@@ -314,15 +296,21 @@ class LVMVolumeDriver(driver.VolumeDriver):
                             self.configuration.lvm_type,
                             mirror_count)
 
+        self.vg.activate_lv(temp_snapshot['name'], is_snapshot=True)
+
+       # copy_volume expects sizes in MiB, we store integer GiB
+       # be sure to convert before passing in
         try:
-            volutils.copy_volume(self.local_path(temp_snapshot),
-                                 self.local_path(volume),
-                                 src_vref['size'] * 1024,
-                                 execute=self._execute)
+            volutils.copy_volume(
+                self.local_path(temp_snapshot),
+                self.local_path(volume),
+                src_vref['size'] * units.KiB,
+                self.configuration.volume_dd_blocksize,
+                execute=self._execute)
         finally:
             self.delete_snapshot(temp_snapshot)
 
-    def clone_image(self, volume, image_location, image_id):
+    def clone_image(self, volume, image_location, image_id, image_meta):
         return None, False
 
     def backup_volume(self, context, backup, backup_service):
@@ -356,7 +344,7 @@ class LVMVolumeDriver(driver.VolumeDriver):
 
         LOG.debug(_("Updating volume stats"))
         if self.vg is None:
-            LOG.warning(_('Unable to update stats on non-intialized '
+            LOG.warning(_('Unable to update stats on non-initialized '
                           'Volume Group: %s'), self.configuration.volume_group)
             return
 
@@ -371,8 +359,17 @@ class LVMVolumeDriver(driver.VolumeDriver):
         data["driver_version"] = self.VERSION
         data["storage_protocol"] = self.protocol
 
-        data['total_capacity_gb'] = float(self.vg.vg_size)
-        data['free_capacity_gb'] = float(self.vg.vg_free_space)
+        if self.configuration.lvm_mirrors > 0:
+            data['total_capacity_gb'] =\
+                self.vg.vg_mirror_size(self.configuration.lvm_mirrors)
+            data['free_capacity_gb'] =\
+                self.vg.vg_mirror_free_space(self.configuration.lvm_mirrors)
+        elif self.configuration.lvm_type == 'thin':
+            data['total_capacity_gb'] = self.vg.vg_thin_pool_size
+            data['free_capacity_gb'] = self.vg.vg_thin_pool_free_space
+        else:
+            data['total_capacity_gb'] = self.vg.vg_size
+            data['free_capacity_gb'] = self.vg.vg_free_space
         data['reserved_percentage'] = self.configuration.reserved_percentage
         data['QoS_support'] = False
         data['location_info'] =\
@@ -386,10 +383,65 @@ class LVMVolumeDriver(driver.VolumeDriver):
         self._stats = data
 
     def extend_volume(self, volume, new_size):
-        """Extend an existing voumes size."""
+        """Extend an existing volume's size."""
         self.vg.extend_volume(volume['name'],
                               self._sizestr(new_size))
 
+    def manage_existing(self, volume, existing_ref):
+        """Manages an existing LV.
+
+        Renames the LV to match the expected name for the volume.
+        Error checking done by manage_existing_get_size is not repeated.
+        """
+        lv_name = existing_ref['lv_name']
+        lv = self.vg.get_volume(lv_name)
+
+        # Attempt to rename the LV to match the OpenStack internal name.
+        try:
+            self.vg.rename_volume(lv_name, volume['name'])
+        except processutils.ProcessExecutionError as exc:
+            exception_message = (_("Failed to rename logical volume %(name)s, "
+                                   "error message was: %(err_msg)s")
+                                 % {'name': lv_name,
+                                    'err_msg': exc.stderr})
+            raise exception.VolumeBackendAPIException(
+                data=exception_message)
+
+    def manage_existing_get_size(self, volume, existing_ref):
+        """Return size of an existing LV for manage_existing.
+
+        existing_ref is a dictionary of the form:
+        {'lv_name': <name of LV>}
+        """
+
+        # Check that the reference is valid
+        if 'lv_name' not in existing_ref:
+            reason = _('Reference must contain lv_name element.')
+            raise exception.ManageExistingInvalidReference(
+                existing_ref=existing_ref, reason=reason)
+        lv_name = existing_ref['lv_name']
+        lv = self.vg.get_volume(lv_name)
+
+        # Raise an exception if we didn't find a suitable LV.
+        if not lv:
+            kwargs = {'existing_ref': lv_name,
+                      'reason': 'Specified logical volume does not exist.'}
+            raise exception.ManageExistingInvalidReference(**kwargs)
+
+        # LV size is returned in gigabytes.  Attempt to parse size as a float
+        # and round up to the next integer.
+        try:
+            lv_size = int(math.ceil(float(lv['size'])))
+        except ValueError:
+            exception_message = (_("Failed to manage existing volume "
+                                   "%(name)s, because reported size %(size)s "
+                                   "was not a floating-point number.")
+                                 % {'name': lv_name,
+                                    'size': lv['size']})
+            raise exception.VolumeBackendAPIException(
+                data=exception_message)
+        return lv_size
+
 
 class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
     """Executes commands relating to ISCSI volumes.
@@ -408,7 +460,8 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
     """
 
     def __init__(self, *args, **kwargs):
-        self.tgtadm = self.get_target_admin()
+        self.db = kwargs.get('db')
+        self.target_helper = self.get_target_helper(self.db)
         super(LVMISCSIDriver, self).__init__(*args, **kwargs)
         self.backend_name =\
             self.configuration.safe_get('volume_backend_name') or 'LVM_iSCSI'
@@ -416,12 +469,13 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
 
     def set_execute(self, execute):
         super(LVMISCSIDriver, self).set_execute(execute)
-        self.tgtadm.set_execute(execute)
+        if self.target_helper is not None:
+            self.target_helper.set_execute(execute)
 
-    def _create_tgtadm_target(self, iscsi_name, iscsi_target,
-                              volume_path, chap_auth, lun=0,
-                              check_exit_code=False, old_name=None):
-        # NOTE(jdg): tgt driver has an issue where with alot of activity
+    def _create_target(self, iscsi_name, iscsi_target,
+                       volume_path, chap_auth, lun=0,
+                       check_exit_code=False, old_name=None):
+        # NOTE(jdg): tgt driver has an issue where with a lot of activity
         # (or sometimes just randomly) it will get *confused* and attempt
         # to reuse a target ID, resulting in a target already exists error
         # Typically a simple retry will address this
@@ -434,7 +488,7 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
             try:
                 # NOTE(jdg): For TgtAdm case iscsi_name is all we need
                 # should clean this all up at some point in the future
-                tid = self.tgtadm.create_iscsi_target(
+                tid = self.target_helper.create_iscsi_target(
                     iscsi_name,
                     iscsi_target,
                     0,
@@ -453,140 +507,18 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
         return tid
 
     def ensure_export(self, context, volume):
-        """Synchronously recreates an export for a logical volume."""
-        # NOTE(jdg): tgtadm doesn't use the iscsi_targets table
-        # TODO(jdg): In the future move all of the dependent stuff into the
-        # cooresponding target admin class
-
-        if isinstance(self.tgtadm, iscsi.LioAdm):
-            try:
-                volume_info = self.db.volume_get(context, volume['id'])
-                (auth_method,
-                 auth_user,
-                 auth_pass) = volume_info['provider_auth'].split(' ', 3)
-                chap_auth = self._iscsi_authentication(auth_method,
-                                                       auth_user,
-                                                       auth_pass)
-            except exception.NotFound:
-                LOG.debug(_("volume_info:%s"), volume_info)
-                LOG.info(_("Skipping ensure_export. No iscsi_target "
-                           "provision for volume: %s"), volume['id'])
-                return
-
-            iscsi_name = "%s%s" % (self.configuration.iscsi_target_prefix,
-                                   volume['name'])
-            volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
-                                          volume['name'])
-            iscsi_target = 1
-
-            self._create_tgtadm_target(iscsi_name, iscsi_target,
-                                       volume_path, chap_auth)
-
-            return
-
-        if not isinstance(self.tgtadm, iscsi.TgtAdm):
-            try:
-                iscsi_target = self.db.volume_get_iscsi_target_num(
-                    context,
-                    volume['id'])
-            except exception.NotFound:
-                LOG.info(_("Skipping ensure_export. No iscsi_target "
-                           "provisioned for volume: %s"), volume['id'])
-                return
-        else:
-            iscsi_target = 1  # dummy value when using TgtAdm
-
-        chap_auth = None
-
-        # Check for https://bugs.launchpad.net/cinder/+bug/1065702
-        old_name = None
         volume_name = volume['name']
-        if (volume['provider_location'] is not None and
-                volume['name'] not in volume['provider_location']):
-
-            msg = _('Detected inconsistency in provider_location id')
-            LOG.debug(_('%s'), msg)
-            old_name = self._fix_id_migration(context, volume)
-            if 'in-use' in volume['status']:
-                volume_name = old_name
-                old_name = None
-
         iscsi_name = "%s%s" % (self.configuration.iscsi_target_prefix,
                                volume_name)
         volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
                                       volume_name)
-
         # NOTE(jdg): For TgtAdm case iscsi_name is the ONLY param we need
         # should clean this all up at some point in the future
-        self._create_tgtadm_target(iscsi_name, iscsi_target,
-                                   volume_path, chap_auth,
-                                   lun=0,
-                                   check_exit_code=False,
-                                   old_name=old_name)
-
-        return
-
-    def _fix_id_migration(self, context, volume):
-        """Fix provider_location and dev files to address bug 1065702.
-
-        For volumes that the provider_location has NOT been updated
-        and are not currently in-use we'll create a new iscsi target
-        and remove the persist file.
-
-        If the volume is in-use, we'll just stick with the old name
-        and when detach is called we'll feed back into ensure_export
-        again if necessary and fix things up then.
-
-        Details at: https://bugs.launchpad.net/cinder/+bug/1065702
-        """
-
-        model_update = {}
-        pattern = re.compile(r":|\s")
-        fields = pattern.split(volume['provider_location'])
-        old_name = fields[3]
-
-        volume['provider_location'] = \
-            volume['provider_location'].replace(old_name, volume['name'])
-        model_update['provider_location'] = volume['provider_location']
-
-        self.db.volume_update(context, volume['id'], model_update)
-
-        start = os.getcwd()
-        os.chdir('/dev/%s' % self.configuration.volume_group)
-
-        try:
-            (out, err) = self._execute('readlink', old_name)
-        except processutils.ProcessExecutionError:
-            link_path = '/dev/%s/%s' % (self.configuration.volume_group,
-                                        old_name)
-            LOG.debug(_('Symbolic link %s not found') % link_path)
-            os.chdir(start)
-            return
-
-        rel_path = out.rstrip()
-        self._execute('ln',
-                      '-s',
-                      rel_path, volume['name'],
-                      run_as_root=True)
-        os.chdir(start)
-        return old_name
-
-    def _ensure_iscsi_targets(self, context, host):
-        """Ensure that target ids have been created in datastore."""
-        # NOTE(jdg): tgtadm doesn't use the iscsi_targets table
-        # TODO(jdg): In the future move all of the dependent stuff into the
-        # cooresponding target admin class
-        if not isinstance(self.tgtadm, iscsi.TgtAdm):
-            host_iscsi_targets = self.db.iscsi_target_count_by_host(context,
-                                                                    host)
-            if host_iscsi_targets >= self.configuration.iscsi_num_targets:
-                return
-
-            # NOTE(vish): Target ids start at 1, not 0.
-            target_end = self.configuration.iscsi_num_targets + 1
-            for target_num in xrange(1, target_end):
-                target = {'host': host, 'target_num': target_num}
-                self.db.iscsi_target_create_safe(context, target)
+        model_update = self.target_helper.ensure_export(context, volume,
+                                                        iscsi_name,
+                                                        volume_path)
+        if model_update:
+            self.db.volume_update(context, volume['id'], model_update)
 
     def create_export(self, context, volume):
         return self._create_export(context, volume)
@@ -596,89 +528,16 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
         if vg is None:
             vg = self.configuration.volume_group
 
-        iscsi_name = "%s%s" % (self.configuration.iscsi_target_prefix,
-                               volume['name'])
         volume_path = "/dev/%s/%s" % (vg, volume['name'])
-        model_update = {}
 
-        # TODO(jdg): In the future move all of the dependent stuff into the
-        # cooresponding target admin class
-        if not isinstance(self.tgtadm, iscsi.TgtAdm):
-            lun = 0
-            self._ensure_iscsi_targets(context, volume['host'])
-            iscsi_target = self.db.volume_allocate_iscsi_target(context,
-                                                                volume['id'],
-                                                                volume['host'])
-        else:
-            lun = 1  # For tgtadm the controller is lun 0, dev starts at lun 1
-            iscsi_target = 0  # NOTE(jdg): Not used by tgtadm
-
-        # Use the same method to generate the username and the password.
-        chap_username = utils.generate_username()
-        chap_password = utils.generate_password()
-        chap_auth = self._iscsi_authentication('IncomingUser', chap_username,
-                                               chap_password)
-
-        tid = self._create_tgtadm_target(iscsi_name, iscsi_target,
-                                         volume_path, chap_auth)
-
-        model_update['provider_location'] = self._iscsi_location(
-            self.configuration.iscsi_ip_address, tid, iscsi_name, lun)
-        model_update['provider_auth'] = self._iscsi_authentication(
-            'CHAP', chap_username, chap_password)
-        return model_update
+        data = self.target_helper.create_export(context, volume, volume_path)
+        return {
+            'provider_location': data['location'],
+            'provider_auth': data['auth'],
+        }
 
     def remove_export(self, context, volume):
-        """Removes an export for a logical volume."""
-        # NOTE(jdg): tgtadm doesn't use the iscsi_targets table
-        # TODO(jdg): In the future move all of the dependent stuff into the
-        # cooresponding target admin class
-
-        if isinstance(self.tgtadm, iscsi.LioAdm):
-            try:
-                iscsi_target = self.db.volume_get_iscsi_target_num(
-                    context,
-                    volume['id'])
-            except exception.NotFound:
-                LOG.info(_("Skipping remove_export. No iscsi_target "
-                           "provisioned for volume: %s"), volume['id'])
-                return
-
-            self.tgtadm.remove_iscsi_target(iscsi_target, 0, volume['id'],
-                                            volume['name'])
-
-            return
-
-        elif not isinstance(self.tgtadm, iscsi.TgtAdm):
-            try:
-                iscsi_target = self.db.volume_get_iscsi_target_num(
-                    context,
-                    volume['id'])
-            except exception.NotFound:
-                LOG.info(_("Skipping remove_export. No iscsi_target "
-                           "provisioned for volume: %s"), volume['id'])
-                return
-        else:
-            iscsi_target = 0
-
-        try:
-
-            # NOTE: provider_location may be unset if the volume hasn't
-            # been exported
-            location = volume['provider_location'].split(' ')
-            iqn = location[1]
-
-            # ietadm show will exit with an error
-            # this export has already been removed
-            self.tgtadm.show_target(iscsi_target, iqn=iqn)
-
-        except Exception:
-            LOG.info(_("Skipping remove_export. No iscsi_target "
-                       "is presently exported for volume: %s"), volume['id'])
-            return
-
-        self.tgtadm.remove_iscsi_target(iscsi_target, 0, volume['name_id'],
-                                        volume['name'])
+        self.target_helper.remove_export(context, volume)
 
     def migrate_volume(self, ctxt, volume, host, thin=False, mirror_count=0):
         """Optimize the migration if the destination is on the same server.
@@ -705,15 +564,15 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
 
         if dest_vg != self.vg.vg_name:
             vg_list = volutils.get_all_volume_groups()
-            vg_dict = \
+            try:
                 (vg for vg in vg_list if vg['name'] == dest_vg).next()
-            if vg_dict is None:
+            except StopIteration:
                 message = ("Destination Volume Group %s does not exist" %
                            dest_vg)
                 LOG.error(_('%s'), message)
                 return false_ret
 
-            helper = 'sudo cinder-rootwrap %s' % CONF.rootwrap_config
+            helper = utils.get_root_helper()
             dest_vg_ref = lvm.LVM(dest_vg, helper,
                                   lvm_type=lvm_type,
                                   executor=self._execute)
@@ -727,6 +586,7 @@ class LVMISCSIDriver(LVMVolumeDriver, driver.ISCSIDriver):
         volutils.copy_volume(self.local_path(volume),
                              self.local_path(volume, vg=dest_vg),
                              volume['size'],
+                             self.configuration.volume_dd_blocksize,
                              execute=self._execute)
         self._delete_volume(volume)
         model_update = self._create_export(ctxt, volume, vg=dest_vg)
@@ -758,143 +618,8 @@ class LVMISERDriver(LVMISCSIDriver, driver.ISERDriver):
     """
 
     def __init__(self, *args, **kwargs):
-        self.tgtadm = self.get_target_admin()
+        self.target_helper = self.get_target_helper(kwargs.get('db'))
         LVMVolumeDriver.__init__(self, *args, **kwargs)
         self.backend_name =\
             self.configuration.safe_get('volume_backend_name') or 'LVM_iSER'
         self.protocol = 'iSER'
-        self.tgtadm.set_execute(self._execute)
-
-    def set_execute(self, execute):
-        LVMVolumeDriver.set_execute(self, execute)
-        self.tgtadm.set_execute(execute)
-
-    def ensure_export(self, context, volume):
-        """Synchronously recreates an export for a logical volume."""
-
-        if not isinstance(self.tgtadm, iser.TgtAdm):
-            try:
-                iser_target = self.db.volume_get_iscsi_target_num(
-                    context,
-                    volume['id'])
-            except exception.NotFound:
-                LOG.info(_("Skipping ensure_export. No iser_target "
-                           "provisioned for volume: %s"), volume['id'])
-                return
-        else:
-            iser_target = 1  # dummy value when using TgtAdm
-
-        chap_auth = None
-
-        # Check for https://bugs.launchpad.net/cinder/+bug/1065702
-        old_name = None
-        volume_name = volume['name']
-        if (volume['provider_location'] is not None and
-                volume['name'] not in volume['provider_location']):
-
-            msg = _('Detected inconsistency in provider_location id')
-            LOG.debug(msg)
-            old_name = self._fix_id_migration(context, volume)
-            if 'in-use' in volume['status']:
-                volume_name = old_name
-                old_name = None
-
-        iser_name = "%s%s" % (self.configuration.iser_target_prefix,
-                              volume_name)
-        volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
-                                      volume_name)
-
-        self.tgtadm.create_iser_target(iser_name, iser_target,
-                                       0, volume_path, chap_auth,
-                                       check_exit_code=False,
-                                       old_name=old_name)
-
-    def _ensure_iser_targets(self, context, host):
-        """Ensure that target ids have been created in datastore."""
-        if not isinstance(self.tgtadm, iser.TgtAdm):
-            host_iser_targets = self.db.iscsi_target_count_by_host(context,
-                                                                   host)
-            if host_iser_targets >= self.configuration.iser_num_targets:
-                return
-
-            # NOTE(vish): Target ids start at 1, not 0.
-            target_end = self.configuration.iser_num_targets + 1
-            for target_num in xrange(1, target_end):
-                target = {'host': host, 'target_num': target_num}
-                self.db.iscsi_target_create_safe(context, target)
-
-    def create_export(self, context, volume):
-        """Creates an export for a logical volume."""
-
-        iser_name = "%s%s" % (self.configuration.iser_target_prefix,
-                              volume['name'])
-        volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
-                                      volume['name'])
-        model_update = {}
-
-        # TODO(jdg): In the future move all of the dependent stuff into the
-        # cooresponding target admin class
-        if not isinstance(self.tgtadm, iser.TgtAdm):
-            lun = 0
-            self._ensure_iser_targets(context, volume['host'])
-            iser_target = self.db.volume_allocate_iscsi_target(context,
-                                                               volume['id'],
-                                                               volume['host'])
-        else:
-            lun = 1  # For tgtadm the controller is lun 0, dev starts at lun 1
-            iser_target = 0
-
-        # Use the same method to generate the username and the password.
-        chap_username = utils.generate_username()
-        chap_password = utils.generate_password()
-        chap_auth = self._iser_authentication('IncomingUser', chap_username,
-                                              chap_password)
-        tid = self.tgtadm.create_iser_target(iser_name,
-                                             iser_target,
-                                             0,
-                                             volume_path,
-                                             chap_auth)
-        model_update['provider_location'] = self._iser_location(
-            self.configuration.iser_ip_address, tid, iser_name, lun)
-        model_update['provider_auth'] = self._iser_authentication(
-            'CHAP', chap_username, chap_password)
-        return model_update
-
-    def remove_export(self, context, volume):
-        """Removes an export for a logical volume."""
-
-        if not isinstance(self.tgtadm, iser.TgtAdm):
-            try:
-                iser_target = self.db.volume_get_iscsi_target_num(
-                    context,
-                    volume['id'])
-            except exception.NotFound:
-                LOG.info(_("Skipping remove_export. No iser_target "
-                           "provisioned for volume: %s"), volume['id'])
-                return
-        else:
-            iser_target = 0
-
-        try:
-
-            # NOTE: provider_location may be unset if the volume hasn't
-            # been exported
-            location = volume['provider_location'].split(' ')
-            iqn = location[1]
-
-            self.tgtadm.show_target(iser_target, iqn=iqn)
-
-        except Exception:
-            LOG.info(_("Skipping remove_export. No iser_target "
-                       "is presently exported for volume: %s"), volume['id'])
-            return
-
-        self.tgtadm.remove_iser_target(iser_target, 0, volume['id'],
-                                       volume['name'])
-
-    def _iser_location(self, ip, target, iqn, lun=None):
-        return "%s:%s,%s %s %s" % (ip, self.configuration.iser_port,
-                                   target, iqn, lun)
-
-    def _iser_authentication(self, chap, name, password):
-        return "%s %s %s" % (chap, name, password)
