@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2012 NetApp, Inc.
 # All Rights Reserved.
 #
@@ -21,12 +19,11 @@ Volume driver for NetApp NFS storage.
 import copy
 import os
 import re
-import socket
 from threading import Timer
 import time
-import urlparse
+import uuid
 
-from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
 
 from cinder import exception
 from cinder.image import image_utils
@@ -42,6 +39,7 @@ from cinder.volume.drivers.netapp.options import netapp_basicauth_opts
 from cinder.volume.drivers.netapp.options import netapp_cluster_opts
 from cinder.volume.drivers.netapp.options import netapp_connection_opts
 from cinder.volume.drivers.netapp.options import netapp_img_cache_opts
+from cinder.volume.drivers.netapp.options import netapp_nfs_extra_opts
 from cinder.volume.drivers.netapp.options import netapp_transport_opts
 from cinder.volume.drivers.netapp import ssc_utils
 from cinder.volume.drivers.netapp import utils as na_utils
@@ -54,13 +52,6 @@ from cinder.volume.drivers import nfs
 LOG = logging.getLogger(__name__)
 
 
-CONF = cfg.CONF
-CONF.register_opts(netapp_connection_opts)
-CONF.register_opts(netapp_transport_opts)
-CONF.register_opts(netapp_basicauth_opts)
-CONF.register_opts(netapp_img_cache_opts)
-
-
 class NetAppNFSDriver(nfs.NfsDriver):
     """Base class for NetApp NFS driver.
       Executes commands relating to Volumes.
@@ -232,7 +223,8 @@ class NetAppNFSDriver(nfs.NfsDriver):
         """Fetch the image from image_service and write it to the volume."""
         super(NetAppNFSDriver, self).copy_image_to_volume(
             context, volume, image_service, image_id)
-        LOG.info(_('Copied image to volume %s'), volume['name'])
+        LOG.info(_('Copied image to volume %s using regular download.'),
+                 volume['name'])
         self._register_image_in_cache(volume, image_id)
 
     def _register_image_in_cache(self, volume, image_id):
@@ -271,7 +263,7 @@ class NetAppNFSDriver(nfs.NfsDriver):
             dir = self._get_mount_point_for_share(share)
             file_path = '%s/%s' % (dir, dst)
             if not os.path.exists(file_path):
-                LOG.info(_('Cloning img from cache for %s'), dst)
+                LOG.info(_('Cloning from cache to destination %s'), dst)
                 self._clone_volume(src, dst, volume_id=None, share=share)
         _do_clone()
 
@@ -374,7 +366,7 @@ class NetAppNFSDriver(nfs.NfsDriver):
             LOG.warning(_('Exception during deleting %s'), ex.__str__())
             return False
 
-    def clone_image(self, volume, image_location, image_id):
+    def clone_image(self, volume, image_location, image_id, image_meta):
         """Create a volume efficiently from an existing image.
 
         image_location is a string whose format depends on the
@@ -403,7 +395,7 @@ class NetAppNFSDriver(nfs.NfsDriver):
                 post_clone = self._post_clone_image(volume)
         except Exception as e:
             msg = e.msg if getattr(e, 'msg', None) else e.__str__()
-            LOG.warn(_('Unexpected exception in cloning image'
+            LOG.info(_('Image cloning unsuccessful for image'
                        ' %(image_id)s. Message: %(msg)s')
                      % {'image_id': image_id, 'msg': msg})
             vol_path = self.local_path(volume)
@@ -439,7 +431,7 @@ class NetAppNFSDriver(nfs.NfsDriver):
 
     def _direct_nfs_clone(self, volume, image_location, image_id):
         """Clone directly in nfs share."""
-        LOG.info(_('Cloning image %s directly in share'), image_id)
+        LOG.info(_('Checking image clone %s from glance share.'), image_id)
         cloned = False
         image_location = self._construct_image_nfs_url(image_location)
         share = self._is_cloneable_share(image_location)
@@ -525,24 +517,31 @@ class NetAppNFSDriver(nfs.NfsDriver):
                     retry_seconds = retry_seconds - sleep_interval
 
     def _is_cloneable_share(self, image_location):
-        """Finds if the image at location is cloneable.
+        """Finds if the image at location is cloneable."""
+        conn, dr = self._check_get_nfs_path_segs(image_location)
+        return self._check_share_in_use(conn, dr)
 
-             WebNFS url format with relative-path is supported.
-             Accepting all characters in path-names and checking
-             against the mounted shares which will contain only
-             allowed path segments.
+    def _check_get_nfs_path_segs(self, image_location):
+        """Checks if the nfs path format is matched.
+
+            WebNFS url format with relative-path is supported.
+            Accepting all characters in path-names and checking
+            against the mounted shares which will contain only
+            allowed path segments. Returns connection and dir details.
         """
-
-        nfs_loc_pattern =\
-            '^nfs://(([\w\-\.]+:{1}[\d]+|[\w\-\.]+)(/[^\/].*)*(/[^\/\\\\]+)$)'
-        matched = re.match(nfs_loc_pattern, image_location, flags=0)
-        if not matched:
-            LOG.debug(_('Image location not in the'
-                        ' expected format %s'), image_location)
-            return None
-        conn = matched.group(2)
-        dir = matched.group(3) or '/'
-        return self._check_share_in_use(conn, dir)
+        conn, dr = None, None
+        if image_location:
+            nfs_loc_pattern =\
+                ('^nfs://(([\w\-\.]+:{1}[\d]+|[\w\-\.]+)(/[^\/].*)'
+                 '*(/[^\/\\\\]+)$)')
+            matched = re.match(nfs_loc_pattern, image_location, flags=0)
+            if not matched:
+                LOG.debug(_('Image location not in the'
+                            ' expected format %s'), image_location)
+            else:
+                conn = matched.group(2)
+                dr = matched.group(3) or '/'
+        return (conn, dr)
 
     def _share_match_for_ip(self, ip, shares):
         """Returns the share that is served by ip.
@@ -554,11 +553,11 @@ class NetAppNFSDriver(nfs.NfsDriver):
         raise NotImplementedError()
 
     def _check_share_in_use(self, conn, dir):
-        """Checks if share is cinder mounted and returns it. """
+        """Checks if share is cinder mounted and returns it."""
         try:
             if conn:
                 host = conn.split(':')[0]
-                ipv4 = socket.gethostbyname(host)
+                ip = na_utils.resolve_hostname(host)
                 share_candidates = []
                 for sh in self._mounted_shares:
                     sh_exp = sh.split(':')[1]
@@ -567,7 +566,7 @@ class NetAppNFSDriver(nfs.NfsDriver):
                 if share_candidates:
                     LOG.debug(_('Found possible share matches %s'),
                               share_candidates)
-                    return self._share_match_for_ip(ipv4, share_candidates)
+                    return self._share_match_for_ip(ip, share_candidates)
         except Exception:
             LOG.warn(_("Unexpected exception while short listing used share."))
         return None
@@ -583,6 +582,8 @@ class NetAppNFSDriver(nfs.NfsDriver):
         """
 
         direct_url, locations = image_location
+        if not direct_url and not locations:
+            raise exception.NotFound(_('Image location not present.'))
 
         # Locations will be always a list of one until
         # bp multiple-image-locations is introduced
@@ -615,6 +616,30 @@ class NetAppNFSDriver(nfs.NfsDriver):
         """Checks if share is compatible with volume to host it."""
         raise NotImplementedError()
 
+    def _check_share_can_hold_size(self, share, size):
+        """Checks if volume can hold image with size."""
+        tot_size, tot_available, tot_allocated = self._get_capacity_info(share)
+        if tot_available < size:
+            msg = _("Container size smaller than required file size.")
+            raise exception.VolumeDriverException(msg)
+
+    def _move_nfs_file(self, source_path, dest_path):
+        """Moves source to destination."""
+        @utils.synchronized(dest_path, external=True)
+        def _move_file(src, dst):
+            if os.path.exists(dst):
+                LOG.warn(_("Destination %s already exists."), dst)
+                return False
+            self._execute('mv', src, dst, run_as_root=True)
+            return True
+
+        try:
+            return _move_file(source_path, dest_path)
+        except Exception as e:
+            LOG.warn(_('Exception moving file %(src)s. Message - %(e)s')
+                     % {'src': source_path, 'e': e})
+        return False
+
 
 class NetAppDirectNfsDriver (NetAppNFSDriver):
     """Executes commands related to volumes on NetApp filer."""
@@ -700,6 +725,7 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
     def __init__(self, *args, **kwargs):
         super(NetAppDirectCmodeNfsDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(netapp_cluster_opts)
+        self.configuration.append_config_values(netapp_nfs_extra_opts)
 
     def _do_custom_setup(self, client):
         """Do the customized set up on client for cluster mode."""
@@ -748,6 +774,9 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
         """
         self._ensure_shares_mounted()
         extra_specs = get_volume_extra_specs(volume)
+        qos_policy_group = None
+        if extra_specs:
+            qos_policy_group = extra_specs.pop('netapp:qos_policy_group', None)
         eligible = self._find_shares(volume['size'], extra_specs)
         if not eligible:
             raise exception.NfsNoSuitableShareFound(
@@ -757,12 +786,16 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
                 volume['provider_location'] = sh
                 LOG.info(_('casted to %s') % volume['provider_location'])
                 self._do_create_volume(volume)
+                if qos_policy_group:
+                    self._set_qos_policy_group_on_volume(volume, sh,
+                                                         qos_policy_group)
                 return {'provider_location': volume['provider_location']}
-            except Exception:
-                LOG.warn(_("Exception creating vol %(name)s"
-                           " on share %(share)s")
-                         % {'name': volume['name'],
-                             'share': volume['provider_location']})
+            except Exception as ex:
+                LOG.error(_("Exception creating vol %(name)s on "
+                            "share %(share)s. Details: %(ex)s")
+                          % {'name': volume['name'],
+                             'share': volume['provider_location'],
+                             'ex': ex})
                 volume['provider_location'] = None
             finally:
                 if self.ssc_enabled:
@@ -770,6 +803,19 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
         msg = _("Volume %s could not be created on shares.")
         raise exception.VolumeBackendAPIException(data=msg % (volume['name']))
 
+    def _set_qos_policy_group_on_volume(self, volume, share, qos_policy_group):
+        target_path = '%s' % (volume['name'])
+        export_path = share.split(':')[1]
+        flex_vol_name = self._get_vol_by_junc_vserver(self.vserver,
+                                                      export_path)
+        file_assign_qos = NaElement.create_node_with_children(
+            'file-assign-qos',
+            **{'volume': flex_vol_name,
+               'qos-policy-group-name': qos_policy_group,
+               'file': target_path,
+               'vserver': self.vserver})
+        self._invoke_successfully(file_assign_qos)
+
     def _find_shares(self, size, extra_specs):
         """Finds suitable shares for given params."""
         shares = []
@@ -809,7 +855,8 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
         net_if_iter.add_new_child('max-records', '10')
         query = NaElement('query')
         net_if_iter.add_child_elem(query)
-        query.add_node_with_children('net-interface-info', **{'address': ip})
+        query.add_node_with_children(
+            'net-interface-info', **{'address': na_utils.resolve_hostname(ip)})
         result = self._invoke_successfully(net_if_iter)
         if result.get_child_content('num-records') and\
                 int(result.get_child_content('num-records')) >= 1:
@@ -819,7 +866,7 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
             _('No interface found on cluster for ip %s')
             % (ip))
 
-    def _get_verver_ips(self, vserver):
+    def _get_vserver_ips(self, vserver):
         """Get ips for the vserver."""
         result = na_utils.invoke_api(
             self._client, api_name='net-interface-get-iter',
@@ -861,7 +908,8 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
                                    %(vserver)s and junction path %(junction)s
                                    """) % msg_fmt)
 
-    def _clone_file(self, volume, src_path, dest_path, vserver=None):
+    def _clone_file(self, volume, src_path, dest_path, vserver=None,
+                    dest_exists=False):
         """Clones file on vserver."""
         msg = _("""Cloning with params volume %(volume)s, src %(src_path)s,
                     dest %(dest_path)s, vserver %(vserver)s""")
@@ -872,6 +920,9 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
             'clone-create',
             **{'volume': volume, 'source-path': src_path,
                 'destination-path': dest_path})
+        major, minor = self._client.get_api_version()
+        if major == 1 and minor >= 20 and dest_exists:
+            clone_create.add_new_child('destination-exists', 'true')
         self._invoke_successfully(clone_create, vserver)
 
     def _update_volume_stats(self):
@@ -902,7 +953,7 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
                 len(self.ssc_vols['mirrored']) else 'false'
             data['netapp_dedup'] = 'true'\
                 if self.ssc_vols['dedup'] else 'false'
-            data['netapp_nodedupe'] = 'true'\
+            data['netapp_nodedup'] = 'true'\
                 if len(self.ssc_vols['all']) >\
                 len(self.ssc_vols['dedup']) else 'false'
             data['netapp_compression'] = 'true'\
@@ -945,13 +996,13 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
             LOG.warn(_("No shares found hence skipping ssc refresh."))
             return
         mnt_share_vols = set()
-        vs_ifs = self._get_verver_ips(self.vserver)
+        vs_ifs = self._get_vserver_ips(self.vserver)
         for vol in vols['all']:
             for sh in self._mounted_shares:
                 host = sh.split(':')[0]
                 junction = sh.split(':')[1]
-                ipv4 = socket.gethostbyname(host)
-                if (self._ip_in_ifs(ipv4, vs_ifs) and
+                ip = na_utils.resolve_hostname(host)
+                if (self._ip_in_ifs(ip, vs_ifs) and
                         junction == vol.id['junction_path']):
                     mnt_share_vols.add(vol)
                     vol.export['path'] = sh
@@ -1063,6 +1114,171 @@ class NetAppDirectCmodeNfsDriver (NetAppDirectNfsDriver):
             if netapp_vol:
                 self._update_stale_vols(volume=netapp_vol)
 
+    def copy_image_to_volume(self, context, volume, image_service, image_id):
+        """Fetch the image from image_service and write it to the volume."""
+        copy_success = False
+        try:
+            major, minor = self._client.get_api_version()
+            col_path = self.configuration.netapp_copyoffload_tool_path
+            if (major == 1 and minor >= 20 and col_path):
+                self._try_copyoffload(context, volume, image_service, image_id)
+                copy_success = True
+                LOG.info(_('Copied image %(img)s to volume %(vol)s using copy'
+                           ' offload workflow.')
+                         % {'img': image_id, 'vol': volume['id']})
+            else:
+                LOG.debug(_("Copy offload either not configured or"
+                            " unsupported."))
+        except Exception as e:
+            LOG.exception(_('Copy offload workflow unsuccessful. %s'), e)
+        finally:
+            if not copy_success:
+                super(NetAppDirectCmodeNfsDriver, self).copy_image_to_volume(
+                    context, volume, image_service, image_id)
+            if self.ssc_enabled:
+                sh = self._get_provider_location(volume['id'])
+                self._update_stale_vols(self._get_vol_for_share(sh))
+
+    def _try_copyoffload(self, context, volume, image_service, image_id):
+        """Tries server side file copy offload."""
+        copied = False
+        cache_result = self._find_image_in_cache(image_id)
+        if cache_result:
+            copied = self._copy_from_cache(volume, image_id, cache_result)
+        if not cache_result or not copied:
+            self._copy_from_img_service(context, volume, image_service,
+                                        image_id)
+
+    def _get_ip_verify_on_cluster(self, host):
+        """Verifies if host on same cluster and returns ip."""
+        ip = na_utils.resolve_hostname(host)
+        vserver = self._get_vserver_for_ip(ip)
+        if not vserver:
+            raise exception.NotFound(_("No vserver owning the ip %s.") % ip)
+        return ip
+
+    def _copy_from_cache(self, volume, image_id, cache_result):
+        """Try copying image file_name from cached file_name."""
+        LOG.debug(_("Trying copy from cache using copy offload."))
+        copied = False
+        for res in cache_result:
+            try:
+                (share, file_name) = res
+                LOG.debug(_("Found cache file_name on share %s."), share)
+                if share != self._get_provider_location(volume['id']):
+                    col_path = self.configuration.netapp_copyoffload_tool_path
+                    src_ip = self._get_ip_verify_on_cluster(
+                        share.split(':')[0])
+                    src_path = os.path.join(share.split(':')[1], file_name)
+                    dst_ip = self._get_ip_verify_on_cluster(self._get_host_ip(
+                        volume['id']))
+                    dst_path = os.path.join(
+                        self._get_export_path(volume['id']), volume['name'])
+                    self._execute(col_path, src_ip, dst_ip,
+                                  src_path, dst_path, run_as_root=False,
+                                  check_exit_code=0)
+                    self._register_image_in_cache(volume, image_id)
+                    LOG.debug(_("Copied image from cache to volume %s using"
+                                " copy offload."), volume['id'])
+                else:
+                    self._clone_file_dst_exists(share, file_name,
+                                                volume['name'],
+                                                dest_exists=True)
+                    LOG.debug(_("Copied image from cache to volume %s using"
+                                " cloning."), volume['id'])
+                self._post_clone_image(volume)
+                copied = True
+                break
+            except Exception as e:
+                LOG.exception(_('Error in workflow copy from cache. %s.'), e)
+        return copied
+
+    def _clone_file_dst_exists(self, share, src_name, dst_name,
+                               dest_exists=False):
+        """Clone file even if dest exists."""
+        (vserver, exp_volume) = self._get_vserver_and_exp_vol(share=share)
+        self._clone_file(exp_volume, src_name, dst_name, vserver,
+                         dest_exists=dest_exists)
+
+    def _copy_from_img_service(self, context, volume, image_service,
+                               image_id):
+        """Copies from the image service using copy offload."""
+        LOG.debug(_("Trying copy from image service using copy offload."))
+        image_loc = image_service.get_location(context, image_id)
+        image_loc = self._construct_image_nfs_url(image_loc)
+        conn, dr = self._check_get_nfs_path_segs(image_loc)
+        if conn:
+            src_ip = self._get_ip_verify_on_cluster(conn.split(':')[0])
+        else:
+            raise exception.NotFound(_("Source host details not found."))
+        (__, ___, img_file) = image_loc.rpartition('/')
+        src_path = os.path.join(dr, img_file)
+        dst_ip = self._get_ip_verify_on_cluster(self._get_host_ip(
+            volume['id']))
+        # tmp file is required to deal with img formats
+        tmp_img_file = str(uuid.uuid4())
+        col_path = self.configuration.netapp_copyoffload_tool_path
+        img_info = image_service.show(context, image_id)
+        dst_share = self._get_provider_location(volume['id'])
+        self._check_share_can_hold_size(dst_share, img_info['size'])
+
+        try:
+            # If src and dst share not equal
+            if (('%s:%s' % (src_ip, dr)) !=
+                    ('%s:%s' % (dst_ip, self._get_export_path(volume['id'])))):
+                dst_img_serv_path = os.path.join(
+                    self._get_export_path(volume['id']), tmp_img_file)
+                self._execute(col_path, src_ip, dst_ip, src_path,
+                              dst_img_serv_path, run_as_root=False,
+                              check_exit_code=0)
+            else:
+                self._clone_file_dst_exists(dst_share, img_file, tmp_img_file)
+            dst_dir = self._get_mount_point_for_share(dst_share)
+            dst_img_local = os.path.join(dst_dir, tmp_img_file)
+            self._discover_file_till_timeout(dst_img_local, timeout=120)
+            LOG.debug(_('Copied image %(img)s to tmp file %(tmp)s.')
+                      % {'img': image_id, 'tmp': tmp_img_file})
+            dst_img_cache_local = os.path.join(dst_dir,
+                                               'img-cache-%s' % (image_id))
+            if img_info['disk_format'] == 'raw':
+                LOG.debug(_('Image is raw %s.'), image_id)
+                self._clone_file_dst_exists(dst_share, tmp_img_file,
+                                            volume['name'], dest_exists=True)
+                self._move_nfs_file(dst_img_local, dst_img_cache_local)
+                LOG.debug(_('Copied raw image %(img)s to volume %(vol)s.')
+                          % {'img': image_id, 'vol': volume['id']})
+            else:
+                LOG.debug(_('Image will be converted to raw %s.'), image_id)
+                img_conv = str(uuid.uuid4())
+                dst_img_conv_local = os.path.join(dst_dir, img_conv)
+
+                # Checking against image size which is approximate check
+                self._check_share_can_hold_size(dst_share, img_info['size'])
+                try:
+                    image_utils.convert_image(dst_img_local,
+                                              dst_img_conv_local, 'raw')
+                    data = image_utils.qemu_img_info(dst_img_conv_local)
+                    if data.file_format != "raw":
+                        raise exception.InvalidResults(
+                            _("Converted to raw, but format is now %s.")
+                            % data.file_format)
+                    else:
+                        self._clone_file_dst_exists(dst_share, img_conv,
+                                                    volume['name'],
+                                                    dest_exists=True)
+                        self._move_nfs_file(dst_img_conv_local,
+                                            dst_img_cache_local)
+                        LOG.debug(_('Copied locally converted raw image'
+                                    ' %(img)s to volume %(vol)s.')
+                                  % {'img': image_id, 'vol': volume['id']})
+                finally:
+                    if os.path.exists(dst_img_conv_local):
+                        self._delete_file(dst_img_conv_local)
+            self._post_clone_image(volume)
+        finally:
+            if os.path.exists(dst_img_local):
+                self._delete_file(dst_img_local)
+
 
 class NetAppDirect7modeNfsDriver (NetAppDirectNfsDriver):
     """Executes commands related to volumes on 7 mode."""
@@ -1076,7 +1292,7 @@ class NetAppDirect7modeNfsDriver (NetAppDirectNfsDriver):
         client.set_api_version(major, minor)
 
     def check_for_setup_error(self):
-        """Checks if setup occured properly."""
+        """Checks if setup occurred properly."""
         api_version = self._client.get_api_version()
         if api_version:
             major, minor = api_version
