@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2012 NetApp, Inc.
 # Copyright (c) 2012 OpenStack Foundation
 # All Rights Reserved.
@@ -535,7 +533,6 @@ def get_volumes_for_specs(ssc_vols, specs):
     result = copy.deepcopy(ssc_vols['all'])
     raid_type = specs.get('netapp:raid_type')
     disk_type = specs.get('netapp:disk_type')
-    qos_policy_group = specs.get('netapp:qos_policy_group')
     bool_specs_list = ['netapp_mirrored', 'netapp_unmirrored',
                        'netapp_dedup', 'netapp_nodedup',
                        'netapp_compression', 'netapp_nocompression',
@@ -584,7 +581,7 @@ def get_volumes_for_specs(ssc_vols, specs):
             result = result & ssc_vols['thin']
         else:
             result = result - ssc_vols['thin']
-    if raid_type or disk_type or qos_policy_group:
+    if raid_type or disk_type:
         tmp = copy.deepcopy(result)
         for vol in tmp:
             if raid_type:
@@ -597,21 +594,19 @@ def get_volumes_for_specs(ssc_vols, specs):
                 vol_dtype = vol_dtype.lower() if vol_dtype else None
                 if disk_type.lower() != vol_dtype:
                     result.discard(vol)
-            if qos_policy_group:
-                vol_qos = vol.qos['qos_policy_group']
-                vol_qos = vol_qos.lower() if vol_qos else None
-                if qos_policy_group.lower() != vol_qos:
-                    result.discard(vol)
     return result
 
 
 def check_ssc_api_permissions(na_server):
     """Checks backend ssc api permissions for the user."""
-    api_map = {'storage-disk-get-iter': ['disk type'],
-               'snapmirror-get-iter': ['data protection mirror'],
-               'sis-get-iter': ['deduplication', 'compression'],
-               'aggr-options-list-info': ['raid type'],
-               'volume-get-iter': ['volume information']}
+    api_map = {'storage-disk-get-iter': ['netapp:disk_type'],
+               'snapmirror-get-iter': ['netapp_mirrored',
+                                       'netapp_unmirrored'],
+               'sis-get-iter': ['netapp_dedup', 'netapp_nodedup',
+                                'netapp_compression',
+                                'netapp_nocompression'],
+               'aggr-options-list-info': ['netapp:raid_type'],
+               'volume-get-iter': []}
     failed_apis = na_utils.check_apis_on_cluster(na_server, api_map.keys())
     if failed_apis:
         if 'volume-get-iter' in failed_apis:
@@ -623,6 +618,6 @@ def check_ssc_api_permissions(na_server):
             for fail in failed_apis:
                 unsupp_ssc_features.extend(api_map[fail])
             LOG.warn(_("The user does not have access or sufficient"
-                       " privileges to use all ssc apis. The ssc"
-                       " features %s may not work as expected."),
+                       " privileges to use all netapp apis. The following"
+                       " extra_specs will fail or be ignored: %s"),
                      unsupp_ssc_features)
