@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2012 NetApp, Inc.
 # Copyright (c) 2012 OpenStack Foundation
 # All Rights Reserved.
@@ -22,8 +20,11 @@ This module contains common utilities to be used by one or more
 NetApp drivers to achieve the desired functionality.
 """
 
+import base64
+import binascii
 import copy
 import socket
+import uuid
 
 from cinder import context
 from cinder import exception
@@ -322,3 +323,34 @@ def check_apis_on_cluster(na_server, api_list=[]):
             msg = _("Api version could not be determined.")
             raise exception.VolumeBackendAPIException(data=msg)
     return failed_apis
+
+
+def resolve_hostname(hostname):
+    """Resolves host name to IP address."""
+    res = socket.getaddrinfo(hostname, None)[0]
+    family, socktype, proto, canonname, sockaddr = res
+    return sockaddr[0]
+
+
+def encode_hex_to_base32(hex_string):
+    """Encodes hex to base32 bit as per RFC4648."""
+    bin_form = binascii.unhexlify(hex_string)
+    return base64.b32encode(bin_form)
+
+
+def decode_base32_to_hex(base32_string):
+    """Decodes base32 string to hex string."""
+    bin_form = base64.b32decode(base32_string)
+    return binascii.hexlify(bin_form)
+
+
+def convert_uuid_to_es_fmt(uuid_str):
+    """Converts uuid to e-series compatible name format."""
+    uuid_base32 = encode_hex_to_base32(uuid.UUID(str(uuid_str)).hex)
+    return uuid_base32.strip('=')
+
+
+def convert_es_fmt_to_uuid(es_label):
+    """Converts e-series name format to uuid."""
+    es_label_b32 = es_label.ljust(32, '=')
+    return uuid.UUID(binascii.hexlify(base64.b32decode(es_label_b32)))
