@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-#
 # Copyright 2013 Nexenta Systems, Inc.
 # All Rights Reserved.
 #
@@ -75,20 +73,10 @@ NEXENTA_NFS_OPTIONS = [
     cfg.StrOpt('nexenta_volume_compression',
                default='on',
                help='Default compression value for new ZFS folders.'),
-    cfg.StrOpt('nexenta_mount_options',
-               default=None,
-               help='Mount options passed to the nfs client. See section '
-                    'of the nfs man page for details'),
-    cfg.FloatOpt('nexenta_used_ratio',
-                 default=0.95,
-                 help=('Percent of ACTUAL usage of the underlying volume '
-                       'before no new volumes can be allocated to the volume '
-                       'destination.')),
-    cfg.FloatOpt('nexenta_oversub_ratio',
-                 default=1.0,
-                 help=('This will compare the allocated to available space on '
-                       'the volume destination.  If the ratio exceeds this '
-                       'number, the destination will no longer be valid.'))
+    cfg.BoolOpt('nexenta_nms_cache_volroot',
+                default=True,
+                help=('If set True cache NexentaStor appliance volroot option '
+                      'value.'))
 ]
 
 NEXENTA_VOLUME_OPTIONS = [
@@ -99,3 +87,23 @@ NEXENTA_VOLUME_OPTIONS = [
                 default=False,
                 help='flag to create sparse volumes'),
 ]
+
+NEXENTA_RRMGR_OPTIONS = [
+    cfg.IntOpt('nexenta_rrmgr_compression',
+               default=0,
+               help=('Enable stream compression, level 1..9. 1 - gives best '
+                     'speed; 9 - gives best compression.')),
+    cfg.IntOpt('nexenta_rrmgr_tcp_buf_size',
+               default=4096,
+               help='TCP Buffer size in KiloBytes.'),
+    cfg.IntOpt('nexenta_rrmgr_connections',
+               default=2,
+               help='Number of TCP connections.'),
+]
+
+CONF = cfg.CONF
+CONF.register_opts(NEXENTA_CONNECTION_OPTIONS)
+CONF.register_opts(NEXENTA_ISCSI_OPTIONS)
+CONF.register_opts(NEXENTA_VOLUME_OPTIONS)
+CONF.register_opts(NEXENTA_NFS_OPTIONS)
+CONF.register_opts(NEXENTA_RRMGR_OPTIONS)
