@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright (c) 2012 NetApp, Inc.
 # All Rights Reserved.
 #
@@ -62,9 +60,28 @@ volume_opts = [
                      'of the nfs man page for details.')),
 ]
 
+nas_opts = [
+    cfg.StrOpt('nas_ip',
+               default='',
+               help='IP address or Hostname of NAS system.'),
+    cfg.StrOpt('nas_login',
+               default='admin',
+               help='User name to connect to NAS system.'),
+    cfg.StrOpt('nas_password',
+               default='',
+               help='Password to connect to NAS system.',
+               secret=True),
+    cfg.IntOpt('nas_ssh_port',
+               default=22,
+               help='SSH port to use to connect to NAS system.'),
+    cfg.StrOpt('nas_private_key',
+               default='',
+               help='Filename of private key to use for SSH authentication.'),
+]
 
 CONF = cfg.CONF
 CONF.register_opts(volume_opts)
+CONF.register_opts(nas_opts)
 
 
 class RemoteFsDriver(driver.VolumeDriver):
@@ -74,6 +91,8 @@ class RemoteFsDriver(driver.VolumeDriver):
 
     def __init__(self, *args, **kwargs):
         super(RemoteFsDriver, self).__init__(*args, **kwargs)
+        self.shares = {}
+        self._mounted_shares = []
 
     def check_for_setup_error(self):
         """Just to override parent behavior."""
@@ -158,7 +177,7 @@ class RemoteFsDriver(driver.VolumeDriver):
             except Exception as exc:
                 LOG.warning(_('Exception during mounting %s') % (exc,))
 
-        LOG.debug('Available shares %s' % str(self._mounted_shares))
+        LOG.debug('Available shares %s' % self._mounted_shares)
 
     def create_cloned_volume(self, volume, src_vref):
         raise NotImplementedError()
@@ -243,6 +262,7 @@ class RemoteFsDriver(driver.VolumeDriver):
                                  image_service,
                                  image_id,
                                  self.local_path(volume),
+                                 self.configuration.volume_dd_blocksize,
                                  size=volume['size'])
 
         # NOTE (leseb): Set the virtual size of the image
@@ -370,14 +390,6 @@ class RemoteFsDriver(driver.VolumeDriver):
     def _ensure_share_mounted(self, nfs_share):
         raise NotImplementedError()
 
-    def backup_volume(self, context, backup, backup_service):
-        """Create a new backup from an existing volume."""
-        raise NotImplementedError()
-
-    def restore_backup(self, context, backup, volume, backup_service):
-        """Restore an existing backup to a new or existing volume."""
-        raise NotImplementedError()
-
 
 class NfsDriver(RemoteFsDriver):
     """NFS based cinder driver. Creates file on NFS share for using it
@@ -412,7 +424,7 @@ class NfsDriver(RemoteFsDriver):
             self._remotefsclient.set_execute(execute)
 
     def do_setup(self, context):
-        """Any initialization the volume driver does while starting"""
+        """Any initialization the volume driver does while starting."""
         super(NfsDriver, self).do_setup(context)
 
         config = self.configuration.nfs_shares_config
