@@ -1,10 +1,6 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-#
-#    (c) Copyright 2012-2013 Hewlett-Packard Development Company, L.P.
+#    (c) Copyright 2012-2014 Hewlett-Packard Development Company, L.P.
 #    All Rights Reserved.
 #
-#    Copyright 2012 OpenStack Foundation
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -19,11 +15,11 @@
 #
 """
 Volume driver for HP 3PAR Storage array.
-This driver requires 3.1.2 MU2 firmware on the 3PAR array, using
-the 2.x version of the hp3parclient.
+This driver requires 3.1.3 firmware on the 3PAR array, using
+the 3.x version of the hp3parclient.
 
 You will need to install the python hp3parclient.
-sudo pip install --upgrade "hp3parclient>=2.0"
+sudo pip install --upgrade "hp3parclient>=3.0"
 
 Set the following in the cinder.conf file to enable the
 3PAR iSCSI Driver along with the required flags:
@@ -59,9 +55,16 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
         1.2.2 - Added try/finally around client login/logout.
         1.2.3 - log exceptions before raising
         1.2.4 - Fixed iSCSI active path bug #1224594
+        1.2.5 - Added metadata during attach/detach bug #1258033
+        1.2.6 - Use least-used iscsi n:s:p for iscsi volume attach bug #1269515
+                This update now requires 3.1.2 MU3 firmware
+        1.3.0 - Removed all SSH code.  We rely on the hp3parclient now.
+        2.0.0 - Update hp3parclient API uses 3.0.x
+        2.0.2 - Add back-end assisted volume migrate
+
     """
 
-    VERSION = "1.2.4"
+    VERSION = "2.0.2"
 
     def __init__(self, *args, **kwargs):
         super(HP3PARISCSIDriver, self).__init__(*args, **kwargs)
@@ -198,8 +201,7 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
 
     @utils.synchronized('3par', external=True)
     def create_volume_from_snapshot(self, volume, snapshot):
-        """
-        Creates a volume from a snapshot.
+        """Creates a volume from a snapshot.
 
         TODO: support using the size from the user.
         """
@@ -258,11 +260,18 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
 
             # we have to make sure we have a host
             host = self._create_host(volume, connector)
+            least_used_nsp = self._get_least_used_nsp_for_host(host['name'])
 
             # now that we have a host, create the VLUN
-            vlun = self.common.create_vlun(volume, host)
+            vlun = self.common.create_vlun(volume, host, least_used_nsp)
 
-            iscsi_ip = self._get_iscsi_ip(host['name'])
+            if least_used_nsp is None:
+                msg = _("Least busy iSCSI port not found, "
+                        "using first iSCSI port in list.")
+                LOG.warn(msg)
+                iscsi_ip = self.iscsi_ips.keys()[0]
+            else:
+                iscsi_ip = self._get_ip_using_nsp(least_used_nsp)
 
             iscsi_ip_port = self.iscsi_ips[iscsi_ip]['ip_port']
             iscsi_target_iqn = self.iscsi_ips[iscsi_ip]['iqn']
@@ -296,16 +305,21 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
         the same iqn but with a different hostname, return the hostname
         used by 3PAR.
         """
-        if domain is not None:
-            cmd = ['createhost', '-iscsi', '-persona', persona_id, '-domain',
-                   domain, hostname, iscsi_iqn]
+        # first search for an existing host
+        host_found = self.common.client.findHost(iqn=iscsi_iqn)
+        if host_found is not None:
+            self.common.hosts_naming_dict[hostname] = host_found
+            return host_found
         else:
-            cmd = ['createhost', '-iscsi', '-persona', persona_id, hostname,
-                   iscsi_iqn]
-        out = self.common._cli_run(cmd)
-        if out and len(out) > 1:
-            return self.common.parse_create_host_error(hostname, out)
-        return hostname
+            if isinstance(iscsi_iqn, str) or isinstance(iscsi_iqn, unicode):
+                iqn = [iscsi_iqn]
+            else:
+                iqn = iscsi_iqn
+            persona_id = int(persona_id)
+            self.common.client.createHost(hostname, iscsiNames=iqn,
+                                          optional={'domain': domain,
+                                                    'persona': persona_id})
+            return hostname
 
     def _modify_3par_iscsi_host(self, hostname, iscsi_iqn):
         mod_request = {'pathOperation': self.common.client.HOST_EDIT_ADD,
@@ -349,38 +363,34 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
     def remove_export(self, context, volume):
         pass
 
-    def _get_iscsi_ip(self, hostname):
-        """Get an iSCSI IP address to use.
+    def _get_least_used_nsp_for_host(self, hostname):
+        """Get the least used NSP for the current host.
 
-        Steps to determine which IP address to use.
-          * If only one IP address, return it
-          * If there is an active vlun, return the IP associated with it
-          * Return IP with fewest active vluns
+        Steps to determine which NSP to use.
+            * If only one iSCSI NSP, return it
+            * If there is already an active vlun to this host, return its NSP
+            * Return NSP with fewest active vluns
         """
-        if len(self.iscsi_ips) == 1:
-            return self.iscsi_ips.keys()[0]
 
+        iscsi_nsps = self._get_iscsi_nsps()
+        # If there's only one path, use it
+        if len(iscsi_nsps) == 1:
+            return iscsi_nsps[0]
+
+        # Try to reuse an existing iscsi path to the host
         vluns = self.common.client.getVLUNs()
-        # see if there is already a path to the
-        # host, if so use it
         for vlun in vluns['members']:
             if vlun['active']:
                 if vlun['hostname'] == hostname:
-                    # this host already has a path, so use it
-                    nsp = self.common.build_nsp(vlun['portPos'])
-                    return self._get_ip_using_nsp(nsp)
+                    temp_nsp = self.common.build_nsp(vlun['portPos'])
+                    if temp_nsp in iscsi_nsps:
+                        # this host already has an iscsi path, so use it
+                        return temp_nsp
 
-        # no current path find least used port
+        # Calculate the least used iscsi nsp
         least_used_nsp = self._get_least_used_nsp(vluns['members'],
                                                   self._get_iscsi_nsps())
-
-        if least_used_nsp is None:
-            msg = _("Least busy iSCSI port not found, "
-                    "using first iSCSI port in list.")
-            LOG.warn(msg)
-            return self.iscsi_ips.keys()[0]
-
-        return self._get_ip_using_nsp(least_used_nsp)
+        return least_used_nsp
 
     def _get_iscsi_nsps(self):
         """Return the list of candidate nsps."""
@@ -390,7 +400,7 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
         return nsps
 
     def _get_ip_using_nsp(self, nsp):
-        """Return IP assiciated with given nsp."""
+        """Return IP associated with given nsp."""
         for (key, value) in self.iscsi_ips.items():
             if value['nsp'] == nsp:
                 return key
@@ -423,4 +433,25 @@ class HP3PARISCSIDriver(cinder.volume.driver.ISCSIDriver):
 
     @utils.synchronized('3par', external=True)
     def extend_volume(self, volume, new_size):
-        self.common.extend_volume(volume, new_size)
+        self.common.client_login()
+        try:
+            self.common.extend_volume(volume, new_size)
+        finally:
+            self.common.client_logout()
+
+    @utils.synchronized('3par', external=True)
+    def attach_volume(self, context, volume, instance_uuid, host_name,
+                      mountpoint):
+        self.common.attach_volume(volume, instance_uuid)
+
+    @utils.synchronized('3par', external=True)
+    def detach_volume(self, context, volume):
+        self.common.detach_volume(volume)
+
+    @utils.synchronized('3par', external=True)
+    def migrate_volume(self, context, volume, host):
+        self.common.client_login()
+        try:
+            return self.common.migrate_volume(volume, host)
+        finally:
+            self.common.client_logout()
