@@ -21,19 +21,18 @@ operations on the SAN.
 """
 
 from lxml import etree
-from oslo_concurrency import processutils
-from oslo_log import log as logging
-from oslo_utils import units
 
 from cinder import exception
-from cinder.i18n import _, _LE, _LW
-from cinder.volume.drivers.san import san
+from cinder.openstack.common import log as logging
+from cinder.openstack.common import processutils
+from cinder import units
+from cinder.volume.drivers.san.san import SanISCSIDriver
 
 
 LOG = logging.getLogger(__name__)
 
 
-class HPLeftHandCLIQProxy(san.SanISCSIDriver):
+class HPLeftHandCLIQProxy(SanISCSIDriver):
     """Executes commands relating to HP/LeftHand SAN ISCSI volumes.
 
     We use the CLIQ interface, over SSH.
@@ -73,22 +72,15 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         1.2.0 - Ported into the new HP LeftHand driver.
         1.2.1 - Fixed bug #1279897, HP LeftHand CLIQ proxy may return incorrect
                 capacity values.
-        1.2.2 - Fixed driver with Paramiko 1.13.0, bug #1298608.
-        1.2.3 - Added update_migrated_volume #1493546
     """
 
-    VERSION = "1.2.3"
+    VERSION = "1.2.1"
 
     device_stats = {}
 
     def __init__(self, *args, **kwargs):
         super(HPLeftHandCLIQProxy, self).__init__(*args, **kwargs)
         self.cluster_vip = None
-        LOG.warning(_LW('The HPLeftHandISCSIDriver CLIQ driver has been '
-                        'DEPRECATED as of the 2015.2 release. This driver '
-                        'will be removed in the 2016.1 release. Please use '
-                        'the HPLeftHandISCSIDriver REST based driver '
-                        'instead.'))
 
     def do_setup(self, context):
         pass
@@ -112,9 +104,9 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         cliq_args['output'] = 'XML'
         (out, _err) = self._cliq_run(verb, cliq_args, check_cliq_result)
 
-        LOG.debug("CLIQ command returned %s", out)
+        LOG.debug(_("CLIQ command returned %s"), out)
 
-        result_xml = etree.fromstring(out.encode('utf8'))
+        result_xml = etree.fromstring(out)
         if check_cliq_result:
             response_node = result_xml.find("response")
             if response_node is None:
@@ -209,7 +201,7 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
             for k, v in status_node.attrib.items():
                 volume_attributes["permission." + k] = v
 
-        LOG.debug("Volume info: %(volume_name)s => %(volume_attributes)s",
+        LOG.debug(_("Volume info: %(volume_name)s => %(volume_attributes)s") %
                   {'volume_name': volume_name,
                    'volume_attributes': volume_attributes})
         return volume_attributes
@@ -265,7 +257,7 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
             for k, v in status_node.attrib.items():
                 snapshot_attributes["permission." + k] = v
 
-        LOG.debug("Snapshot info: %(name)s => %(attributes)s",
+        LOG.debug(_("Snapshot info: %(name)s => %(attributes)s") %
                   {'name': snapshot_name, 'attributes': snapshot_attributes})
         return snapshot_attributes
 
@@ -280,7 +272,10 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
             cliq_args['thinProvision'] = '0'
 
         cliq_args['volumeName'] = volume['name']
-        cliq_args['size'] = '%sGB' % volume['size']
+        if int(volume['size']) == 0:
+            cliq_args['size'] = '100MB'
+        else:
+            cliq_args['size'] = '%sGB' % volume['size']
 
         self._cliq_run_xml("createVolume", cliq_args)
 
@@ -318,9 +313,9 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         cliq_args['volumeName'] = volume['name']
         cliq_args['prompt'] = 'false'  # Don't confirm
         try:
-            self._cliq_get_volume_info(volume['name'])
+            volume_info = self._cliq_get_volume_info(volume['name'])
         except processutils.ProcessExecutionError:
-            LOG.error(_LE("Volume did not exist. It will not be deleted"))
+            LOG.error(_("Volume did not exist. It will not be deleted"))
             return
         self._cliq_run_xml("deleteVolume", cliq_args)
 
@@ -330,9 +325,9 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         cliq_args['snapshotName'] = snapshot['name']
         cliq_args['prompt'] = 'false'  # Don't confirm
         try:
-            self._cliq_get_snapshot_info(snapshot['name'])
+            volume_info = self._cliq_get_snapshot_info(snapshot['name'])
         except processutils.ProcessExecutionError:
-            LOG.error(_LE("Snapshot did not exist. It will not be deleted"))
+            LOG.error(_("Snapshot did not exist. It will not be deleted"))
             return
         try:
             self._cliq_run_xml("deleteSnapshot", cliq_args)
@@ -421,7 +416,7 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         cliq_args['serverName'] = connector['host']
         self._cliq_run_xml("unassignVolumeToServer", cliq_args)
 
-    def get_volume_stats(self, refresh=False):
+    def get_volume_stats(self, refresh):
         if refresh:
             self._update_backend_status()
 
@@ -442,7 +437,7 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
         cluster_node = result_xml.find("response/cluster")
         total_capacity = cluster_node.attrib.get("spaceTotal")
         free_capacity = cluster_node.attrib.get("unprovisionedSpace")
-        GB = units.Gi
+        GB = units.GiB
 
         data['total_capacity_gb'] = int(total_capacity) / GB
         data['free_capacity_gb'] = int(free_capacity) / GB
@@ -451,7 +446,7 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
     def create_cloned_volume(self, volume, src_vref):
         raise NotImplementedError()
 
-    def create_export(self, context, volume, connector):
+    def create_export(self, context, volume):
         pass
 
     def ensure_export(self, context, volume):
@@ -485,7 +480,3 @@ class HPLeftHandCLIQProxy(san.SanISCSIDriver):
                      dictionary of its reported capabilities.
         """
         return (False, None)
-
-    def update_migrated_volume(self, context, volume, new_volume,
-                               original_volume_status):
-        raise NotImplementedError()
