@@ -1,4 +1,4 @@
-#    Copyright 2012 OpenStack LLC
+#    Copyright 2012 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,7 +15,6 @@
 from oslo.config import cfg
 
 from cinder import exception
-from cinder import flags
 from cinder.openstack.common import log as logging
 from cinder.volume.drivers.san.san import SanISCSIDriver
 
@@ -26,8 +25,8 @@ solaris_opts = [
                default='rpool/',
                help='The ZFS path under which to create zvols for volumes.'), ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(solaris_opts)
+CONF = cfg.CONF
+CONF.register_opts(solaris_opts)
 
 
 class SolarisISCSIDriver(SanISCSIDriver):
@@ -58,11 +57,11 @@ class SolarisISCSIDriver(SanISCSIDriver):
     Also make sure you can login using san_login & san_password/san_private_key
     """
     def __init__(self, *cmd, **kwargs):
-        super(SolarisISCSIDriver, self).__init__(*cmd,
-                                                 execute=self._execute,
-                                                 **kwargs)
+        super(SolarisISCSIDriver, self).__init__(execute=self.solaris_execute,
+                                                 *cmd, **kwargs)
+        self.configuration.append_config_values(solaris_opts)
 
-    def _execute(self, *cmd, **kwargs):
+    def solaris_execute(self, *cmd, **kwargs):
         new_cmd = ['pfexec']
         new_cmd.extend(cmd)
         return super(SolarisISCSIDriver, self)._execute(*new_cmd,
@@ -123,7 +122,8 @@ class SolarisISCSIDriver(SanISCSIDriver):
         return iscsi_target_name in self._get_iscsi_targets()
 
     def _build_zfs_poolname(self, volume):
-        zfs_poolname = '%s%s' % (FLAGS.san_zfs_volume_base, volume['name'])
+        zfs_poolname = '%s%s' % (self.configuration.san_zfs_volume_base,
+                                 volume['name'])
         return zfs_poolname
 
     def create_volume(self, volume):
@@ -137,7 +137,7 @@ class SolarisISCSIDriver(SanISCSIDriver):
 
         # Create a zfs volume
         cmd = ['/usr/sbin/zfs', 'create']
-        if FLAGS.san_thin_provision:
+        if self.configuration.san_thin_provision:
             cmd.append('-s')
         cmd.extend(['-V', sizestr])
         cmd.append(zfs_poolname)
@@ -172,7 +172,7 @@ class SolarisISCSIDriver(SanISCSIDriver):
                 return luid
 
         msg = _('LUID not found for %(zfs_poolname)s. '
-                'Output=%(out)s') % locals()
+                'Output=%(out)s') % {'zfs_poolname': zfs_poolname, 'out': out}
         raise exception.VolumeBackendAPIException(data=msg)
 
     def _is_lu_created(self, volume):
@@ -186,7 +186,7 @@ class SolarisISCSIDriver(SanISCSIDriver):
 
     def local_path(self, volume):
         # TODO(justinsb): Is this needed here?
-        escaped_group = FLAGS.volume_group.replace('-', '--')
+        escaped_group = self.configuration.volume_group.replace('-', '--')
         escaped_name = volume['name'].replace('-', '--')
         return "/dev/mapper/%s-%s" % (escaped_group, escaped_name)
 
@@ -233,7 +233,8 @@ class SolarisISCSIDriver(SanISCSIDriver):
 
         #TODO(justinsb): Is this always 1? Does it matter?
         iscsi_portal_interface = '1'
-        iscsi_portal = FLAGS.san_ip + ":3260," + iscsi_portal_interface
+        iscsi_portal = \
+            self.configuration.san_ip + ":3260," + iscsi_portal_interface
 
         db_update = {}
         db_update['provider_location'] = ("%s %s" %
@@ -266,7 +267,7 @@ class SolarisISCSIDriver(SanISCSIDriver):
             self._execute('/usr/sbin/sbdadm', 'delete-lu', luid)
 
     def _collect_lines(self, data):
-        """Split lines from data into an array, trimming them """
+        """Split lines from data into an array, trimming them."""
         matches = []
         for line in data.splitlines():
             match = line.strip()
@@ -274,7 +275,7 @@ class SolarisISCSIDriver(SanISCSIDriver):
         return matches
 
     def _get_prefixed_values(self, data, prefix):
-        """Collect lines which start with prefix; with trimming"""
+        """Collect lines which start with prefix; with trimming."""
         matches = []
         for line in data.splitlines():
             line = line.strip()
