@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 SolidFire Inc
 # All Rights Reserved.
 #
@@ -18,7 +16,6 @@
 import base64
 import httplib
 import json
-import math
 import random
 import socket
 import string
@@ -31,7 +28,9 @@ from cinder import context
 from cinder import exception
 from cinder.openstack.common import log as logging
 from cinder.openstack.common import timeutils
+from cinder import units
 from cinder.volume.drivers.san.san import SanISCSIDriver
+from cinder.volume import qos_specs
 from cinder.volume import volume_types
 
 LOG = logging.getLogger(__name__)
@@ -46,8 +45,11 @@ sf_opts = [
                 help='Allow tenants to specify QOS on create'),
 
     cfg.StrOpt('sf_account_prefix',
-               default=socket.gethostname(),
-               help='Create SolidFire accounts with this prefix'),
+               default=None,
+               help='Create SolidFire accounts with this prefix. Any string '
+                    'can be used here, but the string \"hostname\" is special '
+                    'and will create a prefix using the cinder node hostsname '
+                    '(previous default behavior).  The default is NO prefix.'),
 
     cfg.IntOpt('sf_api_port',
                default=443,
@@ -87,21 +89,19 @@ class SolidFireDriver(SanISCSIDriver):
     sf_qos_keys = ['minIOPS', 'maxIOPS', 'burstIOPS']
     cluster_stats = {}
 
-    GB = math.pow(2, 30)
-
     def __init__(self, *args, **kwargs):
-            super(SolidFireDriver, self).__init__(*args, **kwargs)
-            self.configuration.append_config_values(sf_opts)
-            try:
-                self._update_cluster_status()
-            except exception.SolidFireAPIException:
-                pass
+        super(SolidFireDriver, self).__init__(*args, **kwargs)
+        self.configuration.append_config_values(sf_opts)
+        try:
+            self._update_cluster_status()
+        except exception.SolidFireAPIException:
+            pass
 
     def _issue_api_request(self, method_name, params, version='1.0'):
         """All API requests to SolidFire device go through this method.
 
         Simple json-rpc web based API calls.
-        each call takes a set of paramaters (dict)
+        each call takes a set of parameters (dict)
         and returns results in a dict as well.
 
         """
@@ -223,15 +223,17 @@ class SolidFireDriver(SanISCSIDriver):
 
     def _get_sf_account_name(self, project_id):
         """Build the SolidFire account name to use."""
-        return '%s%s%s' % (self.configuration.sf_account_prefix,
-                           '-' if self.configuration.sf_account_prefix else '',
-                           project_id)
+        prefix = self.configuration.sf_account_prefix or ''
+        if prefix == 'hostname':
+            prefix = socket.gethostname()
+        return '%s%s%s' % (prefix, '-' if prefix else '', project_id)
 
     def _get_sfaccount(self, project_id):
         sf_account_name = self._get_sf_account_name(project_id)
         sfaccount = self._get_sfaccount_by_name(sf_account_name)
         if sfaccount is None:
-            raise exception.SfAccountNotFound(account_name=sf_account_name)
+            raise exception.SolidFireAccountNotFound(
+                account_name=sf_account_name)
 
         return sfaccount
 
@@ -354,7 +356,7 @@ class SolidFireDriver(SanISCSIDriver):
 
         params = {'volumeID': int(sf_vol['volumeID']),
                   'name': 'UUID-%s' % v_ref['id'],
-                  'newSize': int(new_size * self.GB),
+                  'newSize': int(new_size * units.GiB),
                   'newAccountID': sfaccount['accountID']}
         data = self._issue_api_request('CloneVolume', params)
 
@@ -413,7 +415,7 @@ class SolidFireDriver(SanISCSIDriver):
         qos = {}
         valid_presets = self.sf_qos_dict.keys()
 
-        # First look to see if they included a preset
+        #First look to see if they included a preset
         presets = [i.value for i in volume.get('volume_metadata')
                    if i.key == 'sf-qos' and i.value in valid_presets]
         if len(presets) > 0:
@@ -422,7 +424,7 @@ class SolidFireDriver(SanISCSIDriver):
                               'detected, using %s') % presets[0])
             qos = self.sf_qos_dict[presets[0]]
         else:
-            # look for explicit settings
+            #look for explicit settings
             for i in volume.get('volume_metadata'):
                 if i.key in self.sf_qos_keys:
                     qos[i.key] = int(i.value)
@@ -431,8 +433,18 @@ class SolidFireDriver(SanISCSIDriver):
     def _set_qos_by_volume_type(self, ctxt, type_id):
         qos = {}
         volume_type = volume_types.get_volume_type(ctxt, type_id)
+        qos_specs_id = volume_type.get('qos_specs_id')
         specs = volume_type.get('extra_specs')
-        for key, value in specs.iteritems():
+
+        # NOTE(jdg): We prefer the qos_specs association
+        # and over-ride any existing
+        # extra-specs settings if present
+        if qos_specs_id is not None:
+            kvs = qos_specs.get_qos_specs(ctxt, qos_specs_id)['specs']
+        else:
+            kvs = specs
+
+        for key, value in kvs.iteritems():
             if ':' in key:
                 fields = key.split(':')
                 key = fields[1]
@@ -509,7 +521,7 @@ class SolidFireDriver(SanISCSIDriver):
         params = {'name': 'UUID-%s' % volume['id'],
                   'accountID': None,
                   'sliceCount': slice_count,
-                  'totalSize': int(volume['size'] * self.GB),
+                  'totalSize': int(volume['size'] * units.GiB),
                   'enable512e': self.configuration.sf_emulate_512,
                   'attributes': attributes,
                   'qos': qos}
@@ -528,7 +540,7 @@ class SolidFireDriver(SanISCSIDriver):
     def delete_volume(self, volume):
         """Delete SolidFire Volume from device.
 
-        SolidFire allows multipe volumes with same name,
+        SolidFire allows multiple volumes with same name,
         volumeID is what's guaranteed unique.
 
         """
@@ -540,7 +552,7 @@ class SolidFireDriver(SanISCSIDriver):
             LOG.error(_("Account for Volume ID %s was not found on "
                         "the SolidFire Cluster!") % volume['id'])
             LOG.error(_("This usually means the volume was never "
-                        "succesfully created."))
+                        "successfully created."))
             return
 
         params = {'accountID': sfaccount['accountID']}
@@ -635,7 +647,7 @@ class SolidFireDriver(SanISCSIDriver):
 
         params = {
             'volumeID': sf_vol['volumeID'],
-            'totalSize': int(new_size * self.GB)
+            'totalSize': int(new_size * units.GiB)
         }
         data = self._issue_api_request('ModifyVolume',
                                        params, version='5.0')
@@ -671,7 +683,7 @@ class SolidFireDriver(SanISCSIDriver):
 
         data['total_capacity_gb'] = results['maxProvisionedSpace']
 
-        data['free_capacity_gb'] = float(free_capacity)
+        data['free_capacity_gb'] = float(free_capacity / units.GiB)
         data['reserved_percentage'] = 0
         data['QoS_support'] = True
         data['compression_percent'] =\
@@ -756,8 +768,42 @@ class SolidFireDriver(SanISCSIDriver):
         if 'result' not in data:
             raise exception.SolidFireAPIDataException(data=data)
 
-        volume['project_id'] = new_project
-        volume['user_id'] = new_user
-        model_update = self._do_export(volume)
-        LOG.debug("Leaving SolidFire transfer volume")
-        return model_update
+        LOG.debug(_("Leaving SolidFire transfer volume"))
+
+    def retype(self, ctxt, volume, new_type, diff, host):
+        """Convert the volume to be of the new type.
+
+        Returns a boolean indicating whether the retype occurred.
+
+        :param ctxt: Context
+        :param volume: A dictionary describing the volume to migrate
+        :param new_type: A dictionary describing the volume type to convert to
+        :param diff: A dictionary with the difference between the two types
+        :param host: A dictionary describing the host to migrate to, where
+                     host['host'] is its name, and host['capabilities'] is a
+                     dictionary of its reported capabilities (Not Used).
+
+        """
+        qos = {}
+        attributes = {}
+
+        sfaccount = self._get_sfaccount(volume['project_id'])
+        params = {'accountID': sfaccount['accountID']}
+        sf_vol = self._get_sf_volume(volume['id'], params)
+
+        if sf_vol is None:
+            raise exception.VolumeNotFound(volume_id=volume['id'])
+
+        attributes = sf_vol['attributes']
+        attributes['retyped_at'] = timeutils.strtime()
+        params = {'volumeID': sf_vol['volumeID']}
+        qos = self._set_qos_by_volume_type(ctxt, new_type['id'])
+
+        if qos:
+            params['qos'] = qos
+            for k, v in qos.items():
+                attributes[k] = str(v)
+            params['attributes'] = attributes
+
+        self._issue_api_request('ModifyVolume', params)
+        return True
