@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 Pedro Navarro Perez
 # All Rights Reserved.
 #
@@ -70,8 +68,8 @@ class WindowsDriver(driver.ISCSIDriver):
         initiator_name = connector['initiator']
         target_name = volume['provider_location']
 
-        self.utils.associate_initiator_with_iscsi_target(target_name,
-                                                         initiator_name)
+        self.utils.associate_initiator_with_iscsi_target(initiator_name,
+                                                         target_name)
 
         properties = self.utils.get_host_information(volume, target_name)
 
@@ -170,7 +168,8 @@ class WindowsDriver(driver.ISCSIDriver):
         """Fetch the image from image_service and write it to the volume."""
         # Convert to VHD and file back to VHD
         image_utils.fetch_to_vhd(context, image_service, image_id,
-                                 self.local_path(volume))
+                                 self.local_path(volume),
+                                 self.configuration.volume_dd_blocksize)
 
     def copy_volume_to_image(self, context, volume, image_service, image_meta):
         """Copy the volume to the specified image."""
