@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 Pedro Navarro Perez
 # All Rights Reserved.
 #
@@ -92,6 +90,8 @@ class WindowsUtils(object):
             properties['auth_username'] = auth_username
             properties['auth_password'] = auth_secret
 
+        return properties
+
     def associate_initiator_with_iscsi_target(self, initiator_name,
                                               target_name):
         """Sets information used by the iSCSI target entry."""
@@ -131,7 +131,7 @@ class WindowsUtils(object):
             raise exception.VolumeBackendAPIException(data=err_msg)
 
     def create_volume(self, vhd_path, vol_name, vol_size):
-        """Creates a volume"""
+        """Creates a volume."""
         try:
             cl = self._conn_wmi.__getattr__("WT_Disk")
             cl.NewWTDisk(DevicePath=vhd_path,
@@ -235,7 +235,12 @@ class WindowsUtils(object):
     def remove_iscsi_target(self, target_name):
         """Removes ISCSI target."""
         try:
-            wt_host = self._conn_wmi.WT_Host(HostName=target_name)[0]
+            host = self._conn_wmi.WT_Host(HostName=target_name)
+            if not host:
+                LOG.debug(_('Skipping removing target %s as it does not '
+                            'exist.') % target_name)
+                return
+            wt_host = host[0]
             wt_host.RemoveAllWTDisks()
             wt_host.Delete_()
         except wmi.x_wmi as exc:
@@ -247,7 +252,7 @@ class WindowsUtils(object):
             raise exception.VolumeBackendAPIException(data=err_msg)
 
     def add_disk_to_target(self, vol_name, target_name):
-        """Adds the disk to the target"""
+        """Adds the disk to the target."""
         try:
             q = self._conn_wmi.WT_Disk(Description=vol_name)
             wt_disk = q[0]
@@ -290,7 +295,7 @@ class WindowsUtils(object):
             wt_disk.Extend(additional_size)
         except wmi.x_wmi as exc:
             err_msg = (_(
-                'extend: error when extending the volumne: %(vol_name)s '
+                'extend: error when extending the volume: %(vol_name)s '
                 '.WMI exception: %(wmi_exc)s') % {'vol_name': vol_name,
                                                   'wmi_exc': exc})
             LOG.error(err_msg)
