@@ -12,14 +12,12 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-from oslo_log import log as logging
-from oslo_utils import excutils
 import taskflow.engines
 from taskflow.patterns import linear_flow
 
 from cinder import exception
 from cinder import flow_utils
-from cinder.i18n import _
+from cinder.openstack.common import log as logging
 from cinder.volume.flows.api import create_volume as create_api
 from cinder.volume.flows import common as flow_common
 from cinder.volume.flows.manager import create_volume as create_mgr
@@ -40,38 +38,27 @@ class PrepareForQuotaReservationTask(flow_utils.CinderTask):
         self.db = db
         self.driver = driver
 
-    def execute(self, context, volume, manage_existing_ref):
-        driver_name = self.driver.__class__.__name__
+    def execute(self, context, volume_ref, manage_existing_ref):
+        volume_id = volume_ref['id']
         if not self.driver.initialized:
-            LOG.error("Unable to manage existing volume. "
-                      "Volume driver %s not initialized.", driver_name)
-            flow_common.error_out(volume, _("Volume driver %s not "
-                                            "initialized.") % driver_name,
-                                  status='error_managing')
+            driver_name = self.driver.__class__.__name__
+            LOG.error(_("Unable to manage existing volume. "
+                        "Volume driver %s not initialized.") % driver_name)
+            flow_common.error_out_volume(context, self.db, volume_id,
+                                         reason=_("Volume driver %s "
+                                                  "not initialized.") %
+                                         driver_name)
             raise exception.DriverNotInitialized()
 
-        size = 0
-        try:
-            size = self.driver.manage_existing_get_size(volume,
-                                                        manage_existing_ref)
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                reason = _("Volume driver %s get exception.") % driver_name
-                flow_common.error_out(volume, reason,
-                                      status='error_managing')
+        size = self.driver.manage_existing_get_size(volume_ref,
+                                                    manage_existing_ref)
 
         return {'size': size,
-                'volume_type_id': volume.volume_type_id,
-                'volume_properties': volume,
-                'volume_spec': {'status': volume.status,
-                                'volume_name': volume.name,
-                                'volume_id': volume.id}}
-
-    def revert(self, context, result, flow_failures, volume, **kwargs):
-        reason = _('Volume manage failed.')
-        flow_common.error_out(volume, reason=reason,
-                              status='error_managing')
-        LOG.error("Volume %s: manage failed.", volume.id)
+                'volume_type_id': volume_ref['volume_type_id'],
+                'volume_properties': volume_ref,
+                'volume_spec': {'status': volume_ref['status'],
+                                'volume_name': volume_ref['name'],
+                                'volume_id': volume_ref['id']}}
 
 
 class ManageExistingTask(flow_utils.CinderTask):
@@ -84,27 +71,26 @@ class ManageExistingTask(flow_utils.CinderTask):
         self.db = db
         self.driver = driver
 
-    def execute(self, context, volume, manage_existing_ref, size):
-        model_update = self.driver.manage_existing(volume,
+    def execute(self, context, volume_ref, manage_existing_ref, size):
+        model_update = self.driver.manage_existing(volume_ref,
                                                    manage_existing_ref)
-
         if not model_update:
             model_update = {}
         model_update.update({'size': size})
         try:
-            volume.update(model_update)
-            volume.save()
+            volume_ref = self.db.volume_update(context, volume_ref['id'],
+                                               model_update)
         except exception.CinderException:
-            LOG.exception("Failed updating model of volume %(volume_id)s"
-                          " with creation provided model %(model)s",
-                          {'volume_id': volume.id,
+            LOG.exception(_("Failed updating model of volume %(volume_id)s"
+                            " with creation provided model %(model)s") %
+                          {'volume_id': volume_ref['id'],
                            'model': model_update})
             raise
 
-        return {'volume': volume}
+        return {'volume': volume_ref}
 
 
-def get_flow(context, db, driver, host, volume, ref):
+def get_flow(context, db, driver, host, volume_id, ref):
     """Constructs and returns the manager entrypoint flow."""
 
     flow_name = ACTION.replace(":", "_") + "_manager"
@@ -115,20 +101,18 @@ def get_flow(context, db, driver, host, volume, ref):
     # determined.
     create_what = {
         'context': context,
-        'volume': volume,
-        'manage_existing_ref': ref,
-        'group_snapshot': None,
-        'optional_args': {'is_quota_committed': False, 'update_size': True}
+        'volume_id': volume_id,
+        'manage_existing_ref': ref
     }
 
-    volume_flow.add(create_mgr.NotifyVolumeActionTask(db,
+    volume_flow.add(create_mgr.ExtractVolumeRefTask(db, host),
+                    create_mgr.NotifyVolumeActionTask(db,
                                                       "manage_existing.start"),
                     PrepareForQuotaReservationTask(db, driver),
                     create_api.QuotaReserveTask(),
                     ManageExistingTask(db, driver),
                     create_api.QuotaCommitTask(),
-                    create_mgr.CreateVolumeOnFinishTask(db,
-                                                        "manage_existing.end"))
+                    create_mgr.CreateVolumeOnFinishTask(db, "create.end"))
 
     # Now load (but do not run) the flow using the provided initial data.
     return taskflow.engines.load(volume_flow, store=create_what)
