@@ -1,6 +1,8 @@
-#    (c) Copyright 2019 Brocade, a Broadcom Company
+#    (c) Copyright 2014 Brocade Communications Systems Inc.
 #    All Rights Reserved.
 #
+#    Copyright 2014 OpenStack Foundation
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -23,16 +25,13 @@ import random
 import re
 
 from eventlet import greenthread
-from oslo_concurrency import processutils
-from oslo_log import log as logging
-from oslo_utils import excutils
-import six
 
 from cinder import exception
-from cinder.i18n import _
-from cinder import ssh_utils
+from cinder.openstack.common import excutils
+from cinder.openstack.common import log as logging
+from cinder.openstack.common import processutils
 from cinder import utils
-import cinder.zonemanager.drivers.brocade.fc_zone_constants as zone_constant
+import cinder.zonemanager.drivers.brocade.fc_zone_constants as ZoneConstant
 
 LOG = logging.getLogger(__name__)
 
@@ -42,17 +41,14 @@ class BrcdFCZoneClientCLI(object):
     switch_port = '22'
     switch_user = 'admin'
     switch_pwd = 'none'
-    switch_key = 'none'
-    patrn = re.compile(r'[;\s]+')
+    patrn = re.compile('[;\s]+')
 
-    def __init__(self, ipaddress, username,
-                 password, port, key):
-        """Initializing the client."""
+    def __init__(self, ipaddress, username, password, port):
+        """initializing the client."""
         self.switch_ip = ipaddress
         self.switch_port = port
         self.switch_user = username
         self.switch_pwd = password
-        self.switch_key = key
         self.sshpool = None
 
     def get_active_zone_set(self):
@@ -62,16 +58,13 @@ class BrcdFCZoneClientCLI(object):
         are active then it will return empty map.
 
         :returns: Map -- active zone set map in the following format
-
-        .. code-block:: python
-
-            {
-                'zones':
-                    {'openstack50060b0000c26604201900051ee8e329':
-                        ['50060b0000c26604', '201900051ee8e329']
-                    },
-                'active_zone_config': 'OpenStack_Cfg'
-            }
+        {
+            'zones':
+                {'openstack50060b0000c26604201900051ee8e329':
+                    ['50060b0000c26604', '201900051ee8e329']
+                },
+            'active_zone_config': 'OpenStack_Cfg'
+        }
         """
         zone_set = {}
         zone = {}
@@ -81,11 +74,11 @@ class BrcdFCZoneClientCLI(object):
         zone_set_name = None
         try:
             switch_data = self._get_switch_info(
-                [zone_constant.GET_ACTIVE_ZONE_CFG])
+                [ZoneConstant.GET_ACTIVE_ZONE_CFG])
         except exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
-                LOG.error("Failed getting active zone set "
-                          "from fabric %s", self.switch_ip)
+                LOG.error(_("Failed getting active zone set "
+                            "from fabric %s"), self.switch_ip)
         try:
             for line in switch_data:
                 line_split = re.split('\\t', line)
@@ -95,7 +88,7 @@ class BrcdFCZoneClientCLI(object):
                     line_split = [x.replace(
                         ' ',
                         '') for x in line_split]
-                    if zone_constant.CFG_ZONESET in line_split:
+                    if ZoneConstant.CFG_ZONESET in line_split:
                         zone_set_name = line_split[1]
                         continue
                     if line_split[1]:
@@ -103,192 +96,116 @@ class BrcdFCZoneClientCLI(object):
                         zone[zone_name] = list()
                     if line_split[2]:
                         zone_member = line_split[2]
+                    if zone_member:
                         zone_member_list = zone.get(zone_name)
                         zone_member_list.append(zone_member)
-            zone_set[zone_constant.CFG_ZONES] = zone
-            zone_set[zone_constant.ACTIVE_ZONE_CONFIG] = zone_set_name
-        except Exception:
-            # In case of parsing error here, it should be malformed cli output.
+            zone_set[ZoneConstant.CFG_ZONES] = zone
+            zone_set[ZoneConstant.ACTIVE_ZONE_CONFIG] = zone_set_name
+        except Exception as ex:
+            # Incase of parsing error here, it should be malformed cli output.
             msg = _("Malformed zone configuration: (switch=%(switch)s "
                     "zone_config=%(zone_config)s)."
                     ) % {'switch': self.switch_ip,
                          'zone_config': switch_data}
-            LOG.exception(msg)
+            LOG.error(msg)
+            LOG.exception(ex)
             raise exception.FCZoneDriverException(reason=msg)
         switch_data = None
         return zone_set
 
-    def add_zones(self, zones, activate, active_zone_set=None):
+    def add_zones(self, zones, activate):
         """Add zone configuration.
 
         This method will add the zone configuration passed by user.
-
-        :param zones: zone names mapped to members. Zone members
-                      are colon separated but case-insensitive
-
-        .. code-block:: python
-
-            {   zonename1:[zonememeber1, zonemember2,...],
+            input params:
+            zones - zone names mapped to members.
+            zone members are colon separated but case-insensitive
+            {   zonename1:[zonememeber1,zonemember2,...],
                 zonename2:[zonemember1, zonemember2,...]...}
-
-            e.g:
-
-            {
-                'openstack50060b0000c26604201900051ee8e329':
-                        ['50:06:0b:00:00:c2:66:04',
-                         '20:19:00:05:1e:e8:e3:29']
-            }
-
-        :param activate: True/False
-        :param active_zone_set: active zone set dict retrieved from
-                                get_active_zone_set method
-
+            e.g: {'openstack50060b0000c26604201900051ee8e329':
+                    ['50:06:0b:00:00:c2:66:04', '20:19:00:05:1e:e8:e3:29']
+                }
+            activate - True/False
         """
-        LOG.debug("Add Zones - Zones passed: %s", zones)
+        LOG.debug(_("Add Zones - Zones passed: %s"), zones)
         cfg_name = None
         iterator_count = 0
         zone_with_sep = ''
-        if not active_zone_set:
-            active_zone_set = self.get_active_zone_set()
-            LOG.debug("Active zone set: %s", active_zone_set)
-        zone_list = active_zone_set[zone_constant.CFG_ZONES]
-        LOG.debug("zone list: %s", zone_list)
+        active_zone_set = self.get_active_zone_set()
+        LOG.debug(_("Active zone set:%s"), active_zone_set)
+        zone_list = active_zone_set[ZoneConstant.CFG_ZONES]
+        LOG.debug(_("zone list:%s"), zone_list)
         for zone in zones.keys():
+            # if zone exists, its an update. Delete & insert
+            # TODO(skolathur): This can be optimized to an update call later
+            LOG.debug("Update call")
+            if (zone in zone_list):
+                try:
+                    self.delete_zones(zone, activate)
+                except exception.BrocadeZoningCliException:
+                    with excutils.save_and_reraise_exception():
+                        LOG.error(_("Deleting zone failed %s"), zone)
+                LOG.debug(_("Deleted Zone before insert : %s"), zone)
             zone_members_with_sep = ';'.join(str(member) for
                                              member in zones[zone])
-            LOG.debug("Forming command for create zone")
+            LOG.debug(_("Forming command for add zone"))
             cmd = 'zonecreate "%(zone)s", "%(zone_members_with_sep)s"' % {
                 'zone': zone,
                 'zone_members_with_sep': zone_members_with_sep}
-            LOG.debug("Creating zone, cmd to run %s", cmd)
+            LOG.debug(_("Adding zone, cmd to run %s"), cmd)
             self.apply_zone_change(cmd.split())
+            LOG.debug(_("Created zones on the switch"))
             if(iterator_count > 0):
                 zone_with_sep += ';'
             iterator_count += 1
             zone_with_sep += zone
-        if not zone_with_sep:
-            return
         try:
-            # If zone_list exists, there are active zones,
-            # so add new zone to existing active config.
-            # Otherwise, create the zone config.
-            if zone_list:
-                cfg_name = active_zone_set[zone_constant.ACTIVE_ZONE_CONFIG]
-            else:
-                cfg_name = None
+            cfg_name = active_zone_set[ZoneConstant.ACTIVE_ZONE_CONFIG]
             cmd = None
             if not cfg_name:
-                cfg_name = zone_constant.OPENSTACK_CFG_NAME
+                cfg_name = ZoneConstant.OPENSTACK_CFG_NAME
                 cmd = 'cfgcreate "%(zoneset)s", "%(zones)s"' \
                     % {'zoneset': cfg_name, 'zones': zone_with_sep}
             else:
                 cmd = 'cfgadd "%(zoneset)s", "%(zones)s"' \
                     % {'zoneset': cfg_name, 'zones': zone_with_sep}
-            LOG.debug("Zone config cmd to run %s", cmd)
+            LOG.debug(_("New zone %s"), cmd)
             self.apply_zone_change(cmd.split())
+            self._cfg_save()
             if activate:
                 self.activate_zoneset(cfg_name)
-            else:
-                self._cfg_save()
         except Exception as e:
             self._cfg_trans_abort()
             msg = _("Creating and activating zone set failed: "
                     "(Zone set=%(cfg_name)s error=%(err)s)."
-                    ) % {'cfg_name': cfg_name, 'err': six.text_type(e)}
-            LOG.error(msg)
-            raise exception.BrocadeZoningCliException(reason=msg)
-
-    def update_zones(self, zones, activate, operation, active_zone_set=None):
-        """Update the zone configuration.
-
-        This method will update the zone configuration passed by user.
-
-        :param zones: zone names mapped to members. Zone members
-                      are colon separated but case-insensitive
-
-        .. code-block:: python
-
-            {   zonename1:[zonememeber1, zonemember2,...],
-                zonename2:[zonemember1, zonemember2,...]...}
-
-            e.g:
-
-            {
-                'openstack50060b0000c26604201900051ee8e329':
-                        ['50:06:0b:00:00:c2:66:04',
-                         '20:19:00:05:1e:e8:e3:29']
-            }
-
-        :param activate: True/False
-        :param operation: zone add or zone remove
-        :param active_zone_set: active zone set dict retrieved from
-                                get_active_zone_set method
-
-        """
-        LOG.debug("Update Zones - Zones passed: %s", zones)
-        cfg_name = None
-        iterator_count = 0
-        zone_with_sep = ''
-        if not active_zone_set:
-            active_zone_set = self.get_active_zone_set()
-            LOG.debug("Active zone set: %s", active_zone_set)
-        zone_list = active_zone_set[zone_constant.CFG_ZONES]
-        LOG.debug("Active zone list: %s", zone_list)
-        for zone in zones.keys():
-            zone_members_with_sep = ';'.join(str(member) for
-                                             member in zones[zone])
-            cmd = '%(operation)s "%(zone)s", "%(zone_members_with_sep)s"' % {
-                'operation': operation,
-                'zone': zone,
-                'zone_members_with_sep': zone_members_with_sep}
-            LOG.debug("Updating zone, cmd to run %s", cmd)
-            self.apply_zone_change(cmd.split())
-            if(iterator_count > 0):
-                zone_with_sep += ';'
-            iterator_count += 1
-            zone_with_sep += zone
-        if not zone_with_sep:
-            return
-        try:
-            cfg_name = active_zone_set[zone_constant.ACTIVE_ZONE_CONFIG]
-            if activate:
-                self.activate_zoneset(cfg_name)
-            else:
-                self._cfg_save()
-        except Exception as e:
-            self._cfg_trans_abort()
-            msg = _("Activating zone set failed: "
-                    "(Zone set=%(cfg_name)s error=%(err)s)."
-                    ) % {'cfg_name': cfg_name, 'err': six.text_type(e)}
+                    ) % {'cfg_name': cfg_name, 'err': e}
             LOG.error(msg)
             raise exception.BrocadeZoningCliException(reason=msg)
 
     def activate_zoneset(self, cfgname):
         """Method to Activate the zone config. Param cfgname - ZonesetName."""
-        cmd_list = [zone_constant.ACTIVATE_ZONESET, cfgname]
+        cmd_list = [ZoneConstant.ACTIVATE_ZONESET, cfgname]
         return self._ssh_execute(cmd_list, True, 1)
 
     def deactivate_zoneset(self):
         """Method to deActivate the zone config."""
-        return self._ssh_execute([zone_constant.DEACTIVATE_ZONESET], True, 1)
+        return self._ssh_execute([ZoneConstant.DEACTIVATE_ZONESET], True, 1)
 
-    def delete_zones(self, zone_names, activate, active_zone_set=None):
+    def delete_zones(self, zone_names, activate):
         """Delete zones from fabric.
 
         Method to delete the active zone config zones
 
-        :param zone_names: zoneNames separated by semicolon
-        :param activate: True/False
-        :param active_zone_set: the active zone set dict retrieved
-                                from get_active_zone_set method
+        params zone_names: zoneNames separated by semicolon
+        params activate: True/False
         """
         active_zoneset_name = None
+        active_zone_set = None
         zone_list = []
-        if not active_zone_set:
-            active_zone_set = self.get_active_zone_set()
+        active_zone_set = self.get_active_zone_set()
         active_zoneset_name = active_zone_set[
-            zone_constant.ACTIVE_ZONE_CONFIG]
-        zone_list = active_zone_set[zone_constant.CFG_ZONES]
+            ZoneConstant.ACTIVE_ZONE_CONFIG]
+        zone_list = active_zone_set[ZoneConstant.CFG_ZONES]
         zones = self.patrn.split(''.join(zone_names))
         cmd = None
         try:
@@ -296,24 +213,23 @@ class BrcdFCZoneClientCLI(object):
                 self.deactivate_zoneset()
                 cmd = 'cfgdelete "%(active_zoneset_name)s"' \
                     % {'active_zoneset_name': active_zoneset_name}
-                # Active zoneset is being deleted, hence reset activate flag
+                # Active zoneset is being deleted, hence reset is_active
                 activate = False
             else:
                 cmd = 'cfgremove "%(active_zoneset_name)s", "%(zone_names)s"' \
                     % {'active_zoneset_name': active_zoneset_name,
                        'zone_names': zone_names
                        }
-            LOG.debug("Delete zones: Config cmd to run: %s", cmd)
+            LOG.debug(_("Delete zones: Config cmd to run:%s"), cmd)
             self.apply_zone_change(cmd.split())
             for zone in zones:
                 self._zone_delete(zone)
+            self._cfg_save()
             if activate:
                 self.activate_zoneset(active_zoneset_name)
-            else:
-                self._cfg_save()
         except Exception as e:
             msg = _("Deleting zones failed: (command=%(cmd)s error=%(err)s)."
-                    ) % {'cmd': cmd, 'err': six.text_type(e)}
+                    ) % {'cmd': cmd, 'err': e}
             LOG.error(msg)
             self._cfg_trans_abort()
             raise exception.BrocadeZoningCliException(reason=msg)
@@ -327,21 +243,26 @@ class BrcdFCZoneClientCLI(object):
         cli_output = None
         return_list = []
         try:
-            cmd = '%(nsshow)s;%(nscamshow)s' % {
-                'nsshow': zone_constant.NS_SHOW,
-                'nscamshow': zone_constant.NS_CAM_SHOW}
-            cli_output = self._get_switch_info([cmd])
+            cli_output = self._get_switch_info([ZoneConstant.NS_SHOW])
         except exception.BrocadeZoningCliException:
             with excutils.save_and_reraise_exception():
-                LOG.error("Failed collecting nsshow "
-                          "info for fabric %s", self.switch_ip)
+                LOG.error(_("Failed collecting nsshow "
+                            "info for fabric %s"), self.switch_ip)
         if (cli_output):
             return_list = self._parse_ns_output(cli_output)
+        try:
+            cli_output = self._get_switch_info([ZoneConstant.NS_CAM_SHOW])
+        except exception.BrocadeZoningCliException:
+            with excutils.save_and_reraise_exception():
+                LOG.error(_("Failed collecting nscamshow "
+                            "info for fabric %s"), self.switch_ip)
+        if (cli_output):
+            return_list.extend(self._parse_ns_output(cli_output))
         cli_output = None
         return return_list
 
     def _cfg_save(self):
-        self._ssh_execute([zone_constant.CFG_SAVE], True, 1)
+        self._ssh_execute([ZoneConstant.CFG_SAVE], True, 1)
 
     def _zone_delete(self, zone_name):
         cmd = 'zonedelete "%(zone_name)s"' % {'zone_name': zone_name}
@@ -350,17 +271,17 @@ class BrcdFCZoneClientCLI(object):
     def _cfg_trans_abort(self):
         is_abortable = self._is_trans_abortable()
         if(is_abortable):
-            self.apply_zone_change([zone_constant.CFG_ZONE_TRANS_ABORT])
+            self.apply_zone_change([ZoneConstant.CFG_ZONE_TRANS_ABORT])
 
     def _is_trans_abortable(self):
         is_abortable = False
         stdout, stderr = None, None
         stdout, stderr = self._run_ssh(
-            [zone_constant.CFG_SHOW_TRANS], True, 1)
+            [ZoneConstant.CFG_SHOW_TRANS], True, 1)
         output = stdout.splitlines()
         is_abortable = False
         for line in output:
-            if(zone_constant.TRANS_ABORTABLE in line):
+            if(ZoneConstant.TRANS_ABORTABLE in line):
                 is_abortable = True
                 break
         if stderr:
@@ -376,7 +297,7 @@ class BrcdFCZoneClientCLI(object):
         not expected.
         """
         stdout, stderr = None, None
-        LOG.debug("Executing command via ssh: %s", cmd_list)
+        LOG.debug(_("Executing command via ssh: %s"), cmd_list)
         stdout, stderr = self._run_ssh(cmd_list, True, 1)
         # no output expected, so output means there is an error
         if stdout:
@@ -399,17 +320,17 @@ class BrcdFCZoneClientCLI(object):
             if (stdout):
                 for line in stdout:
                     if 'Fabric OS:  v' in line:
-                        LOG.debug("Firmware version string: %s", line)
+                        LOG.debug(_("Firmware version string:%s"), line)
                         ver = line.split('Fabric OS:  v')[1].split('.')
                         if (ver):
                             firmware = int(ver[0] + ver[1])
                 return firmware > 63
             else:
-                LOG.error("No CLI output for firmware version check")
+                LOG.error(_("No CLI output for firmware version check"))
                 return False
         except processutils.ProcessExecutionError as e:
             msg = _("Error while getting data via ssh: (command=%(cmd)s "
-                    "error=%(err)s).") % {'cmd': cmd, 'err': six.text_type(e)}
+                    "error=%(err)s).") % {'cmd': cmd, 'err': e}
             LOG.error(msg)
             raise exception.BrocadeZoningCliException(reason=msg)
 
@@ -423,7 +344,7 @@ class BrcdFCZoneClientCLI(object):
         except processutils.ProcessExecutionError as e:
             msg = _("Error while getting data via ssh: (command=%(cmd)s "
                     "error=%(err)s).") % {'cmd': cmd_list,
-                                          'err': six.text_type(e)}
+                                          'err': e}
             LOG.error(msg)
             raise exception.BrocadeZoningCliException(reason=msg)
 
@@ -455,14 +376,13 @@ class BrcdFCZoneClientCLI(object):
         command = ' '. join(cmd_list)
 
         if not self.sshpool:
-            self.sshpool = ssh_utils.SSHPool(self.switch_ip,
-                                             self.switch_port,
-                                             None,
-                                             self.switch_user,
-                                             self.switch_pwd,
-                                             self.switch_key,
-                                             min_size=1,
-                                             max_size=5)
+            self.sshpool = utils.SSHPool(self.switch_ip,
+                                         self.switch_port,
+                                         None,
+                                         self.switch_user,
+                                         self.switch_pwd,
+                                         min_size=1,
+                                         max_size=5)
         last_exception = None
         try:
             with self.sshpool.item() as ssh:
@@ -474,7 +394,7 @@ class BrcdFCZoneClientCLI(object):
                             command,
                             check_exit_code=check_exit_code)
                     except Exception as e:
-                        LOG.exception('Error executing SSH command.')
+                        LOG.error(e)
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
                 try:
@@ -491,7 +411,7 @@ class BrcdFCZoneClientCLI(object):
                         cmd=command)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error("Error running SSH command: %s", command)
+                LOG.error(_("Error running SSH command: %s") % command)
 
     def _ssh_execute(self, cmd_list, check_exit_code=True, attempts=1):
         """Execute cli with status update.
@@ -502,16 +422,15 @@ class BrcdFCZoneClientCLI(object):
         command = ' '. join(cmd_list)
 
         if not self.sshpool:
-            self.sshpool = ssh_utils.SSHPool(self.switch_ip,
-                                             self.switch_port,
-                                             None,
-                                             self.switch_user,
-                                             self.switch_pwd,
-                                             self.switch_key,
-                                             min_size=1,
-                                             max_size=5)
+            self.sshpool = utils.SSHPool(self.switch_ip,
+                                         self.switch_port,
+                                         None,
+                                         self.switch_user,
+                                         self.switch_pwd,
+                                         min_size=1,
+                                         max_size=5)
         stdin, stdout, stderr = None, None, None
-        LOG.debug("Executing command via ssh: %s", command)
+        LOG.debug(_("Executing command via ssh: %s") % command)
         last_exception = None
         try:
             with self.sshpool.item() as ssh:
@@ -519,13 +438,14 @@ class BrcdFCZoneClientCLI(object):
                     attempts -= 1
                     try:
                         stdin, stdout, stderr = ssh.exec_command(command)
-                        stdin.write("%s\n" % zone_constant.YES)
+                        greenthread.sleep(random.randint(20, 500) / 100.0)
+                        stdin.write("%s\n" % ZoneConstant.YES)
                         channel = stdout.channel
                         exit_status = channel.recv_exit_status()
-                        LOG.debug("Exit Status from ssh: %s", exit_status)
+                        LOG.debug(_("Exit Status from ssh:%s"), exit_status)
                         # exit_status == -1 if no exit code was returned
                         if exit_status != -1:
-                            LOG.debug('Result was %s', exit_status)
+                            LOG.debug(_('Result was %s') % exit_status)
                             if check_exit_code and exit_status != 0:
                                 raise processutils.ProcessExecutionError(
                                     exit_code=exit_status,
@@ -537,11 +457,11 @@ class BrcdFCZoneClientCLI(object):
                         else:
                             return True
                     except Exception as e:
-                        LOG.exception('Error executing SSH command.')
+                        LOG.error(e)
                         last_exception = e
                         greenthread.sleep(random.randint(20, 500) / 100.0)
-                LOG.debug("Handling error case after "
-                          "SSH: %s", last_exception)
+                LOG.debug(_("Handling error case after "
+                            "SSH:%s"), last_exception)
                 try:
                     raise processutils.ProcessExecutionError(
                         exit_code=last_exception.exit_code,
@@ -556,7 +476,7 @@ class BrcdFCZoneClientCLI(object):
                         cmd=command)
         except Exception as e:
             with excutils.save_and_reraise_exception():
-                LOG.error("Error executing command via ssh: %s", e)
+                LOG.error(_("Error executing command via ssh: %s"), e)
         finally:
             if stdin:
                 stdin.flush()
@@ -569,7 +489,7 @@ class BrcdFCZoneClientCLI(object):
     def _execute_shell_cmd(self, cmd):
         """Run command over shell for older firmware versions.
 
-        Invokes shell and issue the command and return the output.
+        We invoke shell and issue the command and return the output.
         This is primarily used for issuing read commands when we are not sure
         if the firmware supports exec_command.
         """
@@ -577,16 +497,15 @@ class BrcdFCZoneClientCLI(object):
         command = ' '. join(cmd)
         stdout, stderr = None, None
         if not self.sshpool:
-            self.sshpool = ssh_utils.SSHPool(self.switch_ip,
-                                             self.switch_port,
-                                             None,
-                                             self.switch_user,
-                                             self.switch_pwd,
-                                             self.switch_key,
-                                             min_size=1,
-                                             max_size=5)
+            self.sshpool = utils.SSHPool(self.switch_ip,
+                                         self.switch_port,
+                                         None,
+                                         self.switch_user,
+                                         self.switch_pwd,
+                                         min_size=1,
+                                         max_size=5)
         with self.sshpool.item() as ssh:
-            LOG.debug('Running cmd (SSH): %s', command)
+            LOG.debug('Running cmd (SSH): %s' % command)
             channel = ssh.invoke_shell()
             stdin_stream = channel.makefile('wb')
             stdout_stream = channel.makefile('rb')
@@ -604,9 +523,10 @@ exit
             exit_status = channel.recv_exit_status()
             # exit_status == -1 if no exit code was returned
             if exit_status != -1:
-                LOG.debug('Result was %s', exit_status)
+                LOG.debug('Result was %s' % exit_status)
                 if exit_status != 0:
-                    LOG.debug("command %s failed", command)
+                    msg = "command %s failed" % command
+                    LOG.debug(msg)
                     raise processutils.ProcessExecutionError(
                         exit_code=exit_status,
                         stdout=stdout,
@@ -614,10 +534,10 @@ exit
                         cmd=command)
             try:
                 channel.close()
-            except Exception:
-                LOG.exception('Error closing channel.')
-            LOG.debug("_execute_cmd: stdout to return: %s", stdout)
-            LOG.debug("_execute_cmd: stderr to return: %s", stderr)
+            except Exception as e:
+                LOG.exception(e)
+            LOG.debug("_execute_cmd: stdout to return:%s" % stdout)
+            LOG.debug("_execute_cmd: stderr to return:%s" % stderr)
         return (stdout, stderr)
 
     def cleanup(self):
